<?php

namespace App\DataTables;

use App\Models\EFSCreateOrder;
use Yajra\DataTables\Html\Button;
use Yajra\DataTables\Services\DataTable;

class CardOrderDataTable extends DataTable
{
    public function dataTable($query)
    {
        return datatables()
            ->eloquent($query)
            ->addColumn('action', function ($cardOrder) {
                return view('cardOrders.action', compact('cardOrder'));
            })
            ->editColumn('CARD_STYLE', function ($row) {
                if ($row->CARD_STYLE == 1) {
                    return 'EFS SECONDARY (Red)/EFS PRIMARY (Silver)';
                } else {
                    return 'SmartFunds';
                }
            })->editColumn('CREATED_DATE', function ($row) {
                return !empty($row->CREATED_DATE) ? date('m/d/Y', strtotime($row->CREATED_DATE)) : '';
            });
    }

    public function query(EFSCreateOrder $model)
    {
        $search = $this->request()->get('search');
        $query = $model->newQuery()
            ->where('USER_ID', getAuthMemberId());
            if (!empty($search)) {
                $query->where(function ($query) use ($search) {
                    $query->where('ORDER_TYPE', 'LIKE', "%$search%")
                        ->orWhere('QTY', 'LIKE', "%$search%")
                        ->orWhere('EMBOSSNAME', 'LIKE', "%$search%")
                        ->orWhere('SHIP_METHOD', 'LIKE', "%$search%");
                });
            }
        $query = $query->orderBy('ORDER_ID', 'DESC');
        return $query;
    }

    public function html()
    {
        return $this->builder()
            ->setTableId('cardorder-table')
            ->columns($this->getColumns())
            ->minifiedAjax()
            ->dom('Bfrtip')
            ->orderBy(1)
            ->setTableAttributes([
                'class' => 'table table-striped table-row-bordered table-row-dashed gy-4 align-middle fw-bold dataTable no-footer',
            ])
            ->headerCallback('
                function(thead, data, start, end, display){
                    $(thead).addClass("thead-bg-color")
                    $(thead).find("th").addClass("fs-7 text-gray-500 text-uppercase");
                }
            ')
            ->createdRow('
                function(row, data, dataIndex) {
                    $(row).addClass("fs-6");
                }
            ')
            ->buttons(
                Button::make('excel'),
                Button::make('csv'),
                Button::make('print'),
                Button::make('reset'),
                Button::make('reload')
            )->drawCallback('
                function() {
                    $(".dataTables_filter").hide();
                }
                ')->initComplete('
                function(settings, json) {
                    $(".dt-buttons").addClass("d-none");
                }
            ');
    }

    protected function getColumns()
    {
        return [
            'ORDER_TYPE' => ['title' => 'ORDER TYPE'],
            'CARD_STYLE' => ['title' => 'CARD STYLE'],
            'STATUS' => ['title' => 'STATUS'],
            'QTY' => ['title' => 'QTY'],
            'EMBOSSNAME' => ['title' => 'EMBOSSNAME'],
            'SHIP_METHOD' => ['title' => 'SHIP METHOD'],
            'CREATED_DATE' => ['title' => 'CREATED DATE'],
            'action' => [
                'title' => 'Action',
                'orderable' => false,
                'searchable' => false,
                'exportable' => false,
                'printable' => false,
                'width' => '8%',
            ],
        ];
    }

    protected function filename()
    {
        return 'CardOrder_' . date('YmdHis');
    }
}
