<?php

namespace App\DataTables;

use App\Models\CheckReportHistory;
use Yajra\DataTables\Html\Button;
use Yajra\DataTables\Services\DataTable;

class CheckReportHistoryDataTable extends DataTable
{
    public function dataTable($query)
    {
        return datatables()
            ->eloquent($query)
            ->editColumn('quarter', function($row){
                if($row->quarter == 1){
                    return 'Q1';
                } else if($row->quarter == 2){
                    return 'Q2';
                } else if($row->quarter == 3){
                    return 'Q3';
                } else if($row->quarter == 4){
                    return 'Q4';
                }
            })
            ->editColumn('created_at', function($row){
                return !empty($row->created_at) ? date('m/d/Y H:i', strtotime($row->created_at)) : '';
            })
            ->editColumn('document', function($row){
                return !empty($row->document) ? '<a href="'.asset('docs/' . $row->document).'" download="'.$row->document.'">Download Report</a>' : '';
            })
            ->addColumn('action', 'checkreporthistory.action')
            ->rawColumns(['document']);
    }

    public function query(CheckReportHistory $model)
    {
        return $model->newQuery()->where('mem_id', getAuthMemberId())->orderBy('id', 'desc')->limit(10);
    }

    public function html()
    {
        return $this->builder()
                    ->setTableId('checkreporthistory-table')
                    ->columns($this->getColumns())
                    ->minifiedAjax()
                    ->dom('Bfrtip')
                    ->orderBy(1)
                    ->setTableAttributes([
                        'class' => 'table table-striped table-row-bordered table-row-dashed gy-4 align-middle fw-bold dataTable',
                    ])
                    ->pageLength(30)
                    ->headerCallback('
                                    function(thead, data, start, end, display){
                                        $(thead).addClass("thead-bg-color")
                                        $(thead).find("th").addClass("fs-7 text-gray-500 text-uppercase");
                                    }
                                ')
                    ->createdRow('
                                    function(row, data, dataIndex) {
                                        $(row).addClass("fs-6");
                                        if (data.ChainID == "2") {
                                            $(row).find("td").css("color", "red");
                                        }
                                    }
                                ')
                    ->buttons(
                        Button::make('excel'),
                        Button::make('csv'),
                        Button::make('print'),
                        Button::make('reset'),
                        Button::make('reload')
                    )
                    ->drawCallback('
                                    function() {
                                        $(".dataTables_filter").hide();
                                    }
                                ')
                    ->initComplete('
                                    function(settings, json) {
                                        $(".dt-buttons").addClass("d-none");
                                    }
                                ');
    }

    protected function getColumns()
    {
        return [
            'quarter' => ['title' => 'QUARTER'],
            'year' => ['title' => 'YEAR'],
            'created_at' => ['title' => 'CREATED_AT'],
            'document' => ['title' => 'Report'],
            'action' => [
                'title' => 'Action',
                'orderable' => false,
                'searchable' => false,
                'exportable' => false,
                'printable' => false,
                'width' => '8%',
            ],
        ];
    }

    protected function filename()
    {
        return 'CheckReportHistory_' . date('YmdHis');
    }
}
