<?php

namespace App\DataTables;

use App\Models\Disputes;
use Yajra\DataTables\Html\Button;
use Yajra\DataTables\Services\DataTable;

class DisputesDataTable extends DataTable
{
    public function dataTable($query)
    {
        return datatables()
            ->eloquent($query)
            ->addColumn('action', function ($disputes) {
                return view('Disputes.action', compact('disputes'));
            })->editColumn('CREATED_DATE', function ($disputes) {
                return !empty($disputes->CREATED_DATE) ? date('m/d/Y', strtotime($disputes->CREATED_DATE)) : '';
            });
    }

    public function query(Disputes $model)
    {
        $query = $model->newQuery()->where('CARRIER_ID', getAuthMemberId());
        $search = $this->request()->get('search');
        $card_status = $this->request()->get('card_status');
        if (!empty($search)) {
            $query->where(function ($query) use ($search) {
                $query->where('TICKET_ID', 'LIKE', "%$search%")
                    ->orWhere('STATUS', 'LIKE', "%$search%")
                    ->orWhere('TITLE', 'LIKE', "%$search%")
                    ->orWhere('DEPT', 'LIKE', "%$search%");
            });
        }
        if (!empty($card_status)) {
            $query->where('STATUS', $card_status);
        }

        $query->orderBy('TICKET_ID', 'desc');

        return $query;
    }

    public function html()
    {
        return $this->builder()
                    ->setTableId('disputes-table')
                    ->columns($this->getColumns())
                    ->minifiedAjax()
                    ->dom('Bfrtip')
                    ->orderBy(1)
                    ->setTableAttributes([
                    'class' => 'table table-striped table-row-bordered table-row-dashed gy-4 align-middle fw-bold dataTable no-footer',
                ])
                ->headerCallback('
                function(thead, data, start, end, display){
                    $(thead).addClass("thead-bg-color")
                    $(thead).find("th").addClass("fs-7 text-gray-500 text-uppercase");
                }
            ')
                ->createdRow('
                function(row, data, dataIndex) {
                    $(row).addClass("fs-6");
                }
            ')
                ->buttons(
                    Button::make('excel'),
                    Button::make('csv'),
                    Button::make('print'),
                    Button::make('reset'),
                    Button::make('reload')
                )->drawCallback('
                function() {
                    $(".dataTables_filter").hide();
                }
                ')->initComplete('
                function(settings, json) {
                    $(".dt-buttons").addClass("d-none");
                }
            ');
    }

    protected function getColumns()
    {
        return [
            'TICKET_ID' => ['title' => 'TICKET ID'],
            'STATUS' => ['title' => 'STATUS'],
            'TITLE' => ['title' => 'TITLE'],
            'DEPT' => ['title' => 'CATEGORY'],
            'CREATED_BY' => ['title' => 'CREATED BY'],
            'CREATED_DATE' => ['title' => 'CREATED DATE'],
             'action' => [
                'title' => 'Action',
                'orderable' => false,
                'searchable' => false,
                'exportable' => false,
                'printable' => false,
                'width' => '8%',
            ],
        ];
    }

    protected function filename()
    {
        return 'Disputes_' . date('YmdHis');
    }
}
