<?php

namespace App\DataTables;

use App\Models\Driver;
use Yajra\DataTables\Html\Button;
use Yajra\DataTables\Services\DataTable;

class DriverDataTable extends DataTable
{
    public function dataTable($query)
    {
        return datatables()
            ->eloquent($query)
            ->editColumn('NAME', function ($row) {
                $age = '';
                if(!empty($row->BIRTH_DATE)){
                    $BIRTH_DATE = strtotime($row->BIRTH_DATE);
                    $current_date = time();
                    $age = date('Y', $current_date) - date('Y', $BIRTH_DATE);
                    if (date('md', $current_date) < date('md', $BIRTH_DATE)) {
                        $age--;
                    }
                    $age = '<br>Age : '.$age;
                }

                return '<a href="'.route('assets.drivers.edit', ['driverId' => $row->{'DRIVER_ID'}]).'">' . $row->{'FIRST_NAME'} . ' '. $row->{'LAST_NAME'}.'</a>
                        '.$age;
            })
            ->editColumn('START_DATE', function ($row) {
                if(!empty ($row->START_DATE)){
                    $start_date = $row->START_DATE;
                    $current_date = date("Y-m-d");

                    $start_timestamp = strtotime($start_date);
                    $current_timestamp = strtotime($current_date);

                    $difference_seconds = $current_timestamp - $start_timestamp;

                    return date('m/d/Y', strtotime($start_date)) .'<br>'. floor($difference_seconds / (60 * 60 * 24)) .' Days';
                }else{
                    return '-';
                }
            })
            ->editColumn('TERMINATION_DATE', function ($row) {
                return !empty ($row->TERMINATION_DATE) ? date('m/d/Y', strtotime($row->TERMINATION_DATE)) : '';
            })
            ->addColumn('action', function ($row) {
                return view('assets.drivers.action', compact('row'));
            })
            ->rawColumns(['NAME', 'START_DATE']);
    }

    public function query(Driver $model)
    {
        $search = $this->request()->get('search');
        $status = $this->request()->get('status');
        $startDate = $this->request()->get('start_date');
        $endDate = $this->request()->get('end_date');
        $query = $model->newQuery()->where('mem_id', getAuthMemberId());

        if (!empty($search)) {
            $query->where(function ($query) use ($search) {
                $query->where('NAME', 'LIKE', "%$search%")
                    ->orWhere('PHONE', 'LIKE', "%$search%")
                    ->orWhere('CODE', 'LIKE', "%$search%")
                    ->orWhere('EMPLOYEE_CODE', 'LIKE', "%$search%")
                    ->orWhere('STATUS', 'LIKE', "%$search%")
                    ->orWhere('EMAIL', 'LIKE', "%$search%");
            });
        }
        if (!empty($status)) {
            $query->where('STATUS', $status);
        }
        if (!empty($startDate) && !empty($endDate)) {
            $query->whereBetween('START_DATE', [$startDate, $endDate]);
        }

        return $query;
    }

    public function html()
    {
        return $this->builder()
            ->setTableId('driver-table')
            ->columns($this->getColumns())
            ->minifiedAjax()
            ->dom('Bfrtip')
            ->orderBy(1)
            ->setTableAttributes([
                'class' => 'table table-striped table-row-bordered table-row-dashed gy-4 align-middle fw-bold dataTable no-footer nowrap',
            ])
            ->pageLength(30)
            ->headerCallback('
                    function(thead, data, start, end, display){
                        $(thead).addClass("thead-bg-color")
                        $(thead).find("th").addClass("fs-7 text-gray-500 text-uppercase");
                    }
                ')
            ->createdRow('
                    function(row, data, dataIndex) {
                        $(row).addClass("fs-6");
                    }
                ')
            ->buttons(
                Button::make('excel'),
                Button::make('csv'),
                Button::make('print'),
                Button::make('reset'),
                Button::make('reload')
            )->drawCallback('
                    function() {
                        $(".dataTables_filter").hide();
                    }
                    ')->initComplete('
                    function(settings, json) {
                        $(".dt-buttons").addClass("d-none");
                    }
                ');
    }

    protected function getColumns()
    {
        return [
            'NAME' => ['title' => 'Name'],
            'EMAIL' => ['title' => 'Email'],
            'PHONE' => ['title' => 'Phone'],
            'LICENCE_NUMBER' => ['title' => 'License #'],
            'START_DATE' => ['title' => 'Hired'],
            'STATUS' => ['title' => 'Status'],
            'action' => [
                'title' => 'Action',
                'orderable' => false,
                'searchable' => false,
                'exportable' => false,
                'printable' => false,
                'width' => '8%',
            ],
        ];
    }

    protected function filename()
    {
        return 'Driver_' . date('YmdHis');
    }
}
