<?php

namespace App\DataTables;

use App\Models\FuelCard;
use Yajra\DataTables\Html\Button;
use Yajra\DataTables\Services\DataTable;

class FuelCardDataTable extends DataTable
{
    public function dataTable($query)
    {
        return datatables()
            ->eloquent($query)
            ->addColumn('action', function ($fuelCard) {
                return view('fuelCard.action', compact('fuelCard'));
            })->editColumn('CARD_NO', function ($fuelCard) {
                return '<a href="'.route('fuelCards.edit', $fuelCard->ID).'">' . $fuelCard->{'CARD_NO'} . '</a>';
            })
            ->rawColumns(['CARD_NO']);
    }

    public function query(FuelCard $model)
    {
        $query = $model->newQuery()->where('FC_VENDOR_ID', getAuthMemberId());
        $search = $this->request()->get('search');
        $card_status = $this->request()->get('card_status');
        $punit_id = $this->request()->get('punit_id');
        if (!empty($search)) {
            $query->where(function ($query) use ($search) {
                $query->where('CARD_NO', 'LIKE', "%$search%")
                    ->orWhere('CARD_STATUS', 'LIKE', "%$search%")
                    ->orWhere('CARD_HOLDER', 'LIKE', "%$search%")
                    ->orWhere('PUNIT_ID', 'LIKE', "%$search%")
                    ->orWhere('DRIVER_ID', 'LIKE', "%$search%");
            });
        }
        if (!empty($card_status)) {
            $query->where('CARD_STATUS', $card_status);
        }
        if (!empty($punit_id)) {
            $query->where('PUNIT_ID', $punit_id);
        }

        return $query;
    }

    public function html()
    {
        return $this->builder()
            ->setTableId('fuelcard-table')
            ->columns($this->getColumns())
            ->minifiedAjax()
            ->dom('Bfrtip')
            ->orderBy(1)
            ->setTableAttributes([
                'class' => 'table table-striped table-row-bordered table-row-dashed gy-4 align-middle fw-bold dataTable no-footer',
            ])
            ->pageLength(30)
            ->headerCallback('
            function(thead, data, start, end, display){
                $(thead).addClass("thead-bg-color")
                $(thead).find("th").addClass("fs-7 text-gray-500 text-uppercase");
            }
        ')
            ->createdRow('
            function(row, data, dataIndex) {
                $(row).addClass("fs-6");
            }
        ')
            ->buttons(
                Button::make('excel'),
                Button::make('csv'),
                Button::make('print'),
                Button::make('reset'),
                Button::make('reload')
            )->drawCallback('
            function() {
                $(".dataTables_filter").hide();
            }
            ')->initComplete('
            function(settings, json) {
                $(".dt-buttons").addClass("d-none");
            }
        ');
    }

    protected function getColumns()
    {
        return [
            'CARD_NO' => ['title' => 'Card No'],
            'CARD_STATUS' => ['title' => 'Card Status'],
            'CARD_HOLDER' => ['title' => 'Driver'],
            'PUNIT_ID' => ['title' => 'Unit #'],
            'DRIVER_ID' => ['title' => 'Driver ID'],
            'action' => [
                'title' => 'Action',
                'orderable' => false,
                'searchable' => false,
                'exportable' => false,
                'printable' => false,
                'width' => '8%',
            ],
        ];
    }

    protected function filename()
    {
        return 'FuelCard_' . date('YmdHis');
    }
}
