<?php

namespace App\DataTables;

use Carbon\Carbon;
use DataTables;
use Yajra\DataTables\Html\Button;
use Illuminate\Support\Facades\DB;
use Yajra\DataTables\Services\DataTable;

class GallonsByStoreDataTable extends DataTable
{
    public function dataTable()
    {
        $start_date = '';
        $end_date = '';
        if (!empty($this->request()->get('start_date'))) {
            $start_date = $this->request()->get('start_date');
        } else {
            $start_date = Carbon::now()->startOfMonth();
        }
        if (!empty($this->request()->get('end_date'))) {
            $end_date = $this->request()->get('end_date');
        } else {
            $end_date = Carbon::now()->endOfMonth();
        }

        $query = "SELECT count(*) as totalCount, `LOCATION`, sum(`QTY`) as QTY, avg(`RATE`) as RATE, avg(`REBATE`) as REBATE, `StoreID` FROM `FUEL_PROCESS` WHERE `SUBFLEET` = '" . getAuthMemberId() . "' AND `PRODUCT_CODE` = 'ULSD' AND `TRANS_DATE` BETWEEN '" . $start_date . "' AND '" . $end_date . "' group by `StoreID`
        ORDER BY `QTY` DESC";

        return DataTables::of(DB::select(DB::raw($query)))
            ->editColumn('totalCount', function ($row) {
                return $row->totalCount;
            })
            ->editColumn('LOCATION', function ($row) {
                return $row->LOCATION;
            })
            ->editColumn('QTY', function ($row) {
                return number_format($row->{'QTY'}, 2);
            })
            ->editColumn('RATE', function ($row) {
                return number_format($row->{'RATE'}, 3);
            })
            ->editColumn('REBATE', function ($row) {
                return number_format($row->{'REBATE'}, 3);
            })
            ->addColumn('action', 'carrierpricing.action');
    }

    public function query()
    {
        return $this->dataTable();
    }

    public function html()
    {
        return $this->builder()
            ->setTableId('gallonsbystore-table')
            ->columns($this->getColumns())
            ->minifiedAjax()
            ->dom('Bfrtip')
            ->orderBy(1, 'desc')
            ->pageLength(25)
            ->setTableAttributes([
                'class' => 'table table-striped table-row-bordered table-row-dashed gy-4 align-middle fw-bold dataTable',
            ])
            ->headerCallback('
            function(thead, data, start, end, display){
                $(thead).addClass("thead-bg-color")
                $(thead).find("th").addClass("fs-7 text-gray-500 text-uppercase");
            }
        ')
            ->footerCallback('
            function(row, data, start, end, display) {
                var api = this.api();
                var intVal = function(i){
                    return typeof i === "string" ? i.replace(/[\$,]/g, "") * 1 : typeof i === "number" ? i : 0;
                };

                var total = api.column(0).data().reduce(function(a, b){
                    return intVal(a) + intVal(b);
                }, 0);

                $("#displayGallonsByStoreQTY").html(total.toLocaleString(undefined, {minimumFractionDigits: 3}));
            }
        ')
            ->createdRow('
            function(row, data, dataIndex) {
                $(row).addClass("fs-6");
            }
        ')
            ->buttons(
                Button::make('excel'),
                Button::make('csv'),
                Button::make('print'),
                Button::make('reset'),
                Button::make('reload')
            )
            ->drawCallback('
            function() {
                $(".dataTables_filter").hide();
            }
        ')
            ->initComplete('
            function(settings, json) {
                $(".dt-buttons").addClass("d-none");
            }
        ');
    }

    protected function getColumns()
    {
        return [            
            'totalCount' => ['title' => 'COUNT', 'width' => '40%'],
            'LOCATION' => ['title' => 'LOCATION', 'width' => '40%'],
            'QTY' => ['title' => 'QTY'],
            'RATE' => ['title' => 'AVG RATE'],
            'REBATE' => ['title' => 'AVG REBATE'],
        ];
    }

    protected function filename()
    {
        return 'GallonsByStore_' . date('YmdHis');
    }
}
