<?php

namespace App\DataTables;

use DataTables;
use Carbon\Carbon;
use App\Models\TransactionReport;
use Yajra\DataTables\Html\Button;
use Illuminate\Support\Facades\DB;
use Yajra\DataTables\Services\DataTable;

class GenerateReportDataTable extends DataTable
{
    public function dataTable()
    {
        if (!empty($this->request()->get('QUARTER'))) {
            $quarter = $this->request()->get('QUARTER');
        } else {
            $currentDate = Carbon::now();
            $quarter = ceil($currentDate->month / 3);
        }
        $year = !empty($this->request()->get('YEAR')) ? $this->request()->get('YEAR') : date('Y');
        $startMonth = ($quarter - 1) * 3 + 1;
        $startDate = Carbon::createFromDate($year, $startMonth, 1)->startOfMonth();
        $endDate = $startDate->copy()->addMonths(3)->subDay()->endOfMonth();
        $startDate = $startDate->toDateString();
        $endDate = $endDate->toDateString();
        $memberId = getAuthMemberId();

        $gallonsSql = "(SELECT `Location State/Province` AS STATE,
                    SUM(`Quantity`) AS GALLONS,
                    NULL AS miles,
                    NULL AS totalMiles,
                    NULL AS QUARTER,
                    NULL AS YEAR
            FROM DailyTrans
            WHERE `Sub Fleet` = '" . $memberId . "' AND `Product Code` != 'DEFD'
              AND `Transaction Date` BETWEEN '" . $startDate . "' AND '" . $endDate . "'
            GROUP BY `Location State/Province`)

           UNION

           (SELECT STATE,
                   NULL AS GALLONS,
                   MILES AS miles,
                   SUM(MILES) AS totalMiles,
                   NULL AS QUARTER,
                   NULL AS YEAR
            FROM IFTA_MILES
            WHERE CARRIER_SUFBLEET = '" . $memberId . "'
              AND QUARTER = '" . $quarter . "'
              AND YEAR = '" . $year . "'
            GROUP BY STATE)";

        $query = "SELECT STATE, NULL AS QUARTER, NULL AS YEAR,
                    SUM(GALLONS) AS GALLONS,
                    SUM(miles) AS MILES,
                    SUM(totalMiles) AS totalMiles
                FROM ($gallonsSql) AS subquery
                GROUP BY STATE";
        return DataTables::of(DB::select(DB::raw($query)))
        ->editColumn('GALLONS', function ($row) {
            return number_format($row->{'GALLONS'}, 2);
        })
        ->editColumn('MILES', function ($row) {
            return number_format($row->{'MILES'}, 2);
        })
        ->editColumn('QUARTER', function ($row) use ($quarter) {
            return 'Q' . $quarter;
        })
        ->editColumn('YEAR', function ($row) use ($year) {
            return $year;
        })
        ->editColumn('totalMiles', function ($row) {
            return number_format($row->{'totalMiles'}, 2);
        });
    }

    public function query(TransactionReport $model)
    {
        return $this->dataTable();
    }

    public function html()
    {
        return $this->builder()
            ->setTableId('generatereport-table')
            ->columns($this->getColumns())
            ->minifiedAjax()
            ->dom('Bfrtip')
            ->orderBy(1)
            ->setTableAttributes([
                'class' => 'table table-striped table-row-bordered table-row-dashed gy-4 align-middle fw-bold dataTable',
            ])
            ->pageLength(30)
            ->headerCallback('
                            function(thead, data, start, end, display){
                                $(thead).addClass("thead-bg-color")
                                $(thead).find("th").addClass("fs-7 text-gray-500 text-uppercase");
                            }
                        ')
            ->footerCallback('
                            function(row, data, start, end, display) {
                                var api = this.api();
                                var intVal = function(i){
                                    return typeof i === "string" ? i.replace(/[\$,]/g, "") * 1 : typeof i === "number" ? i : 0;
                                };

                                var gallonTotal = api.column(1).data().reduce(function(a, b){
                                    return intVal(a) + intVal(b);
                                }, 0);

                                var milesTotal = api.column(2).data().reduce(function(a, b){
                                    return intVal(a) + intVal(b);
                                }, 0);

                                var avgmpg = gallonTotal !== 0 ? milesTotal / gallonTotal : 0;

                                $("#displayGallonTotal").html(gallonTotal.toLocaleString(undefined, {minimumFractionDigits: 2}));
                                $("#displayMilesTotal").html(milesTotal.toLocaleString(undefined, {minimumFractionDigits: 2}));
                                $("#displayAvgMpg").html(avgmpg.toLocaleString(undefined, {minimumFractionDigits: 2}));
                            }
                        ')
            ->createdRow('
                            function(row, data, dataIndex) {
                                $(row).addClass("fs-6");
                                if (data.ChainID == "2") {
                                    $(row).find("td").css("color", "red");
                                }
                            }
                        ')
            ->buttons(
                Button::make('excel'),
                Button::make('csv'),
                Button::make('print'),
                Button::make('reset'),
                Button::make('reload')
            )
            ->drawCallback('
                            function() {
                                $(".dataTables_filter").hide();
                            }
                        ')
            ->initComplete('
                            function(settings, json) {
                                $(".dt-buttons").addClass("d-none");
                            }
                        ');
    }

    protected function getColumns()
    {
        return [
            'STATE' => ['title' => 'STATE'],
            'GALLONS' => ['title' => 'GALLONS'],
            'totalMiles' => ['title' => 'MILES'],
            'QUARTER' => ['title' => 'QUARTER'],
            'YEAR' => ['title' => 'YEAR'],
        ];
    }

    protected function filename()
    {
        return 'GenerateReport_' . date('YmdHis');
    }
}
