<?php

namespace App\DataTables;

use App\Models\TransactionReport;
use Yajra\DataTables\Html\Button;
use Illuminate\Support\Facades\DB;
use Yajra\DataTables\Services\DataTable;

class IFTADataTable extends DataTable
{
    public function dataTable($query)
    {
        return datatables()
            ->eloquent($query)
            ->addColumn('Gallons', function ($row) {
                return $row->gallons;
            })
            ->editColumn('Gallons', function ($row) {
                return number_format($row->gallons, 2);
            })
            ->addColumn('QUARTER', function ($row) {
                $quarter = $this->request->get('QUARTER');
                $year = date('Y');
                if (!empty ($this->request->get('YEAR'))) {
                    $year = $this->request->get('YEAR');
                } else if (!empty ($this->request->get('year'))) {
                    $year = $this->request->get('year');
                }
                $quarterValue = '';
                if (empty ($quarter)) {
                    $transactionDate = $row->{'Transaction Date'};
                    if ($transactionDate >= $year . '-01-01' && $transactionDate <= $year . '-03-31') {
                        $quarterValue = 'Q1';
                    } else if ($transactionDate >= $year . '-04-01' && $transactionDate <= $year . '-06-30') {
                        $quarterValue = 'Q2';
                    } else if ($transactionDate >= $year . '-07-01' && $transactionDate <= $year . '-09-30') {
                        $quarterValue = 'Q3';
                    } else if ($transactionDate >= $year . '-10-01' && $transactionDate <= $year . '-12-31') {
                        $quarterValue = 'Q4';
                    }
                } else {
                    if ($quarter == 1) {
                        $quarterValue = 'Q1';
                    } else if ($quarter == 2) {
                        $quarterValue = 'Q2';
                    } else if ($quarter == 3) {
                        $quarterValue = 'Q3';
                    } else if ($quarter == 4) {
                        $quarterValue = 'Q4';
                    }
                }

                return $quarterValue;
            })
            ->addColumn('YEAR', function ($row) {
                $year = date('Y');
                if (!empty ($this->request->get('YEAR'))) {
                    $year = $this->request->get('YEAR');
                } else if (!empty ($this->request->get('year'))) {
                    $year = $this->request->get('year');
                }
                return $year;
            })
            ->addColumn('action', 'ifta.action')
            ->rawColumns(['Gallons', 'action']);
    }

    public function query(TransactionReport $model)
    {
        $quarter = $this->request->get('QUARTER');
        if(empty($quarter)){
            $currentMonth = date('n');
            $quarter = ceil($currentMonth / 3);
        }
        $year = date('Y');
        $unit = $this->request->get('UNIT');
        if (!empty($this->request->get('YEAR'))) {
            $year = $this->request->get('YEAR');
        } else if (!empty($this->request->get('year'))) {
            $year = $this->request->get('year');
        }
        $start_date = '';
        $end_date = '';
        switch ($quarter) {
            case 1:
                $start_date = "{$year}-01-01";
                $end_date = "{$year}-03-31";
                break;
            case 2:
                $start_date = "{$year}-04-01";
                $end_date = "{$year}-06-30";
                break;
            case 3:
                $start_date = "{$year}-07-01";
                $end_date = "{$year}-09-30";
                break;
            case 4:
                $start_date = "{$year}-10-01";
                $end_date = "{$year}-12-31";
                break;
            default:
                $start_date = "{$year}-01-01";
                $end_date = "{$year}-12-31";
                break;
        }
        $query = $model->newQuery()
            ->select(DB::raw('`Location State/Province`, SUM(`Quantity`) as gallons, `Transaction Date`'))
            ->where('Product Code', '!=', 'DEFD')
            ->where('Sub Fleet', getAuthMemberId());

        if(!empty($unit)){
            $query->whereIn('Unit', $unit);
        }

        $query->whereBetween('Transaction Date', [$start_date, $end_date])
            ->groupBy('Location State/Province');

        return $query;
    }

    public function html()
    {
        return $this->builder()
            ->setTableId('ifta-table')
            ->columns($this->getColumns())
            ->minifiedAjax()
            ->dom('Bfrtip')
            ->orderBy(1)
            ->setTableAttributes([
                'class' => 'table table-striped table-row-bordered table-row-dashed gy-4 align-middle fw-bold dataTable',
            ])
            ->pageLength(30)
            ->headerCallback('
            function(thead, data, start, end, display){
                $(thead).addClass("thead-bg-color")
                $(thead).find("th").addClass("fs-7 text-gray-500 text-uppercase");
            }
        ')
            ->footerCallback('
            function(row, data, start, end, display) {
                var api = this.api();
                var intVal = function(i){
                    return typeof i === "string" ? i.replace(/[\$,]/g, "") * 1 : typeof i === "number" ? i : 0;
                };

                var total = api.column(1).data().reduce(function(a, b){
                    return intVal(a) + intVal(b);
                }, 0);

                $("#displayTotal").html(total.toLocaleString(undefined, {minimumFractionDigits: 2}));
            }
        ')
            ->createdRow('
            function(row, data, dataIndex) {
                $(row).addClass("fs-6");
            }
        ')
            ->buttons(
                Button::make('excel'),
                Button::make('csv'),
                Button::make('print'),
                Button::make('reset'),
                Button::make('reload')
            )
            ->drawCallback('
            function() {
                $(".dataTables_filter").hide();
            }
        ')
            ->initComplete('
            function(settings, json) {
                $(".dt-buttons").addClass("d-none");
            }
        ');
    }

    protected function getColumns()
    {
        return [
            'Location State/Province' => ['title' => 'State'],
            'Gallons' => [
                'title' => 'Gallons',
                'orderable' => false,
                'searchable' => true,
                'exportable' => true,
                'printable' => true,
                'width' => '30%',
            ],
            'QUARTER' => [
                'title' => 'QUARTER',
                'orderable' => true,
                'searchable' => true,
                'exportable' => true,
                'printable' => true,
                'width' => '30%',
            ],
            'YEAR' => [
                'title' => 'YEAR',
                'orderable' => true,
                'searchable' => true,
                'exportable' => true,
                'printable' => true,
                'width' => '30%',
            ]
        ];
    }

    protected function filename()
    {
        return 'IFTA_' . date('YmdHis');
    }
}
