<?php

namespace App\DataTables;

use App\Models\Miles;
use Yajra\DataTables\Html\Button;
use Yajra\DataTables\Services\DataTable;

class IFTAMilesDataTable extends DataTable
{
    public function dataTable($query)
    {
        return datatables()
            ->eloquent($query)
            ->editColumn('MILES', function ($row) {
                return number_format($row->MILES, 2);
            })
            ->editColumn('QUARTER', function ($row) {
                if ($row->QUARTER == 1) {
                    return 'Q1';
                } else if ($row->QUARTER == 2) {
                    return 'Q2';
                } else if ($row->QUARTER == 3) {
                    return 'Q3';
                } else if ($row->QUARTER == 4) {
                    return 'Q4';
                }
                return number_format($row->MILES, 2);
            })
            ->addColumn('checkbox', function ($row) {
                return '<input type="checkbox" name="selectedMiles[]" id="delete'.$row->IFTA_ID.'"
                value="'.$row->IFTA_ID.'">';
            })
            ->addColumn('action', function ($row) {
                return view('IFTA.miles.action', compact('row'));
            })->rawColumns(['checkbox']);
    }

    public function query(Miles $model)
    {
        $quarter = $this->request->get('QUARTER');
        $year = $this->request->get('YEAR');
        $state = $this->request->get('STATE');
        $unit = $this->request->get('UNIT');

        $query = $model->newQuery()->where('CARRIER_SUFBLEET', getAuthMemberId())->orderBy('IFTA_ID', 'DESC');
        if (!empty($state)) {
            $query->where('STATE', $state);
        }
        if (!empty($quarter)) {
            $query->where('QUARTER', $quarter);
        }
        if (!empty($year)) {
            $query->where('YEAR', $year);
        }
        if (!empty($unit)) {
            $query->whereIn('UNIT', $unit);
        }

        return $query;
    }

    public function html()
    {
        return $this->builder()
            ->setTableId('iftamiles-table')
            ->columns($this->getColumns())
            ->minifiedAjax()
            ->dom('Bfrtip')
            ->orderBy(1)
            ->setTableAttributes([
                'class' => 'table table-striped table-row-bordered table-row-dashed gy-4 align-middle fw-bold dataTable',
            ])
            ->pageLength(30)
            ->headerCallback('
                    function(thead, data, start, end, display){
                        $(thead).addClass("thead-bg-color")
                        $(thead).find("th").addClass("fs-7 text-gray-500 text-uppercase");
                    }
                ')
            ->footerCallback('
                    function(row, data, start, end, display) {
                        var api = this.api();
                        var intVal = function(i){
                            return typeof i === "string" ? i.replace(/[\$,]/g, "") * 1 : typeof i === "number" ? i : 0;
                        };

                        var total = api.column(2).data().reduce(function(a, b){
                            return intVal(a) + intVal(b);
                        }, 0);

                        $("#displayTotal").html(total.toLocaleString(undefined, {minimumFractionDigits: 2}));
                    }
                ')
            ->createdRow('
                    function(row, data, dataIndex) {
                        $(row).addClass("fs-6");
                        if (data.ChainID == "2") {
                            $(row).find("td").css("color", "red");
                        }
                    }
                ')
            ->buttons(
                Button::make('excel'),
                Button::make('csv'),
                Button::make('print'),
                Button::make('reset'),
                Button::make('reload')
            )
            ->drawCallback('
                    function() {
                        $(".dataTables_filter").hide();
                    }
                ')
            ->initComplete('
                    function(settings, json) {
                        $(".dt-buttons").addClass("d-none");
                    }
                ');
    }

    protected function getColumns()
    {
        return [
            'checkbox' => [
                'title' => '<input type="checkbox" name="deleteAll[]" id="deleteAll" value="deleteAll">',
                'data' => 'checkbox',
                'orderable' => false,
                'searchable' => false,
                'exportable' => false,
                'printable' => false,
                'width' => '8%',
            ],
            'STATE' => ['title' => 'STATE'],
            'MILES' => ['title' => 'MILES'],           
            'QUARTER' => ['title' => 'QUARTER'],
            'YEAR' => ['title' => 'YEAR'],
            'action' => [
                'title' => 'Action',
                'orderable' => false,
                'searchable' => false,
                'exportable' => false,
                'printable' => false,
                'width' => '8%',
            ],
        ];
    }

    protected function filename()
    {
        return 'IFTAMiles_' . date('YmdHis');
    }
}
