<?php

namespace App\DataTables;

use App\Models\TransactionReport;
use Yajra\DataTables\Html\Button;
use Yajra\DataTables\Services\DataTable;
use Carbon\Carbon;
use DB;

class InNetworkGallonsDataTable extends DataTable
{
    public function dataTable($query)
    {
        return datatables()
            ->eloquent($query)
            ->editColumn('Transaction Date', function($row){
                return !empty($row->{'Transaction Date'}) ? date('m/d/Y', strtotime($row->{'Transaction Date'})) : '';
            });
    }

    public function query(TransactionReport $model)
    {
        $start = "";
        $end = "";
        if (!empty($this->request()->get('start_date'))) {
            $start = $this->request()->get('start_date');
        } else {
            $start = Carbon::now()->startOfMonth();
        }
        if (!empty($this->request()->get('end_date'))) {
            $end = $this->request()->get('end_date');
        } else {
            $end = Carbon::now()->endOfMonth();
        }

        $query = $model->newQuery()->select(
            '*',
            DB::raw('IFNULL((SELECT NAME FROM `CARRIER_PROFILE` WHERE `SUBFLEET`=DailyTrans.`Sub Fleet` LIMIT 1), DailyTrans.`Sub Fleet`) as cname')
        )
            ->whereBetween('Transaction Date', [$start, $end])
            ->whereIn('ChainID', ['1'])
            ->whereIn('Product Code', ['ULSD', 'FUEL', 'ULSR'])
            ->where('Sub Fleet', getAuthMemberId())
            ->where(function ($query) {
                $query->where('Location Name', 'like', '%Express%')
                    ->orWhere('Location Name', 'like', '%One9%');
            });

        $search = $this->request()->get('search');
        $card_status = $this->request()->get('card_status');

        if (!empty($search)) {
            $query->where(function ($query) use ($search) {
                $query->where('Card Number', 'LIKE', "%$search%")
                    ->orWhere('Transaction Date', 'LIKE', "%$search%")
                    ->orWhere('Sub Fleet', 'LIKE', "%$search%")
                    ->orWhere('Invoice', 'LIKE', "%$search%")
                    ->orWhere('Unit', 'LIKE', "%$search%")
                    ->orWhere('Driver Name', 'LIKE', "%$search%")
                    ->orWhere('Location Name', 'LIKE', "%$search%")
                    ->orWhere('Location City', 'LIKE', "%$search%")
                    ->orWhere('Location State/Province', 'LIKE', "%$search%")
                    ->orWhere('Transaction Fee', 'LIKE', "%$search%")
                    ->orWhere('Product Code', 'LIKE', "%$search%")
                    ->orWhere('Quantity', 'LIKE', "%$search%")
                    ->orWhere('Retail_PPU', 'LIKE', "%$search%")
                    ->orWhere('Amount', 'LIKE', "%$search%")
                    ->orWhere('ChainID', 'LIKE', "%$search%");
            });
        }

        return $query;
    }

    public function html()
    {
        return $this->builder()
            ->setTableId('inNetworkGallons-table')
            ->columns($this->getColumns())
            ->minifiedAjax()
            ->dom('Bfrtip')
            ->orderBy(1, 'desc')
            ->pageLength(25)
            ->setTableAttributes([
                'class' => 'table table-striped table-row-bordered table-row-dashed gy-4 align-middle fw-bold dataTable no-footer',
            ])
            ->headerCallback('
            function(thead, data, start, end, display){
                $(thead).addClass("thead-bg-color")
                $(thead).find("th").addClass("fs-7 text-gray-500 text-uppercase");
            }
        ')
            ->createdRow('
            function(row, data, dataIndex) {
                $(row).addClass("fs-6");
            }
        ')
            ->buttons(
                Button::make('excel'),
                Button::make('csv'),
                Button::make('print'),
                Button::make('reset'),
                Button::make('reload')
            )->drawCallback('
            function() {
                $(".dataTables_filter").hide();
            }
            ')->initComplete('
            function(settings, json) {
                $(".dt-buttons").addClass("d-none");
            }
        ');
    }

    protected function getColumns()
    {
        return [
            'Card Number' => ['title' => 'Card'],
            'Transaction Date' => ['title' => 'Date'],
            'Invoice' => ['title' => 'Invoice'],
            'Unit' => ['title' => 'Unit'],
            'Driver Name' => ['title' => 'Driver'],
            'Location Name' => ['title' => 'Location'],
            'Location City' => ['title' => 'City'],
            'Location State/Province' => ['title' => 'State'],
            'Transaction Fee' => ['title' => 'Fee'],
            'Product Code' => ['title' => 'Code'],
            'Quantity' => ['title' => 'Quantity'],
            'Retail_PPU' => ['title' => 'Retail'],
            'Amount' => ['title' => 'Amount'],
        ];
    }

    protected function filename()
    {
        return 'inNetworkGallons_' . date('YmdHis');
    }
}
