<?php

namespace App\DataTables;

use App\Models\Invoice;
use Yajra\DataTables\Html\Button;
use Illuminate\Support\Facades\DB;
use Yajra\DataTables\Services\DataTable;

class InvoiceDataTable extends DataTable
{
    public function dataTable($query)
    {
        return datatables()
            ->eloquent($query)
            ->addColumn('range', function ($invoice) {
                return date("m/d/Y", strtotime($invoice->start_date)) . ' - ' . date("m/d/Y", strtotime($invoice->end_date));
            })
            ->editColumn('amount', function ($invoice) {
                $totalAmount = $invoice->amount;
                if (empty($totalAmount)) {
                    $sql2 = DB::select(DB::raw("SELECT SUM((DailyTrans.`Transaction Fee` + DailyTrans.Amount)) as total FROM DailyTrans LEFT JOIN `CARRIER_PROFILE` ON CARRIER_PROFILE.`SUBFLEET` =DailyTrans.`Sub Fleet` WHERE `Transaction Date` BETWEEN '" . date("Y-m-d", strtotime($invoice->start_date)) . "'  AND '" . date("Y-m-d", strtotime($invoice->end_date)) . "' AND DailyTrans.`Sub Fleet`='" . getAuthMemberId() . "'"));
                    $totalAmount = $sql2[0]->total;

                    Invoice::where('id', $invoice->id)->update([
                        'amount' => $totalAmount
                    ]);
                }
                return '$ ' . number_format($totalAmount, 2);
            })
            ->editColumn('is_paid', function ($invoice) {
                return $invoice->is_paid == "Y" ? "Paid" : "Unpaid";
            })
            ->editColumn('inv_no', function ($invoice) {
                return sprintf('%04d', $invoice->inv_no);
            })
            ->editColumn('bill_date', function ($invoice) {
                return !empty($invoice->bill_date) ? date('m/d/Y', strtotime($invoice->bill_date)) : '';
            })
            ->addColumn('action', function ($invoice) {
                return view('invoices.action', compact('invoice'));
            });
    }

    public function query(Invoice $model)
    {
        $inv_no = $this->request()->get('inv_no');
        $bill_date = $this->request()->get('bill_date');
        $start_date = $this->request()->get('start_date');
        $end_date = $this->request()->get('end_date');
        $query = $model->newQuery()->with('invoicePayments')
            ->where('subfleet', getAuthMemberId());
        if (!empty($inv_no)) {
            $query->where('inv_no', $inv_no);
        }
        if (!empty($bill_date)) {
            $query->where('bill_date', $bill_date);
        }
        if (!empty($start_date) && !empty($end_date)) {
            $query->whereBetween('start_date', [$start_date, $end_date])
                ->whereBetween('end_date', [$start_date, $end_date]);
        }
        $query->withCount([
            'invoicePayments as total_paid_amount' => function ($query) {
                $query->where('payment_type', 'P')
                    ->where('status', 'succeeded')
                    ->select(DB::raw('SUM(amount)'));
            }
        ]);
        $query->groupBy('start_date', 'end_date')
            ->orderBy('id', 'desc');

        return $query;
    }

    public function html()
    {
        return $this->builder()
            ->setTableId('invoice-table')
            ->columns($this->getColumns())
            ->minifiedAjax()
            ->dom('Bfrtip')
            ->orderBy(1)
            ->setTableAttributes([
                'class' => 'table table-striped table-row-bordered table-row-dashed gy-4 align-middle fw-bold dataTable no-footer',
            ])
            ->pageLength(30)
            ->headerCallback('
            function(thead, data, start, end, display){
                $(thead).addClass("thead-bg-color")
                $(thead).find("th").addClass("fs-7 text-gray-500 text-uppercase");
            }
        ')
            ->createdRow('
            function(row, data, dataIndex) {
                $(row).addClass("fs-6");
                if (data.is_paid == "Unpaid") {
                    $(row).find("td").css("color", "red");
                }
                var amount = parseFloat(data.amount.replace(/[^\d.-]/g, ""));
                if (parseFloat(data.total_paid_amount) < amount) {
                    $(row).find("td").css("color", "blue");
                }
            }
        ')
            ->buttons(
                Button::make('excel'),
                Button::make('csv'),
                Button::make('print'),
                Button::make('reset'),
                Button::make('reload')
            )->drawCallback('
            function() {
                $(".dataTables_filter").hide();
            }
            ')->initComplete('
            function(settings, json) {
                $(".dt-buttons").addClass("d-none");
            }
        ');
    }

    protected function getColumns()
    {
        return [
            'inv_no' => ['title' => 'INV #', 'width' => '10%'],
            'bill_date' => ['title' => 'Billed Date', 'width' => '10%'],
            'range' => [
                'title' => 'Range',
                'orderable' => true,
                'searchable' => true,
                'exportable' => true,
                'printable' => true,
                'width' => '20%',
            ],
            'amount' => ['title' => 'Amount', 'width' => '10%'],
            'is_paid' => ['title' => 'Paid', 'width' => '10%'],
            'action' => [
                'title' => 'Action',
                'orderable' => false,
                'searchable' => false,
                'exportable' => false,
                'printable' => false,
                'width' => '30%',
            ],
        ];
    }

    protected function filename()
    {
        return 'Invoice_' . date('YmdHis');
    }
}
