<?php

namespace App\DataTables;

use App\Models\Miles;
use Carbon\Carbon;
use Yajra\DataTables\Html\Button;
use Illuminate\Support\Facades\DB;
use Yajra\DataTables\Services\DataTable;

class MilesByStateDataTable extends DataTable
{
    public function dataTable($query)
    {
        return datatables()
            ->eloquent($query)
            ->editColumn('MILES', function ($row) {
                return number_format($row->MILES, 2);
            })
            ->addColumn('RANGE', function ($row) {
                $start_date = '';
                $end_date = '';
                if (!empty($this->request()->get('start_date'))) {
                    $start_date = $this->request()->get('start_date');
                } else {
                    $start_date = Carbon::now()->startOfMonth();
                }
                if (!empty($this->request()->get('end_date'))) {
                    $end_date = $this->request()->get('end_date');
                } else {
                    $end_date = Carbon::now()->endOfMonth();
                }

                return date('m/d/Y', strtotime($start_date)) .' - '. date('m/d/Y', strtotime($end_date));
            })
            ->addColumn('action', 'milesbystate.action');
    }

    public function query(Miles $model)
    {
        $start_date = '';
        $end_date = '';
        if (!empty ($this->request()->get('start_date'))) {
            $start_date = $this->request()->get('start_date');
        } else {
            $start_date = Carbon::now()->startOfMonth();
        }
        if (!empty ($this->request()->get('end_date'))) {
            $end_date = $this->request()->get('end_date');
        } else {
            $end_date = Carbon::now()->endOfMonth();
        }

        $query = $model->newQuery()
            ->select(DB::raw('`STATE`, SUM(`MILES`) as MILES, `CREATED_DATE`'))
            ->where('CARRIER_SUFBLEET', getAuthMemberId())
            ->whereBetween('CREATED_DATE', [$start_date, $end_date])
            ->groupBy('STATE')->orderBy('CREATED_DATE', 'DESC');

        return $query;
    }

    public function html()
    {
        return $this->builder()
            ->setTableId('milesbystate-table')
            ->columns($this->getColumns())
            ->minifiedAjax()
            ->dom('Bfrtip')
            ->orderBy(1, 'desc')
            ->pageLength(25)
            ->setTableAttributes([
                'class' => 'table table-striped table-row-bordered table-row-dashed gy-4 align-middle fw-bold dataTable',
            ])
            ->headerCallback('
            function(thead, data, start, end, display){
                $(thead).addClass("thead-bg-color")
                $(thead).find("th").addClass("fs-7 text-gray-500 text-uppercase");
            }
        ')
            ->footerCallback('
            function(row, data, start, end, display) {
                var api = this.api();
                var intVal = function(i){
                    return typeof i === "string" ? i.replace(/[\$,]/g, "") * 1 : typeof i === "number" ? i : 0;
                };

                var total = api.column(1).data().reduce(function(a, b){
                    return intVal(a) + intVal(b);
                }, 0);

                $("#displayMilesTotal").html(total.toLocaleString(undefined, {minimumFractionDigits: 2}));
            }
        ')
            ->createdRow('
            function(row, data, dataIndex) {
                $(row).addClass("fs-6");
                if (data.ChainID == "2") {
                    $(row).find("td").css("color", "red");
                }
            }
        ')
            ->buttons(
                Button::make('excel'),
                Button::make('csv'),
                Button::make('print'),
                Button::make('reset'),
                Button::make('reload')
            )
            ->drawCallback('
            function() {
                $(".dataTables_filter").hide();
            }
        ')
            ->initComplete('
            function(settings, json) {
                $(".dt-buttons").addClass("d-none");
            }
        ');
    }

    protected function getColumns()
    {
        return [
            'STATE' => ['title' => 'STATE'],
            'MILES' => ['title' => 'Miles'],
            'RANGE' => [
                'title' => 'RANGE',
                'orderable' => true,
                'searchable' => true,
                'exportable' => true,
                'printable' => true,
                'width' => '30%',
            ]
        ];
    }

    protected function filename()
    {
        return 'MilesByState_' . date('YmdHis');
    }
}
