<?php

namespace App\DataTables;

use Carbon\Carbon;
use App\Models\TransactionReport;
use Yajra\DataTables\Html\Button;
use Yajra\DataTables\Services\DataTable;

class OutOfNetworkDataTable extends DataTable
{
    public function dataTable($query)
    {
        return datatables()
            ->eloquent($query)
            ->addColumn('action', 'outofnetwork.action')
            ->editColumn('Transaction Date', function ($row) {
                return date('m/d/Y', strtotime($row->{'Transaction Date'}));
            })->editColumn('Location Name', function ($row) {
                return '<a href="'.route('transactionsReport.priceListDistance',['StoreID'=>$row->{'Location ID'},'distance'=>'50','DAY'=>$row->{'Transaction Date'},'retail'=>number_format($row->{'Price Per Unit'}, 2),'cid'=>$row->{'Sub Fleet'}]).'">' . $row->{'Location Name'} . ' '. $row->{'Location City'} . ' ' .  $row->{"Location State/Province"} . '</a>';
            })->editColumn('Price Per Unit', function ($row) {
                return "$". number_format($row->{'Price Per Unit'}, 2);
            })->editColumn('Quantity', function ($row) {
                return number_format($row->{'Quantity'}, 2);
            })->editColumn('Amount', function ($row) {
                return "$".number_format($row->{'Amount'}, 2);
            })->rawColumns(['Location Name', 'Price Per Unit', 'Amount']);
    }

    public function query(TransactionReport $model)
    {
        $search = $this->request()->get('search');
        $start = "";
        $end = "";
        if (!empty($this->request()->get('start_date'))) {
            $start = $this->request()->get('start_date');
        } else {
            $start = Carbon::now()->startOfMonth();
        }
        if (!empty($this->request()->get('end_date'))) {
            $end = $this->request()->get('end_date');
        } else {
            $end = Carbon::now()->endOfMonth();
        }

        $query = $model->newQuery()->where('Sub Fleet', getAuthMemberId())->where('ChainID', '2')->where('Product Code', 'ULSD')->whereBetween('Transaction Date', [$start, $end]);
        if (!empty($search)) {
            $query->where(function ($query) use ($search) {
                $query->where('Unit', 'LIKE', "%$search%")
                    ->orWhere('Card Number', 'LIKE', "%$search%")
                    ->orWhere('Driver Name', 'LIKE', "%$search%")
                    ->orWhere('Invoice', 'LIKE', "%$search%")
                    ->orWhere('Product Code', 'LIKE', "%$search%")
                    ->orWhere('Location Name', 'LIKE', "%$search%")
                    ->orWhere('Location City', 'LIKE', "%$search%")
                    ->orWhere('Location State/Province', 'LIKE', "%$search%");
            });
        }

        return $query->orderBy('Transaction Date', 'DESC');
    }

    public function html()
    {
        return $this->builder()
                    ->setTableId('outofnetwork-table')
                    ->columns($this->getColumns())
            ->minifiedAjax()
            ->dom('Bfrtip')
            ->orderBy(1, 'desc')
            ->pageLength(25)
            ->setTableAttributes([
                'class' => 'table table-striped table-row-bordered table-row-dashed dataTable no-footer gy-4 align-middle fw-bold',
            ])
            ->headerCallback('
            function(thead, data, start, end, display){
                $(thead).addClass("thead-bg-color")
                $(thead).find("th").addClass("fs-7 text-gray-500 text-uppercase");
            }
            ')
            ->createdRow('
            function(row, data, dataIndex) {
                $(row).addClass("fs-6");
                $(row).find("td").css("color", "red");
            }
        ')
            ->buttons(
                Button::make('excel'),
                Button::make('csv'),
                Button::make('print'),
                Button::make('reset'),
                Button::make('reload')
            )
            ->drawCallback('
                    function() {
                        $(".dataTables_filter").hide();
                    }
            ')->initComplete('
            function(settings, json) {
                $(".dt-buttons").addClass("d-none");
            }
        ');
    }

    protected function getColumns()
    {
        return [
            'Transaction Date' => ['title' => 'Date'],
            'Unit' => ['title' => 'Unit'],
            'Card Number' => ['title' => 'Card #'],
            'Location Name' => ['title' => 'Location'],
            'Product Code' => ['title' => 'Item'],
            'Price Per Unit' => ['title' => 'Retail'],
            'Quantity' => ['title' => 'Qty'],
            'Amount' => ['title' => 'Amount'],
        ];
    }

    protected function filename()
    {
        return 'OutOfNetwork_' . date('YmdHis');
    }
}
