<?php

namespace App\DataTables;

use App\Models\Trailer;
use Yajra\DataTables\Html\Button;
use Yajra\DataTables\Services\DataTable;

class TrailerDataTable extends DataTable
{
    public function dataTable($query)
    {
        return datatables()
            ->eloquent($query)
            ->addColumn('action', function ($row) {
                return view('assets.trailers.action', compact('row'));
            });
    }

    public function query(Trailer $model)
    {
        $search = $this->request()->get('search');

        $query = $model->newQuery()->where('mem_id', getAuthMemberId());

        if (!empty($search)) {
            $query->where(function ($query) use ($search) {
                $query->where('TRAILER_ID', 'LIKE', "%$search%")
                    ->orWhere('USER1', 'LIKE', "%$search%")
                    ->orWhere('PLATE', 'LIKE', "%$search%")
                    ->orWhere('YEAR', 'LIKE', "%$search%")
                    ->orWhere('TRAILER_TYPE', 'LIKE', "%$search%")
                    ->orWhere('LENGTH_1', 'LIKE', "%$search%");
            });
        }

        return $query;
    }

    public function html()
    {
        return $this->builder()
                    ->setTableId('trailer-table')
                    ->columns($this->getColumns())
                    ->minifiedAjax()
                    ->dom('Bfrtip')
                    ->orderBy(1)
                    ->setTableAttributes([
                        'class' => 'table table-striped table-row-bordered table-row-dashed gy-4 align-middle fw-bold dataTable no-footer',
                    ])
                    ->pageLength(30)
                    ->headerCallback('
                    function(thead, data, start, end, display){
                        $(thead).addClass("thead-bg-color")
                        $(thead).find("th").addClass("fs-7 text-gray-500 text-uppercase");
                    }
                ')
                    ->createdRow('
                    function(row, data, dataIndex) {
                        $(row).addClass("fs-6");
                    }
                ')
                    ->buttons(
                        Button::make('excel'),
                        Button::make('csv'),
                        Button::make('print'),
                        Button::make('reset'),
                        Button::make('reload')
                    )->drawCallback('
                    function() {
                        $(".dataTables_filter").hide();
                    }
                    ')->initComplete('
                    function(settings, json) {
                        $(".dt-buttons").addClass("d-none");
                    }
                ');
    }

    protected function getColumns()
    {
        return [
            'TRAILER_ID' => ['title' => 'Number'],
            'USER1' => ['title' => 'Make'],
            'PLATE' => ['title' => 'Plate'],
            'YEAR' => ['title' => 'Year'],
            'TRAILER_TYPE' => ['title' => 'TYPE'],
            'LENGTH_1' => ['title' => 'Length'],
            'action' => [
                'title' => 'Action',
                'orderable' => false,
                'searchable' => false,
                'exportable' => false,
                'printable' => false,
                'width' => '8%',
            ],
        ];
    }

    protected function filename()
    {
        return 'Trailer_' . date('YmdHis');
    }
}
