<?php

namespace App\DataTables;

use App\Models\TransactionReport;
use Yajra\DataTables\Html\Button;
use Yajra\DataTables\Services\DataTable;

class TransactionReportDataTable extends DataTable
{
    public function dataTable($query)
    {
        return datatables()
            ->eloquent($query)
            ->addColumn('action', 'transactionreport.action')
            ->editColumn('Transaction Date', function ($row) {
                return date('m/d/Y', strtotime($row->{'Transaction Date'}));
            })
            ->editColumn('TIME', function ($row) {
                return date('h:i:s', strtotime($row->{'TIME'}));
            })
            ->editColumn('Location Name', function ($row) {
                return '<a href="'.route('transactionsReport.priceListDistance',['StoreID'=>$row->{'Location ID'},'distance'=>'50','DAY'=>$row->{'Transaction Date'},'retail'=>number_format($row->{'Price Per Unit'}, 2),'cid'=>$row->{'Sub Fleet'}]).'">' . $row->{'Location Name'} . '</a>';
            })
            ->editColumn('Transaction Fee', function ($row) {
                return number_format($row->{'Transaction Fee'}, 2);
            })
            ->editColumn('Price Per Unit', function ($row) {
                return '<div style="white-space: nowrap;">$ ' . number_format($row->{'Price Per Unit'}, 2) . '</div>';
            })
            ->editColumn('Quantity', function ($row) {
                return number_format($row->{'Quantity'}, 2);
            })
            ->editColumn('Amount', function ($row) {
                return '<div style="white-space: nowrap;">$ ' . number_format($row->{'Amount'}, 2) . '</div>';
            })
            ->editColumn('ODOMETER', function ($row) {
                return !empty($row->{'ODOMETER'}) ? $row->{'ODOMETER'} : '0';
            })
            ->rawColumns(['Location Name', 'Price Per Unit', 'Amount']);
    }

    public function query(TransactionReport $model)
    {
        $query = $model->newQuery()->where('Sub Fleet', getAuthMemberId());
        $search = $this->request()->get('search');
        $unit = $this->request()->get('Unit');
        $cardNumber = $this->request()->get('Card_Number');
        $driverName = $this->request()->get('Driver_Name');
        $Location_Names = $this->request()->get('Location_Names');
        $locationCity = $this->request()->get('Location_City');
        $locationState = $this->request()->get('Location_State_Province');
        $startDate = $this->request()->get('start_date');
        $endDate = $this->request()->get('end_date');
        $items = $this->request()->get('items');
        $chain_id = $this->request()->get('ChainID');
        if (!empty($search)) {
            $query->where(function ($query) use ($search) {
                $query->where('Unit', 'LIKE', "%$search%")
                    ->orWhere('Card Number', 'LIKE', "%$search%")
                    ->orWhere('Driver Name', 'LIKE', "%$search%")
                    ->orWhere('Invoice', 'LIKE', "%$search%")
                    ->orWhere('Product Code', 'LIKE', "%$search%")
                    ->orWhere('Location Name', 'LIKE', "%$search%")
                    ->orWhere('Location City', 'LIKE', "%$search%")
                    ->orWhere('Location State/Province', 'LIKE', "%$search%");
            });
        }
        if (!empty($unit)) {
            $query->whereIn('Unit', $unit);
        }
        if (!empty($cardNumber)) {
            $query->whereIn('Card Number', $cardNumber);
        }
        if (!empty($driverName)) {
            $query->where('Driver Name', $driverName);
        }
        if (!empty($Location_Names)) {

            if(count($Location_Names) == 2) {
                $query->whereRaw('(`Location Name` Like "one9%" OR ChainID in (2))');
            }else if(in_array('Love',$Location_Names)){
                $query->whereRaw('ChainID in (2)');
            }else if(in_array('one9',$Location_Names)){
                $query->whereRaw('(`Location Name` Like "one9%")');
            }

        }
        if (!empty($locationCity)) {
            $query->where('Location City', $locationCity);
        }
        if (!empty($locationState)) {
            $query->whereIn('Location State/Province', $locationState);
        }
        if (!empty($startDate) && !empty($endDate)) {
            $query->whereBetween('Transaction Date', [$startDate, $endDate]);
        }
        if (!empty($items)) {
            $query->whereIn('Product Code', $items);
        }
        if (!empty($chain_id)) {
            $query->whereIn('ChainID', $chain_id);
        }

        return $query->orderBy('Transaction Date', 'DESC');
    }

    public function html()
    {
        return $this->builder()
            ->setTableId('transactionreport-table')
            ->columns($this->getColumns())
            ->minifiedAjax()
            ->dom('Bfrtip')
            ->orderBy(1)
            ->setTableAttributes([
                'class' => 'table table-striped table-row-bordered table-row-dashed dataTable no-footer gy-4 align-middle fw-bold',
            ])
            ->pageLength(30)
            ->headerCallback('
            function(thead, data, start, end, display){
                $(thead).addClass("thead-bg-color")
                $(thead).find("th").addClass("fs-7 text-gray-500 text-uppercase");
            }
            ')
            ->createdRow('
            function(row, data, dataIndex) {
                $(row).addClass("fs-6");
                if (data.ChainID == "2") {
                    $(row).find("td").css("color", "red");
                }
            }
        ')
            ->buttons(
                Button::make('excel'),
                Button::make('csv'),
                Button::make('print'),
                Button::make('reset'),
                Button::make('reload')
            )
            ->drawCallback('
                    function() {
                        $(".dataTables_filter").hide();
                    }
            ')->initComplete('
            function(settings, json) {
                $(".dt-buttons").addClass("d-none");
            }
        ');
    }

    protected function getColumns()
    {
        return [
            'Transaction Date' => ['title' => 'Date'],
            'TIME' => ['title' => 'Time'],
            'Invoice' => ['title' => 'Invoice'],
            'Unit' => ['title' => 'Unit'],
            'Card Number' => ['title' => 'Card #'],
            'Driver Name' => ['title' => 'Driver'],
            'ODOMETER' => ['title' => 'ODOMETER'],
            'Location Name' => ['title' => 'Location'],
            'Location City' => ['title' => 'City'],
            'Location State/Province' => ['title' => 'State'],
            'Transaction Fee' => ['title' => 'Fee'],
            'Product Code' => ['title' => 'Item'],
            'Price Per Unit' => ['title' => 'Retail'],
            'Quantity' => ['title' => 'Qty'],
            'Amount' => ['title' => 'Amount'],
        ];
    }

    protected function filename()
    {
        return 'TransactionReport_' . date('YmdHis');
    }
}
