<?php

namespace App\DataTables;

use App\Models\Truck;
use Yajra\DataTables\Html\Button;
use Yajra\DataTables\Services\DataTable;

class TruckDataTable extends DataTable
{
    public function dataTable($query)
    {
        return datatables()
            ->eloquent($query)
            ->editColumn('UNIT_ID', function($row){
                $img = '';
                if(!empty($row->PHOTOS) && $row->PHOTOS!= ''){
                    $img= '<img src="'.asset('trucks/PHOTOS/' . $row->PHOTOS).'"  border="0" height="" width="84" data-no-statview="no" style="max-width: 130px;">';
                }

                return '<a href="'.route('assets.trucks.edit', ['truckId' => $row->{'TRUCK_ID'}]).'">  '. $img . $row->UNIT_ID.'</a>' ;
            }) 
            ->editColumn('LIC_1_EXP_DATE', function($row){
                return !empty($row->LIC_1_EXP_DATE) ? date('m/d/Y', strtotime($row->LIC_1_EXP_DATE)) : '';
            })
            ->editColumn('ODOMETER_READING', function($row){
                return !empty($row->ODOMETER_READING) ? date('m/d/Y', strtotime($row->ODOMETER_READING)) : '';
            })
            ->editColumn('STATUS', function($row){
                $status = '';
                if($row->STATUS == 'AVAILABLE'){
                    $status = 'checked';                    
                }
                return '<label class="switch">
                            <input class="switch-input" type="checkbox" id="STATUS" ' . $status .'  onclick="statusChange(this,'.$row->TRUCK_ID.')">
                            <span class="switch-label fw-bold" data-on="AVAILABLE" data-off="INACTIVE"></span>
                            <span class="switch-handle"></span>
                        </label>';
            })
            ->addColumn('action', function ($row) {
                return view('assets.trucks.action', compact('row'));
            })
            ->rawColumns(['UNIT_ID','STATUS']);;
    }

    public function query(Truck $model)
    {
        $search = $this->request()->get('search');

        $query = $model->newQuery()->where('mem_id', getAuthMemberId());

        if (!empty($search)) {
            $query->where(function ($query) use ($search) {
                $query->where('UNIT_ID', 'LIKE', "%$search%")
                    ->orWhere('MAKE', 'LIKE', "%$search%")
                    ->orWhere('MODEL', 'LIKE', "%$search%")
                    ->orWhere('YEAR', 'LIKE', "%$search%")
                    ->orWhere('STATUS', 'LIKE', "%$search%")
                    // ->orWhere('VIN', 'LIKE', "%$search%")
                    // ->orWhere('AXLES', 'LIKE', "%$search%")
                    ->orWhere('PLATE', 'LIKE', "%$search%");
                    //->orWhere('ODOMETER', 'LIKE', "%$search%");
            });
        }

        return $query;
    }

    public function html()
    {
        return $this->builder()
                    ->setTableId('truck-table')
                    ->columns($this->getColumns())
                    ->minifiedAjax()
                    ->dom('Bfrtip')
                    ->orderBy(1)
                    ->setTableAttributes([
                        'class' => 'table table-striped table-row-bordered table-row-dashed gy-4 align-middle fw-bold dataTable no-footer nowrap',
                    ])
                    ->pageLength(30)
                    ->headerCallback('
                    function(thead, data, start, end, display){
                        $(thead).addClass("thead-bg-color")
                        $(thead).find("th").addClass("fs-7 text-gray-500 text-uppercase");
                    }
                ')
                    ->createdRow('
                    function(row, data, dataIndex) {
                        $(row).addClass("fs-6");
                    }
                ')
                    ->buttons(
                        Button::make('excel'),
                        Button::make('csv'),
                        Button::make('print'),
                        Button::make('reset'),
                        Button::make('reload')
                    )->drawCallback('
                    function() {
                        $(".dataTables_filter").hide();
                    }
                    ')->initComplete('
                    function(settings, json) {
                        $(".dt-buttons").addClass("d-none");
                    }
                ');
    }

    protected function getColumns()
    {
        return [
            'UNIT_ID' => ['title' => 'Unit #'],
            'MAKE' => ['title' => 'Make'],
            'MODEL' => ['title' => 'Model'],
            'YEAR' => ['title' => 'Year'],
            // 'VIN' => ['title' => 'VIN #'],
            // 'AXLES' => ['title' => 'Axles'],
            'PLATE' => ['title' => 'Plate #'],
            'LIC_1_EXP_DATE' => ['title' => 'Plate Expires'],
            'STATUS' => ['title' => 'Status'],
            // 'ODOMETER_READING' => ['title' => 'Odo Date'],
            // 'ODOMETER' => ['title' => 'Odo Miles'],
            'action' => [
                'title' => 'Action',
                'orderable' => false,
                'searchable' => false,
                'exportable' => false,
                'printable' => false,
                'width' => '8%',
            ],
        ];
    }

    protected function filename()
    {
        return 'Truck_' . date('YmdHis');
    }
}
