<?php

namespace App\Http\Controllers\Auth;

use App\Models\CarrierProfile;
use App\Models\Member;
use App\Models\LoginHistory;
use App\Models\Token;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;

class LoginController extends Controller
{
    public function login()
    {
        if (Auth::check()) {
            return redirect()->back();
        }

        return view('auth.login');
    }

    public function loginCheck(Request $request)
    {
        $this->validate($request, [
            'username' => 'required',
            'password' => 'required',
        ]);

        $ipaddress = 'UNKNOWN';

        $username = $request->username;
        $password = $request->password;

        $member = Member::where('username', $username)->where('password', $password)->first();

        if (!empty($member)) {
            LoginHistory::create([
                'USER' => $member->username,
                'DATE' => date("Y-m-d"),
                'TIME' => date("h:i A"),
                'IP' => $ipaddress,
                'IN_OUT' => 'LOGIN',
            ]);
            $checkLastLoginIp = Member::where('username', $member->username)->first();
            if ($checkLastLoginIp->{'2FA'} == 1) {
                $memuserId = $checkLastLoginIp->id;
                $memId = $checkLastLoginIp->mem_id;
                $CarrierData = CarrierProfile::where('SUBFLEET', $memId)->first();

                $digits = 4;
                $token = rand(pow(10, $digits - 1), pow(10, $digits) - 1);
                $cdate = date("Y-m-d H:i:s");

                Token::create([
                    'token' => $token,
                    'memberuser_id' => $memuserId,
                    'created_at' => $cdate,
                ]);

                $name = $CarrierData->CONTACT_NAME;
                $email = $CarrierData->EMAIL;
                $phone = preg_replace('/[^0-9]/', '', $CarrierData->PHONE);

                /*send Mail*/
                $email_data = [
                    'email' => $email,
                    'name' => $name,
                    'token' => $token
                ];
                $emailURL = "https://efspro.tmdashboard.com/sendgrid/login_verification_new.php?" . http_build_query($email_data) . "";
                $ch = curl_init();
                curl_setopt($ch, CURLOPT_URL, $emailURL);
                curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
                $ch_res = curl_exec($ch);
                curl_close($ch);
                /*send Mail*/

                /*send SMS*/
                $body = $token . " is OTP for login verification.";
                $SMSurl = "https://tms.tmdashboard.com//twiliosms/smssend.php?to=$phone&message=" . urlencode($body) . "";
                $ch = curl_init();
                curl_setopt($ch, CURLOPT_URL, $SMSurl);
                curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
                $ch_res = curl_exec($ch);
                curl_close($ch);
                /*send SMS*/

                return redirect()->route('two_fator', md5($memuserId));
            }else{
                Auth::login($member);
                return redirect()->route('dashboard')->with('success', 'Logged In Successfully');
            }
        } else {
            return redirect()->back()->with('error', 'Incorrect Username or Password')->withInput();
        }
    }

    public function logout()
    {
        $ipaddress = 'UNKNOWN';

        LoginHistory::create([
            'USER' => Auth::user()->username,
            'DATE' => date("Y-m-d"),
            'TIME' => date("h:i A"),
            'IP' => $ipaddress,
            'IN_OUT' => 'LOGOUT',
        ]);

        Auth::logout();
        return redirect()->route('login')->with('error', 'OOPS ! You Have Been Signed Out');
    }

    public function two_fator($id)
    {
        return view('auth.two_factor', compact('id'));
    }

    public function two_factor_check(Request $request)
    {
        $this->validate($request, [
            'code' => 'required'
        ]);

        $code = $request->code;
        $id = $request->id;

        $rowc1 = Member::where(function ($query) use ($id) {
            $query->whereRaw('MD5(id) = ?', [$id]);
        })->first();

        $memuserId = $rowc1->id;

        $rowc2 = Token::where('token', $code)->where('memberuser_id', $memuserId)->orderBy('id', 'DESC')->limit(1)->get();
        if($rowc2->isEmpty()){
            return redirect()->back()->with('error', 'verification code not found with associated user')->withInput();
        } else {
            Auth::login($rowc1);
            return redirect()->route('dashboard')->with('success', 'Logged In Successfully');
        }
    }

    public function resend_login_token($id)
    {
        $rowc1 = Member::where(function ($query) use ($id) {
            $query->whereRaw('MD5(id) = ?', [$id]);
        })->first();

        $memuserId = $rowc1->id;
        $memId = $rowc1->mem_id;

        $CarrierData = CarrierProfile::where('SUBFLEET', $memId)->first();

        $digits = 4;
        $token = rand(pow(10, $digits - 1), pow(10, $digits) - 1);
        $cdate = date("Y-m-d H:i:s");

        Token::create([
            'token' => $token,
            'memberuser_id' => $memuserId,
            'created_at' => $cdate,
        ]);

        $name = "";
        $email = "";
        $phone = "9092244154";

        /*send Mail*/
        $email_data = [
            'email' => $email,
            'name' => $name,
            'token' => $token
        ];
        $emailURL = "https://efspro.tmdashboard.com/sendgrid/login_verification_new.php?" . http_build_query($email_data) . "";
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $emailURL);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        $ch_res = curl_exec($ch);
        curl_close($ch);
        /*send Mail*/

        /*send SMS*/
        $body = $token . " is OTP for login verification.";
        $SMSurl = "https://tms.tmdashboard.com//twiliosms/smssend.php?to=$phone&message=" . urlencode($body) . "";
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $SMSurl);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        $ch_res = curl_exec($ch);
        curl_close($ch);
        /*send SMS*/

        return response()->json(['status' => true, 'message' => 'Verification code re-sent successfully']);
    }
}
