<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\CarrierProfile;
use App\Models\EFSCreateOrder;
use App\DataTables\CardOrderDataTable;

require public_path() . '/setting/sendgrid/vendor/autoload.php';

class CardOrderController extends Controller
{
    public function index(CardOrderDataTable $dataTable)
    {
        return $dataTable->render('cardOrders.index');
    }

    public function add()
    {
        $dot = CarrierProfile::Where('SUBFLEET', getAuthMemberId())->first();
        return view('cardOrders.add', compact('dot'));
    }

    public function view($id)
    {
        $cardOrder = EFSCreateOrder::where('ORDER_ID', $id)->first();
        return view('cardOrders.view', compact('cardOrder'));
    }

    public function store(Request $request)
    {
        $this->validate($request, [
            'order_type' => 'required',
            'card_style' => 'required',
            'qty' => 'required',
            'embossname' => 'required|max:25',
            'ship_method' => 'required',
        ]);

        $card_order = EFSCreateOrder::create([
            'USER_ID' => $request->USER_ID,
            'STATUS' => $request->STATUS,
            'ORDER_TYPE' => $request->order_type,
            'CARD_STYLE' => $request->card_style,
            'QTY' => $request->qty,
            'EMBOSSNAME' => $request->embossname,
            'SHIP_METHOD' => $request->ship_method,
            'CREATED_DATE' => date('Y-m-d'),
        ]);

        $user = CarrierProfile::where('SUBFLEET', $request->USER_ID)->first();
        $email_data = [
            'USER_ID' => $card_order->USER_ID,
            'NAME' => $user->NAME,
            'ORDER_TYPE' => $card_order->ORDER_TYPE,
            'CARD_STYLE' => $card_order->CARD_STYLE,
            'QTY' => $card_order->QTY,
            'EMBOSSNAME' => $card_order->EMBOSSNAME,
            'SHIP_METHOD' => $card_order->SHIP_METHOD,
            'EMAIL' => $user->EMAIL,
            'C411' => $user->C411,
            'CONTRACT_ID' => $user->CONTRACT_ID
        ];
        $body_email = view('card_order_mail', compact('user', 'email_data'))->render();
        $email = new \SendGrid\Mail\Mail();
            $email->setFrom("support@tmscloud.com", "Card Orders");
            $email->setSubject("New Card Order");
            $email->addTo("cardservices@tmscloud.com", "Card Services");
            $email->addCc("daryn@tmscloud.com", "Daryn Duff");
            $email->addContent(
                "text/html",
                $body_email
            );
            $sendgrid = new \SendGrid('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');
            try {
                $response = $sendgrid->send($email);
            } catch (\Exception $e) {
                echo 'Caught exception: ' . $e->getMessage() . "\n";
            }

        return redirect()->route('card_orders.index')->with('success', 'Card Ordered Successfully');
    }
}
