<?php

namespace App\Http\Controllers;

use App\Models\CarrierProfile;
use App\Models\Disputes;
use Illuminate\Http\Request;
use App\DataTables\DisputesDataTable;

require public_path() . '/setting/sendgrid/vendor/autoload.php';

class DisputesController extends Controller
{
    public function index(DisputesDataTable $dataTable)
    {
        return $dataTable->render('Disputes.index');
    }

    public function newIndex(DisputesDataTable $dataTable)
    {
        return $dataTable->render('Disputes.newIndex');
    }

    public function edit($id)
    {
        $disputes = Disputes::find($id);
        return view('Disputes.edit', compact('disputes'));
    }

    public function view($id)
    {
        $disputes = Disputes::find($id);
        return view('Disputes.view', compact('disputes'));
    }

    public function create()
    {
        return view('Disputes.add');
    }

    public function store(Request $request)
    {
        $maxFileSize = 20000;
        $this->validate($request, [
            'DEPT' => 'required',
            'TITLE' => 'required',
            'MESSAGE' => 'required',
            'STATUS' => 'required',
            'DOCUMENTS' => 'file|mimes:doc,docx,pdf,png,jpg,jpeg,xls,xlsx|max:' . $maxFileSize,
        ], [
            'DEPT.required' => 'DEPT is Required.',
            'TITLE.required' => 'TITLE is Required.',
            'MESSAGE.required' => 'MESSAGE is Required.',
            'STATUS.required' => 'STATUS is Required.',
            'DOCUMENTS.file' => 'DOCUMENTS must be a file.',
            'DOCUMENTS.mimes' => 'Allowed file types are: doc, docx, pdf, png, jpg, jpeg, xls, xlsx.',
            'DOCUMENTS.size' => 'Maximum file size allowed is ' . $maxFileSize . ' KB.',
        ]);

        $carrierInfo = CarrierProfile::where('SUBFLEET', getAuthMemberId())->first();

        $fileName = '';
        if ($request->hasFile('DOCUMENTS')) {
            $file = $request->file('DOCUMENTS');
            $fileName = time() . '_' . $file->getClientOriginalName();
            $file->move(public_path('documents'), $fileName);
        }

        $disputes = new Disputes();
        $disputes->DEPT = $request->DEPT;
        $disputes->STATUS = $request->STATUS;
        $disputes->CARRIER_ID = getAuthMemberId();
        $disputes->CREATED_BY = 'Administrator';
        $disputes->TITLE = $request->TITLE;
        $disputes->MESSAGE = $request->MESSAGE;
        $disputes->CREATED_DATE = date('Y-m-d');
        $disputes->DOCUMENTS = $fileName;
        $disputes->save();

        $url = 'https://pro.tmdashboard.com/login';

        $subject = 'Net Ticket - ' . $disputes->DEPT . ' - ' . date('m/d/Y');

        $body = '<div class=""
                    style="font-size: 14px; font-family: Helvetica Neue, Helvetica, Arial, sans-serif; mso-line-height-alt: 16.8px; color: #40507a; line-height: 1.2;">
                    <p>
                        Ticket Number : ' . $disputes->TICKET_ID . '<br>
                        Carrier : ' . $carrierInfo->NAME . '<br>
                        Policy # : ' . $carrierInfo->POLICY_NUM . '<br>
                        EFS Customer # : ' . $carrierInfo->C411 . '<br>
                        Category : ' . $disputes->DEPT . '<br>
                        Status : ' . $disputes->STATUS . '<br>
                        Title : ' . $disputes->TITLE . '<br>
                        Summary : ' . $disputes->MESSAGE . '<br>
                        Created Date : ' . date('m/d/Y', strtotime($disputes->CREATED_DATE)) . '
                    </p>
                    <P>
                        login for Fuel TMS <a
                            href="' . $url . '">Click
                            here</a>
                    </P>
                </div>';

        if (isset($fileName) && $fileName != '') {
            $email = new \SendGrid\Mail\Mail();
            $email->setfrom("no_reply@freeifta.com", "Fuel TMS");
            $email->setSubject($subject);
            $email->addTo('daryn@tmscloud.com');
            $email->addTo('apply@cdlapps.com');
            $email->addContent(
                "text/html",
                $body
            );
            $fileContents = file_get_contents(public_path('documents/' . $fileName));
            $attachment = new \SendGrid\Mail\Attachment();
            $attachment->setContent(base64_encode($fileContents));
            $attachment->setType(mimes_type(public_path('documents/' . $fileName)));
            $attachment->setFilename($fileName);
            $attachment->setDisposition("attachment");
            $email->addAttachment($attachment);
            $sendgrid = new \SendGrid('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');
            try {
                $response = $sendgrid->send($email);
            } catch (\Exception $e) {
                echo 'Caught exception: ' . $e->getMessage() . "\n";
            }
        } else {
            $email = new \SendGrid\Mail\Mail();
            $email->setfrom("no_reply@freeifta.com", "Fuel TMS");
            $email->setSubject($subject);
            $email->addTo('daryn@tmscloud.com');
            $email->addTo('apply@cdlapps.com');
            $email->addContent(
                "text/html",
                $body
            );
            $sendgrid = new \SendGrid('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');
            try {
                $response = $sendgrid->send($email);
            } catch (\Exception $e) {
                echo 'Caught exception: ' . $e->getMessage() . "\n";
            }
        }

        return redirect()->route('Disputes.index')->with('success', 'Disputes Created Successfully.');
    }

    public function update(Request $request)
    {
        $maxFileSize = 20000;
        $this->validate($request, [
            'DEPT' => 'required',
            'TITLE' => 'required',
            'MESSAGE' => 'required',
            'DOCUMENTS' => 'file|mimes:doc,docx,pdf,png,jpg,jpeg,xls,xlsx|max:' . $maxFileSize,
        ], [
            'DEPT.required' => 'DEPT is Required.',
            'TITLE.required' => 'TITLE is Required.',
            'MESSAGE.required' => 'MESSAGE is Required.',
            'DOCUMENTS.file' => 'DOCUMENTS must be a file.',
            'DOCUMENTS.mimes' => 'Allowed file types are: doc, docx, pdf, png, jpg, jpeg, xls, xlsx.',
            'DOCUMENTS.size' => 'Maximum file size allowed is ' . $maxFileSize . ' KB.',
        ]);

        $carrierInfo = CarrierProfile::where('SUBFLEET', getAuthMemberId())->first();

        $fileName = '';
        if ($request->hasFile('DOCUMENTS')) {
            $file = $request->file('DOCUMENTS');
            $fileName = time() . '_' . $file->getClientOriginalName();
            $file->move(public_path('documents'), $fileName);
        }

        $disputes = Disputes::find($request->TICKET_ID);
        $disputes->DEPT = $request->DEPT;
        $disputes->CARRIER_ID = getAuthMemberId();
        $disputes->CREATED_BY = 'Administrator';
        $disputes->TITLE = $request->TITLE;
        $disputes->MESSAGE = $request->MESSAGE;
        $disputes->CREATED_DATE = date('Y-m-d');

        if ($fileName) {
            $disputes->DOCUMENTS = $fileName;
        }

        $disputes->save();

        $url = 'https://pro.tmdashboard.com/login';

        $subject = 'Net Ticket - ' . $disputes->DEPT . ' - ' . date('m/d/Y');
        $body = '<div class=""
                    style="font-size: 14px; font-family: Helvetica Neue, Helvetica, Arial, sans-serif; mso-line-height-alt: 16.8px; color: #40507a; line-height: 1.2;">
                    <p>
                        Ticket Number : ' . $disputes->TICKET_ID . '<br>
                        Carrier : ' . $carrierInfo->NAME . '<br>
                        Policy # : ' . $carrierInfo->POLICY_NUM . '<br>
                        EFS Customer # : ' . $carrierInfo->C411 . '<br>
                        Category : ' . $disputes->DEPT . '<br>
                        Status : ' . $disputes->STATUS . '<br>
                        Title : ' . $disputes->TITLE . '<br>
                        Summary : ' . $disputes->MESSAGE . '<br>
                        Created Date : ' . date('m/d/Y', strtotime($disputes->CREATED_DATE)) . '
                    </p>
                    <P>
                        login for Fuel TMS <a
                            href="' . $url . '">Click
                            here</a>
                    </P>
                </div>';

        if (isset($fileName) && $fileName != '') {
            $email = new \SendGrid\Mail\Mail();
            $email->setfrom("no_reply@freeifta.com", "Fuel TMS");
            $email->setSubject($subject);
            $email->addTo('daryn@tmscloud.com');
            $email->addTo('apply@cdlapps.com');
            $email->addContent(
                "text/html",
                $body
            );
            $fileContents = file_get_contents(public_path('documents/' . $fileName));
            $attachment = new \SendGrid\Mail\Attachment();
            $attachment->setContent(base64_encode($fileContents));
            $attachment->setType(mimes_type(public_path('documents/' . $fileName)));
            $attachment->setFilename($fileName);
            $attachment->setDisposition("attachment");
            $email->addAttachment($attachment);
            $sendgrid = new \SendGrid('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');
            try {
                $response = $sendgrid->send($email);
            } catch (\Exception $e) {
                echo 'Caught exception: ' . $e->getMessage() . "\n";
            }
        } else {
            $email = new \SendGrid\Mail\Mail();
            $email->setfrom("no_reply@freeifta.com", "Fuel TMS");
            $email->setSubject($subject);
            $email->addTo('daryn@tmscloud.com');
            $email->addTo('apply@cdlapps.com');
            $email->addContent(
                "text/html",
                $body
            );
            $sendgrid = new \SendGrid('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');
            try {
                $response = $sendgrid->send($email);
            } catch (\Exception $e) {
                echo 'Caught exception: ' . $e->getMessage() . "\n";
            }
        }

        return redirect()->route('Disputes.index')->with('success', 'Disputes Update Successfully.');
    }

    public function delete($id)
    {
        $disputes = Disputes::find($id);
        $disputes->delete();
        return redirect()->route('Disputes.index')->with('success', 'Disputes Deleted Successfully.');
    }
}
