<?php

namespace App\Http\Controllers;

use App\Models\Driver;
use App\Models\Truck;
use Illuminate\Http\Request;
use App\DataTables\DriverDataTable;
use Illuminate\Support\Facades\DB;

class DriverController extends Controller
{
    public function index(DriverDataTable $dataTable)
    {
        return $dataTable->render('assets.drivers.index');
    }

    public function add()
    {
        return view('assets.drivers.add');
    }

    public function store(Request $request)
    {
        $this->validate(
            $request,
            [
                'FIRST_NAME' => 'required',
                'LAST_NAME' => 'required',
                'EMAIL' => 'required|email',
                'PHONE' => 'required',
                'STATUS' => 'required',
            ],
            [
                'FIRST_NAME.required' => 'The First Name field is Required',
                'LAST_NAME.required' => 'The Last Name field is Required',
                'EMAIL.required' => 'The Email field is Required',
                'PHONE.required' => 'The PHONE field is Required',
                'STATUS.required' => 'The Profile Status field is Required',
            ]
        );

        $driver = Driver::create([
            'NAME' => $request->FIRST_NAME . ' ' . $request->LAST_NAME,
            'FIRST_NAME' => $request->FIRST_NAME,
            'LAST_NAME' => $request->LAST_NAME,
            'EMAIL' => $request->EMAIL,
            'PHONE' => $request->PHONE,
            'STATUS' => $request->STATUS,
            'LICENCE_NUMBER' => $request->LICENCE_NUMBER,
            'START_DATE' => $request->START_DATE,
            'CREATED_DATE' => date('Y-m-d H:i:s'),
            'CREATED_BY' => getAuthMemberId(),
            'mem_id' => getAuthMemberId(),
        ]);

        if(!empty($request->truckId)){
            Truck::where('TRUCK_ID', $request->truckId)->update(['ASSIGNED_DRIVER' => $driver->DRIVER_ID]);
        }

        if(!empty($request->from && $request->from == 'truckEdit')){
            $url = redirect()->back()->with('success', 'Driver Added Successfully')->getTargetUrl() . '#kt_stats_widget_16_tab_4';
            return redirect()->to($url);
        }else{
            return redirect()->route('assets.drivers.index')->with('success', 'Driver Added Successfully');
        }
    }

    public function edit($driverId)
    {
        $driver = Driver::find($driverId);
        $ENDORSEMENTS = explode(',', $driver->ENDORSEMENTS);
        $badareas = explode(',', $driver->badareas);
        $assignedTruckIds = Driver::where('mem_id', getAuthMemberId())->whereNotNull('TRUCK_ID')->where('TRUCK_ID', '!=', $driver->TRUCK_ID)->pluck('TRUCK_ID')->toArray();
        $trucks = Truck::where('mem_id', getAuthMemberId())->whereNotIn('TRUCK_ID', $assignedTruckIds)->get();
        return view('assets.drivers.edit', compact('driver', 'ENDORSEMENTS', 'badareas', 'trucks'));
    }

    public function update(Request $request)
    {
        $this->validate(
            $request,
            [
                'FIRST_NAME' => 'required',
                'LAST_NAME' => 'required',
                'EMAIL' => 'required|email',
                'STATUS' => 'required',
                'EXPIRY_DATE' => 'required',
                'DL_STATE_PROV' => 'required',
                'CODE' => 'required',
                'BIRTH_DATE' => 'required',
            ],
            [
                'FIRST_NAME.required' => 'The First Name field is Required',
                'LAST_NAME.required' => 'The Last Name field is Required',
                'EMAIL.required' => 'The Email field is Required',
                'STATUS.required' => 'The Profile Status field is Required',
                'EXPIRY_DATE.required' => 'The License Expires field is Required',
                'DL_STATE_PROV.required' => 'The License State field is Required',
                'CODE.required' => 'The License Class field is Required',
                'BIRTH_DATE.required' => 'The Birth Date field is Required',
            ]
        );

        $ENDORSEMENTS = !empty($request->ENDORSEMENTS) ? implode(',', $request->ENDORSEMENTS) : '';
        $filenames = [];
        $profiles = '';
        if ($request->hasFile('DRIVER_BITMAP')) {
            $file = $request->file('DRIVER_BITMAP');
            $profiles = time() . '_' . $file->getClientOriginalName();
            $file->move(public_path('driver/profiles/'), $profiles);
        }

        if ($request->hasFile('DOCUMENTS')) {
            foreach ($request->file('DOCUMENTS') as $file) {
                if ($file->isValid()) {
                    $fileName = time() . '_' . $file->getClientOriginalName();
                    $file->move(public_path('driver/documents/'), $fileName);
                    $filenames[] = $fileName;
                }
            }
        }

        $badareas = !empty($request->badareas) ? implode(",", $request->badareas) : '';
        $documents = implode(',', $filenames);

        Driver::where('DRIVER_ID', $request->DRIVER_ID)->update([
            'NAME' => $request->FIRST_NAME . ' ' . $request->LAST_NAME,
            'SIN_NUMBER' => $request->SIN_NUMBER,
            'FIRST_NAME' => $request->FIRST_NAME,
            'LAST_NAME' => $request->LAST_NAME,
            'LICENCE_NUMBER' => $request->LICENCE_NUMBER,
            'EXPIRY_DATE' => $request->EXPIRY_DATE,
            'DL_STATE_PROV' => $request->DL_STATE_PROV,
            'ENDORSEMENTS' => $ENDORSEMENTS,
            'CODE' => $request->CODE,
            'EMPLOYEE_CODE' => $request->EMPLOYEE_CODE,
            'BIRTH_DATE' => $request->BIRTH_DATE,
            'STATUS' => $request->STATUS,
            'START_DATE' => $request->START_DATE,
            'TERMINATION_DATE' => $request->TERMINATION_DATE,
            'PHONE' => $request->PHONE,
            'EMAIL' => $request->EMAIL,
            'NOSMS' => $request->NOSMS,
            'CELL_PHONE' => $request->CELL_PHONE,
            'EMERGENCY_CONTACT' => $request->EMERGENCY_CONTACT,
            'EMERGENCY_PHONE' => $request->EMERGENCY_PHONE,
            'SECONDARY_CONTRACT' => $request->SECONDARY_CONTRACT,
            'OTHER_CONTACT_2' => $request->OTHER_CONTACT_2,
            'ADDRESS_1' => $request->ADDRESS_1,
            'ADDRESS_2' => $request->ADDRESS_2,
            'CITY' => $request->CITY,
            'PROVINCE' => $request->PROVINCE,
            'POSTAL_CODE' => $request->POSTAL_CODE,
            'Medical_Card_Expiry' => $request->Medical_Card_Expiry,
            'Fit_Test_Exp' => $request->Fit_Test_Exp,
            'WCB_NUMBER' => $request->WCB_NUMBER,
            'WCB_EXPIRY_DATE' => $request->WCB_EXPIRY_DATE,
            'hazmat_num' => $request->hazmat_num,
            'hazmat_exp' => $request->hazmat_exp,
            'ROAD_TEST' => $request->ROAD_TEST,
            'DRUG_TEST' => $request->DRUG_TEST,
            'PEC_Card_Exp' => $request->PEC_Card_Exp,
            'PAY_TYPE' => $request->PAY_TYPE,
            'PER_HOUR' => $request->PER_HOUR,
            'TRIP_REVENUE_SHARE' => $request->TRIP_REVENUE_SHARE,
            'PER_LOADED_MILE' => $request->PER_LOADED_MILE,
            'PER_EMPTY_MILE' => $request->PER_EMPTY_MILE,
            'STOP_RATE' => $request->STOP_RATE,
            'LAYOVER_RATE' => $request->LAYOVER_RATE,
            'DIRECT_DEPOSIT' => $request->DIRECT_DEPOSIT,
            'FORM1099' => isset($request->FORM1099) ? $request->FORM1099 : 0,
            'CHEQUE_NAME' => $request->CHEQUE_NAME,
            'DD_BANK' => $request->DD_BANK,
            'DD_ACCOUNT' => $request->DD_ACCOUNT,
            'DD_TRANSIT' => $request->DD_TRANSIT,
            'INS_COMPANY' => $request->INS_COMPANY,
            'INS_GROUP' => $request->INS_GROUP,
            'INS_EXPIRE' => $request->INS_EXPIRE,
            'INS_IDNUM' => $request->INS_IDNUM,
            'DRIVER_BITMAP' => !empty($profiles) ? $profiles : $request->old_image,
            'DOCUMENTS' => !empty($documents) ? $documents : $request->multi_old_image,
            'DRIVER_TYPE' => $request->DRIVER_TYPE,
            'driver_touch' => $request->driver_touch,
            'DRIVER_NIGHTS' => $request->DRIVER_NIGHTS,
            'DROPS_OK' => $request->mulistop,
            'max_picks' => $request->max_picks,
            'max_drops' => $request->max_drops,
            'MAX_WEIGHT' => $request->max_weg,
            'MILES_PER_DAY' => $request->maxmiles,
            'badareas' => $badareas,
            'DRIVER_NOTES' => htmlspecialchars($request->editorContent),
            'TRUCK_ID' => $request->TRUCK_ID,
        ]);

        return redirect()->route('assets.drivers.edit', [$request->DRIVER_ID . $request->activeTab])->with('success', 'Driver Updated Successfully');
    }

    public function delete($driverId)
    {
        Driver::where('DRIVER_ID', $driverId)->delete();
        return redirect()->route('assets.drivers.index')->with('success', 'Driver Deleted Successfully.');
    }

    public function import(Request $request)
    {
        try {
            $request->validate([
                'file' => 'required|mimes:csv,txt',
            ]);

            $file = $request->file('file');
            $fileName = time() . '_' . $file->getClientOriginalName();
            $destinationPath = public_path('docs/drivers');
            $file->move($destinationPath, $fileName);
            $csvData = array_map('str_getcsv', file($destinationPath . '/' . $fileName));
            $expectedHeaders = ['FirstName', 'LastName', 'Email', 'Phone', 'License #', 'Hire Date', 'Profile Status'];

            if ($csvData && count($csvData) > 0 && $csvData[0] != $expectedHeaders) {
                throw new \Exception('Invalid CSV format.');
            }

            array_shift($csvData);
            $lastElement = end($csvData);

            while ($lastElement === [null]) {
                array_pop($csvData);
                $lastElement = end($csvData);
            }

            foreach ($csvData as $row) {
                Driver::create([
                    'NAME' => $row[0] . ' ' . $row[1],
                    'FIRST_NAME' => $row[0],
                    'LAST_NAME' => $row[1],
                    'EMAIL' => $row[2],
                    'PHONE' => $row[3],
                    'LICENCE_NUMBER' => $row[4],
                    'START_DATE' => date('Y-m-d', strtotime($row[5])),
                    'STATUS' => $row[6],
                    'CREATED_DATE' => date('Y-m-d H:i:s'),
                    'CREATED_BY' => getAuthMemberId(),
                    'mem_id' => getAuthMemberId(),
                ]);
            }

            return redirect()->back()->with('success', 'Records Imported Successfully.');
        } catch (\Exception $e) {
            return redirect()->back()->with('error', $e->getMessage());
        }
    }
}
