<?php

namespace App\Http\Controllers;

use App\Models\FuelCard;
use Illuminate\Http\Request;
use App\DataTables\FuelCardDataTable;
use LDAP\Result;

class FuelCardController extends Controller
{
    public function index(FuelCardDataTable $dataTable)
    {
        return $dataTable->render('fuelCard.index');
    }

    public function newIndex(FuelCardDataTable $dataTable)
    {
        return $dataTable->render('fuelCard.newIndex');
    }

    public function edit($id)
    {
        $fuelCard = FuelCard::find($id);
        return view('fuelCard.edit', compact('fuelCard'));
    }

    public function update(Request $request)
    {
        $this->validate($request, [
            'CARD_STATUS' => 'required',
            'CARD_HOLDER' => 'required',
            'PUNIT_ID' => 'required',
            'DRIVER_ID' => 'required'
        ]);

        FuelCard::where('ID', $request->ID)->update([
            'CARD_STATUS' => $request->CARD_STATUS,
            'CARD_HOLDER' => $request->CARD_HOLDER,
            'PUNIT_ID' => $request->PUNIT_ID,
            'DRIVER_ID' => $request->DRIVER_ID,
        ]);

        $fuelCard = FuelCard::where('ID', $request->ID)->first();

        try {
            if (!empty($fuelCard->CARD_NO)) {
                $cardNumber = $fuelCard->CARD_NO;
                $pin = $request->get('PIN');
                if (!empty($pin)) {
                    $efsdata = [
                        'clientID' => getEFSClientID(),
                        'cardNumber' => $cardNumber,
                        'newPin' => $pin
                    ];
                    $response = CallAPI('POST', 'https://tms.tmdashboard.com/api/esfAPIS/setcardpin_api.php', $efsdata);
                }

                $data = array('clientID' => getEFSClientID(), 'cardNumber' => $fuelCard->CARD_NO);
                $return_data = CallAPI('POST', 'https://tms.tmdashboard.com/api/esfAPIS/getcard_api.php', $data);
                $clean_xml = str_ireplace(['soapenv:', 'SOAP:'], '', $return_data);

                $xml = simplexml_load_string($clean_xml);
                $response = preg_replace("/(<\/?)(\w+):([^>]*>)/", "$1$2$3", $return_data);
                $xml = new \SimpleXMLElement($response);
                $body = $xml->xpath('//soapenvBody')[0];
                $array_getCard = json_decode(json_encode((array) $body), true);
                $status = $fuelCard->CARD_STATUS;

                if (!empty($array_getCard['ns5getCardResponse']['result']['header'])) {
                    $header = $array_getCard['ns5getCardResponse']['result']['header'];
                } else {
                    $header = array(
                        'companyXRef' => '',
                        'handEnter' => '',
                        'infoSource' => 'BOTH',
                        'limitSource' => '',
                        'locationOverride' => '',
                        'locationSource' => '',
                        'overrideAllLocations' => '',
                        'originalStatus' => '',
                        'payrollStatus' => 'FOLLOWS',
                        'override' => 0,
                        'policyNumber' => '0',
                        'status' => $status,
                        'timeSource' => 'BOTH',
                        'lastUsedDate' => '',
                        'lastTransaction' => '',
                        'payrollUse' => 'N',
                        'payrollAtm' => 'DISALLOW',
                        'payrollChk' => 'DISALLOW',
                        'payrollAch' => 'DISALLOW',
                        'payrollWire' => 'DISALLOW',
                        'payrollDebit' => 'DISALLOW',
                    );
                }

                $data = array('clientID' => getEFSClientID(), 'cardNumber' => $fuelCard->CARD_NO, 'status' => $status, 'NAME' => $fuelCard->CARD_HOLDER, 'UNIT' => $fuelCard->PUNIT_ID, 'DRID' => $fuelCard->DRIVER_ID, 'companyXRef' => $header['companyXRef'], 'handEnter' => $header['handEnter'], 'infoSource' => $header['infoSource'], 'limitSource' => $header['limitSource'], 'locationOverride' => $header['locationOverride'], 'locationSource' => $header['locationSource'], 'overrideAllLocations' => $header['overrideAllLocations'], 'originalStatus' => '', 'payrollStatus' => $header['payrollStatus'], 'override' => $header['override'], 'policyNumber' => $header['policyNumber'], 'timeSource' => $header['timeSource'], 'lastUsedDate' => '', 'lastTransaction' => $header['lastTransaction'], 'payrollUse' => $header['payrollUse'], 'payrollAtm' => $header['payrollAtm'], 'payrollChk' => $header['payrollChk'], 'payrollAch' => $header['payrollAch'], 'payrollWire' => $header['payrollWire'], 'payrollDebit' => $header['payrollDebit']);

                $data_setcard = CallAPI('POST', 'https://tms.tmdashboard.com/api/esfAPIS/setcard_api.php', $data);
                $data_return = json_decode($data_setcard, true);
            }
        } catch (\Exception $e) {
            return redirect()->back()->with('error', $e->getMessage());
        }

        return redirect()->route('fuelCards.index')->with('success', 'Fuel Card Updated Successfully.');
    }

    public function get_efs_data($id)
    {
        $fuelCard = FuelCard::find($id);
        if (!empty($fuelCard->CARD_NO)) {
            try {
                // Real Time EFS Data
                $efs_data = array('clientID' => getEFSClientID(), 'cardNumber' => $fuelCard->CARD_NO);
                $return_data = CallAPI('POST', 'https://tms.tmdashboard.com/api/esfAPIS/getcard_api.php', $efs_data);
                $clean_xml = str_ireplace(['soapenv:', 'SOAP:'], '', $return_data);
                $xml = simplexml_load_string($clean_xml);
                $response = preg_replace("/(<\/?)(\w+):([^>]*>)/", "$1$2$3", $return_data);
                $xml = new \SimpleXMLElement($response);
                $body = $xml->xpath('//soapenvBody')[0];
                $array_getCard = json_decode(json_encode((array) $body), true);
                return $array_getCard['ns5getCardResponse']['result'];
            } catch (\Exception $e) {
                return $e->getMessage();
            }
        }
        return $array_getCard['ns5getCardResponse']['result'] = [];
    }

    public function efs_rejected_transactions($id)
    {
        $fuelCard = FuelCard::find($id);
        if (!empty($fuelCard->CARD_NO)) {
            try {
                // EFS Rejected Transactions of Last 60 days
                $current_datetime = date('Y-m-d' . '\T00:00:00');
                $next_datetime = date('Y-m-d' . '\T23:59:59', strtotime("-60 days"));
                $startDate = gmdate('c', strtotime($next_datetime));
                $endDate = gmdate('c', strtotime($current_datetime));
                $data = array('clientID' => getEFSClientID(), 'startDate' => $startDate, 'endDate' => $endDate, 'cardNumber' => $fuelCard->CARD_NO);
                $data_gettrans = CallAPI('POST', 'https://tms.tmdashboard.com/api/esfAPIS/getrejecttransaction_api.php', $data);
                $array_data = json_decode($data_gettrans, true);
                return $array_data['ns5getTranRejectsResponse']['result']['value'];
            } catch (\Exception $e) {
                return $array_data['ns5getTranRejectsResponse']['result']['value'] = [];
            }
        }
        return $array_data['ns5getTranRejectsResponse']['result']['value'] = [];
    }

    public function efs_transactions(Request $request)
    {
        $current_datetime = str_replace('+00:00', '', gmdate('c', strtotime(date("Y-m-d\Th:i:s"))));
        $current_datetime = date($request->begin_date . '\T00:00:00');
        $next_datetime = str_replace('+00:00', '', gmdate('c', strtotime("+1day", strtotime(date("Y-m-d\Th:i:s")))));
        $next_datetime = date($request->end_date . '\T23:59:59');

        $efsData = [
            'clientID' => getEFSClientID(),
            'begDate' => $current_datetime,
            'endDate' => $next_datetime,
        ];
        $data_gettrans = CallAPI('POST', 'https://tms.tmdashboard.com/api/esfAPIS/gettransactionsExtLoc_api.php', $efsData);
        $array_data = json_decode($data_gettrans, true);
        if (isset($array_data['ns5getTransExtLocResponse']['result']['value']) && !empty($array_data['ns5getTransExtLocResponse']['result']['value'])) {
            return $array_data['ns5getTransExtLocResponse']['result']['value'];
        }
    }
}
