<?php

namespace App\Http\Controllers;

use DB;
use Carbon\Carbon;
use Dompdf\Dompdf;
use Dompdf\Options;
use App\Models\Miles;
use App\Models\Gallon;
use Illuminate\Http\Request;
use App\Models\TransactionReport;
use App\Models\CheckReportHistory;
use App\Models\IFTATaxRateInterest;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Facades\Http;
use App\DataTables\GenerateReportDataTable;
use App\DataTables\CheckReportHistoryDataTable;

class GenerateIFTAController extends Controller
{
    public function index(GenerateReportDataTable $dataTable, Request $request)
    {
        $milesUnit = Miles::where('CARRIER_SUFBLEET', getAuthMemberId())->pluck('UNIT')->toArray();
        $combinedUnits = array_filter($milesUnit, function ($value) {
            return $value !== null;
        });
        $milesUnits = array_unique($combinedUnits);

        $gallonsUnit = TransactionReport::where('Sub Fleet', getAuthMemberId())->pluck('Unit')->toArray();
        $combinedUnits = array_filter($gallonsUnit, function ($value) {
            return $value !== null;
        });
        $gallonsUnits = array_unique($combinedUnits);

        $units = array_unique(array_merge($milesUnits, $gallonsUnits));

        return $dataTable->render('IFTA.generateIFTA.index', compact('units'));
    }

    public function checkReport($year = null, $quarter = null, $carrier = null, $units = null, $mode = null)
    {
        $unitValues = $units;
        $interest = .05;
        $interest_rate = round(($interest + .02) / 12, 4);
        $filing_date = date("Y-m-d");

        if ($quarter == 1) {
            $date_range = "January 1st - March 31st";
            $due_date = "April 30";
            $due_date = $due_date . ", " . $year;
            $due_date_cal = "04-30";
            $date1 = $year . "-" . $due_date_cal;
        }

        if ($quarter == 2) {
            $date_range = "April 1st - June 30th";
            $due_date = "July 31";
            $due_date = $due_date . ", " . $year;
            $due_date_cal = "07-31";
            $date1 = $year . "-" . $due_date_cal;
        }

        if ($quarter == 3) {
            $date_range = "July 1st - September 30th";
            $due_date = "October 31";
            $due_date = $due_date . ", " . $year;
            $due_date_cal = "10-31";
            $date1 = $year . "-" . $due_date_cal;
        }

        if ($quarter == 4) {
            $date_range = "October 1st - December 31st";
            $due_date = "January 31";
            $endyear = $year + 1;
            $due_date = $due_date . ", " . $endyear;
            $due_date_cal = "01-31";
            $date1 = $endyear . "-" . $due_date_cal;
        }

        $date2 = $filing_date;

        $dateDiff = $this->dateDiffInDays($date1, $date2);
        if ($dateDiff > 0) {
            $dateDiff = $dateDiff;
        }
        if ($dateDiff <= 0) {
            $dateDiff = 0;
        }
        $interest_months = ceil($dateDiff / 30);
        $startMonth = ($quarter - 1) * 3 + 1;
        $startDate = Carbon::createFromDate($year, $startMonth, 1)->startOfMonth();
        $endDate = $startDate->copy()->addMonths(3)->subDay()->endOfMonth();
        $startDate = $startDate->toDateString();
        $endDate = $endDate->toDateString();

        $gallonsSql = "(SELECT `Location State/Province` AS STATE,
                    SUM(`Quantity`) AS GALLONS,
                    NULL AS MILES,
                    NULL AS totalMiles,
                    '" . $quarter . "' AS QUARTER,
                    '" . $year . "' AS YEAR,
                    '" . $carrier . "' AS CARRIER_SUFBLEET
            FROM DailyTrans
            WHERE `Sub Fleet` = '" . $carrier . "' AND `Product Code` != 'DEFD'";
        if (!empty($unitValues)) {
            $gallonsSql .= "AND Unit IN (" . ($unitValues !== null ? $unitValues : "''") . ")";
        }
        $gallonsSql .= "AND `Transaction Date` BETWEEN '" . $startDate . "' AND '" . $endDate . "'
            GROUP BY `Location State/Province`)

           UNION

           (SELECT STATE,
                   NULL AS GALLONS,
                   NULL AS MILES,
                   SUM(MILES) AS totalMiles,
                   QUARTER,
                   YEAR,
                   '" . $carrier . "' AS CARRIER_SUFBLEET
            FROM IFTA_MILES
            WHERE CARRIER_SUFBLEET = '" . $carrier . "'
              AND QUARTER = '" . $quarter . "'
              AND YEAR = '" . $year . "'";

        if (!empty($unitValues)) {
            $gallonsSql .= "AND UNIT IN (" . ($unitValues !== null ? $unitValues : "''") . ")";
        }

        $gallonsSql .= "GROUP BY STATE, QUARTER, YEAR)";

        $gallonsSql = "SELECT subquery.STATE, subquery.QUARTER, subquery.YEAR,
            SUM(COALESCE(subquery.GALLONS, 0)) AS totalGallons,
            SUM(COALESCE(subquery.totalMiles, 0)) AS totalMiles,
            ifta_rates_new.Rate AS RATE_US
     FROM ($gallonsSql) AS subquery
     LEFT JOIN ifta_rates_new ON subquery.STATE = ifta_rates_new.State
                             AND subquery.QUARTER = ifta_rates_new.Quarter
                             AND subquery.YEAR = ifta_rates_new.Year
     WHERE subquery.CARRIER_SUFBLEET = ?
       AND subquery.QUARTER = ?
       AND subquery.YEAR = ?
     GROUP BY subquery.STATE, subquery.QUARTER, subquery.YEAR";
        $rowData = DB::select($gallonsSql, [$carrier, $quarter, $year]);

        $total_gal = 0.00;
        $total_miles = 0.00;

        if (!empty($rowData)) {
            foreach ($rowData as $row) {
                $total_gal += $row->totalGallons;
                $total_miles += $row->totalMiles;
            }
        }

        $mpg = ($total_gal != 0) ? round(($total_miles / $total_gal), 2) : null;
        $ifta_tax_rate_interest = IFTATaxRateInterest::where('QTR', $quarter)->where('YEAR', $year)->first();
        $data = [
            'date_range' => $date_range,
            'due_date' => $due_date,
            'year' => $year,
            'quarter' => $quarter,
            'carrier' => $carrier,
            'filing_date' => $filing_date,
            'dateDiff' => $dateDiff,
            'interest_rate' => $interest_rate,
            'interest_months' => $interest_months,
            'total_miles_start' => $total_miles,
            'total_gal_start' => $total_gal,
            'mpg' => $mpg,
            'ifta_tax_rate_interest' => $ifta_tax_rate_interest,
            'unitValues' => $unitValues,
            'date1' => $date1
        ];
        $showButtons = true;

        if (!empty($mode) && $mode == 'downloadPDF') {
            $showButtons = false;
            $html = view('IFTA.generateIFTA.checkReport', ['data' => $data, 'rowData' => $rowData, 'showButtons' => $showButtons])->renderSections()['content'];
            $html = trim($html);
            return $html;
        } else {
            return view('IFTA.generateIFTA.checkReport', ['data' => $data, 'rowData' => $rowData, 'showButtons' => $showButtons]);
        }
    }

    public function dateDiffInDays($date1, $date2)
    {
        $diff = strtotime($date2) - strtotime($date1);
        return round($diff / 86400);
    }

    public function downloadPDF($year = null, $quarter = null, $carrier = null, $units = null, $mode = null)
    {
        $html = $this->checkReport($year, $quarter, $carrier, $units, 'downloadPDF');

        $options = new Options();
        $options->set('isHtml5ParserEnabled', true);
        $options->set('isPhpEnabled', true);

        $dompdf = new Dompdf($options);

        $dompdf->loadHtml($html);

        $dompdf->setPaper('A2', 'portrait');

        $dompdf->render();

        $filename = time() . '.pdf';

        $pdfFilePath = public_path('docs') . DIRECTORY_SEPARATOR . $filename;

        File::put($pdfFilePath, $dompdf->output());

        if (!empty($mode) && $mode == 'save') {
            return $filename;
        } else {
            return response()->download($pdfFilePath, 'checkReport.pdf');
        }
    }

    public function checkReportIndex(CheckReportHistoryDataTable $dataTable)
    {
        return $dataTable->render('IFTA.generateIFTA.index');
    }

    public function saveReport(Request $request)
    {
        if (!empty($request->carrier) && !empty($request->QUARTER) && !empty($request->YEAR)) {
            $filename = $this->downloadPDF($request->YEAR, $request->QUARTER, $request->carrier, $request->UNIT, 'save');
            CheckReportHistory::create([
                'year' => $request->YEAR,
                'quarter' => $request->QUARTER,
                'mem_id' => $request->carrier,
                'document' => $filename,
                'created_at' => date('Y-m-d H:i:s')
            ]);
            return redirect()->route('ifta.generateIFTA.index')->with('success', 'Report Saved Successfully.');
        }

        return redirect()->route('ifta.generateIFTA.index')->with('error', 'No Generated Data Found');
    }

    public function delete($id)
    {
        $CheckReportHistory = CheckReportHistory::find($id);
        if ($CheckReportHistory->delete()) {
            return redirect()->back()->with('success', 'History Deleted Successfully.');
        } else {
            return redirect()->back()->with('error', 'Something Went Wrong.');
        }
    }
}
