<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\Miles;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Response;
use App\DataTables\IFTAMilesDataTable;
use Illuminate\Support\Facades\DB;
class IFTAMilesController extends Controller
{
    public function index(IFTAMilesDataTable $dataTable,Request $request)
    {
        $year = !empty($request->year) ? $request->year : date('Y');
        $q1start_date = "{$year}-01-01";
        $q1end_date = "{$year}-03-31";
        $q2start_date = "{$year}-04-01";
        $q2end_date = "{$year}-06-30";
        $q3start_date = "{$year}-07-01";
        $q3end_date = "{$year}-09-30";
        $q4start_date = "{$year}-10-01";
        $q4end_date = "{$year}-12-31";

         $q1Gallons = Miles::select(DB::raw('`STATE`,SUM(`MILES`) as gallons'))
        ->where('CARRIER_SUFBLEET', getAuthMemberId())           
        ->where('QUARTER', '1')
        ->where('YEAR', $year)
        ->groupBy('STATE')->get()->sum('gallons');
        // print_r($q1Gallons);die;

        $q2Gallons = Miles::select(DB::raw('`STATE`,SUM(`MILES`) as gallons'))
        ->where('CARRIER_SUFBLEET', getAuthMemberId())           
        ->where('QUARTER', '2')
        ->where('YEAR', $year)
        ->groupBy('STATE')->get()->sum('gallons');

        $q3Gallons = Miles::select(DB::raw('`STATE`,SUM(`MILES`) as gallons'))
        ->where('CARRIER_SUFBLEET', getAuthMemberId())           
        ->where('QUARTER', '3')
        ->where('YEAR', $year)
        ->groupBy('STATE')->get()->sum('gallons');

        $q4Gallons = Miles::select(DB::raw('`STATE`,SUM(`MILES`) as gallons'))
        ->where('CARRIER_SUFBLEET', getAuthMemberId())           
        ->where('QUARTER', '4')
        ->where('YEAR', $year)
        ->groupBy('STATE')->get()->sum('gallons');
     

        $milesUnit = Miles::where('CARRIER_SUFBLEET', getAuthMemberId())->pluck('UNIT')->toArray();
        $combinedUnits = array_filter($milesUnit, function ($value) {
            return $value !== null;
        });
        $units = array_unique($combinedUnits);

        return $dataTable->render("IFTA.miles.index", compact('q1Gallons', 'q2Gallons', 'q3Gallons', 'q4Gallons','units'));
    }

    public function add()
    {
        return view('IFTA.miles.add');
    }

    public function store(Request $request)
    {
        $this->validate($request, [
            'STATE' => 'required',
            'MILES' => 'required',
            'QUARTER' => 'required',
            'YEAR' => 'required',
        ], [
            'STATE.required' => 'STATE is Required.',
            'MILES.required' => 'MILES is Required.',
            'QUARTER.required' => 'QUARTER is Required.',
            'YEAR.required' => 'YEAR is Required.',
        ]);

        $miles = new Miles();
        $miles->STATE = $request->STATE;
        $miles->MILES = $request->MILES;
        $miles->UNIT = $request->UNIT;
        $miles->CARRIER_SUFBLEET = getAuthMemberId();
        $miles->QUARTER = $request->QUARTER;
        $miles->YEAR = $request->YEAR;
        $miles->CLIENT_ID = 0;
        $miles->CREATED_TIME = date('H:i:s');
        $miles->CREATED_DATE = date('Y-m-d');

        $miles->save();

        return redirect()->route('ifta.miles.index')->with('success', 'Miles Created Successfully.');
    }

    public function edit($milesId)
    {
        $miles = Miles::find($milesId);
        return view('IFTA.miles.edit', compact('miles'));
    }

    public function update(Request $request)
    {
        $this->validate($request, [
            'STATE' => 'required',
            'MILES' => 'required',
            'QUARTER' => 'required',
            'YEAR' => 'required',
        ], [
            'STATE.required' => 'STATE is Required.',
            'MILES.required' => 'MILES is Required.',
            'QUARTER.required' => 'QUARTER is Required.',
            'YEAR.required' => 'YEAR is Required.',
        ]);

        $miles = Miles::find($request->milesId);
        $miles->STATE = $request->STATE;
        $miles->MILES = $request->MILES;
        $miles->UNIT = $request->UNIT;
        $miles->CARRIER_SUFBLEET = getAuthMemberId();
        $miles->QUARTER = $request->QUARTER;
        $miles->YEAR = $request->YEAR;
        $miles->CLIENT_ID = 0;
        $miles->CREATED_TIME = date('H:i:s');
        $miles->CREATED_DATE = date('Y-m-d');

        $miles->update();

        return redirect()->route('ifta.miles.index')->with('success', 'Miles Updated Successfully.');
    }

    public function delete($milesId)
    {
        $miles = Miles::find($milesId);
        $miles->delete();

        return redirect()->route('ifta.miles.index')->with('success', 'Miles Deleted Successfully.');
    }

    public function import(Request $request)
    {
        try {
            $request->validate([
                'docs' => 'required|mimes:csv,txt',
            ]);

            $existingRecord = Miles::where('QUARTER', $request->QUARTER)->where('YEAR', $request->YEAR)
                ->where('CARRIER_SUFBLEET', getAuthMemberId())
                ->first();

            if ($existingRecord) {
                return redirect()->back()->with('error', 'Record Already Exists');
            }

            $file = $request->file('docs');

            $fileName = time() . '_' . $file->getClientOriginalName();
            $destinationPath = public_path('docs/miles');

            $file->move($destinationPath, $fileName);

            $csvData = array_map('str_getcsv', file($destinationPath . '/' . $fileName));

            $expectedHeaders = ['STATE', 'MILES', 'QUARTER', 'YEAR', 'UNIT'];
            if ($csvData && count($csvData) > 0 && $csvData[0] != $expectedHeaders) {
                throw new \Exception('Invalid CSV format.');
            }
            array_shift($csvData);
            $lastElement = end($csvData);

            while ($lastElement === [null]) {
                array_pop($csvData);
                $lastElement = end($csvData);
            }

            foreach ($csvData as $row) {
                Miles::create([
                    'STATE' => $row[0],
                    'MILES' => $row[1],
                    'QUARTER' => $row[2],
                    'YEAR' => $row[3],
                    'UNIT' => $row[4],
                    'CLIENT_ID' => 0,
                    'CREATED_TIME' => date('H:i:s'),
                    'CREATED_DATE' => date('Y-m-d'),
                    'CARRIER_SUFBLEET' => getAuthMemberId(),
                ]);
            }
            return redirect()->back()->with('success', 'Records Imported Successfully.');
        } catch (\Exception $e) {
            return redirect()->back()->with('error', $e->getMessage());
        }
    }

    public function multipleDelete(Request $request)
    {
        Miles::whereIn('IFTA_ID', $request->input('selectedMiles'))->delete();
        return redirect()->route('ifta.miles.index')->with('success', 'Selected Miles Deleted Successfully.');
    }
}
