<?php

namespace App\Http\Controllers;

use App\Models\Invoice;
use App\Models\InvoicePayment;
use Illuminate\Http\Request;
use App\Models\PaymentMethod;
use App\Models\CarrierProfile;
use Illuminate\Support\Facades\DB;
use App\DataTables\InvoiceDataTable;

class InvoiceController extends Controller
{
    public function index(InvoiceDataTable $dataTable)
    {
        return $dataTable->render('invoices.index');
    }

    public function newIndex(InvoiceDataTable $dataTable)
    {
        return $dataTable->render('invoices.newIndex');
    }

    public function view($id)
    {
        $main_loop = Invoice::where('id', $id)->first();
        $row_loop = CarrierProfile::where('SUBFLEET', getAuthMemberId())->first();
        $paid_amounts = InvoicePayment::where('invoice_id', $id)->where('status', 'succeeded')->get();
        $sum_of_paid_amounts = $paid_amounts->sum('amount');
        $start = date("Y-m-d", strtotime($main_loop->start_date));
        $end = date("Y-m-d", strtotime($main_loop->end_date));
        $subfleet = $main_loop->subfleet;
        $invNo = $main_loop->inv_no;
        $invNo = sprintf('%04d', $invNo);
        $totalFee = 0;
        $day = date("D", strtotime($start));
        $AM_REP = date("m/d/Y", strtotime($start)) . " - " . date("m/d/Y", strtotime($end));
        $sql2 = "SELECT
            DailyTrans.`Transaction ID`,
            DailyTrans.`Transaction Date`,
            DailyTrans.Invoice,
            DailyTrans.Unit,
            DailyTrans.`Driver ID`,
            DailyTrans.`Driver Name`,
            DailyTrans.`Location ID`,
            DailyTrans.`Location Name`,
            DailyTrans.`Location City`,
            DailyTrans.`Location State/Province`,
            DailyTrans.`Transaction Fee`,
            DailyTrans.`Product Code`,
            DailyTrans.`Price Per Unit`,
            DailyTrans.Retail_PPU,
            DailyTrans.Quantity,
            DailyTrans.Amount,
            DailyTrans.`Card Number`,
            DailyTrans.Chain_Desc,
            DailyTrans.subfleet,
            DailyTrans.PAID_RETAIL,
            DailyTrans.PAID_RETAIL_DATE,
            DailyTrans.PAID_RETAIL_REF,
            DailyTrans.TID,
            (DailyTrans.`Transaction Fee` + DailyTrans.Amount) AS TotalAmount,
            DailyTrans.ChainID,
            CARRIER_PROFILE.BILL_TO,
            CARRIER_PROFILE.EMAIL,
            CARRIER_PROFILE.NAME,
            CARRIER_PROFILE.CONTRACT_ID,
            DailyTrans.`Sub Fleet`,
            DailyTrans.`Discount Type`,
            DailyTrans.`Direct Bill / Funded`
            FROM DailyTrans
            LEFT JOIN `CARRIER_PROFILE` ON CARRIER_PROFILE.`SUBFLEET` =DailyTrans.`Sub Fleet`
            WHERE
            `Transaction Date` BETWEEN '" . date("Y-m-d", strtotime($start)) . "'  AND '" . date("Y-m-d", strtotime($end)) . "'
            AND DailyTrans.`Sub Fleet`='$subfleet'
            ORDER BY DailyTrans.`Transaction Date` DESC";
        $num_rows = DB::select(DB::raw($sql2));

        $cards = [];

        \Stripe\Stripe::setApiKey(config('custom_key')['STRIPE_SECRET']);

        $email = $row_loop->EMAIL;
        $stripe_customer_id = $row_loop->stripe_customer_id;
        $carrier_name = $row_loop->CONTACT_NAME;

        if (isset($stripe_customer_id) && $stripe_customer_id != "") {
        } else {
            $customer = \Stripe\Customer::create(
                array(
                    "name" => $carrier_name,
                    "email" => strip_tags(trim($email))
                )
            );
            $stripe_customer_id = $customer->id;
            CarrierProfile::where('SUBFLEET', 'like', '%' . getAuthMemberId() . '%')->update([
                'stripe_customer_id' => $stripe_customer_id
            ]);
        }

        $userpwd = config('custom_key')['STRIPE_SECRET'] . ":";
        $ch = curl_init();

        curl_setopt($ch, CURLOPT_URL, 'https://api.stripe.com/v1/customers/' . $stripe_customer_id . '/cards');
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'GET');
        curl_setopt($ch, CURLOPT_USERPWD, $userpwd);

        $headers = array();
        $headers[] = 'Content-Type: application/x-www-form-urlencoded';
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

        $result = curl_exec($ch);
        $cards = json_decode($result);
        $cards = !empty($cards) ? $cards->data : [];

        return view('invoices.view', compact('main_loop', 'row_loop', 'invNo', 'AM_REP', 'num_rows', 'cards', 'paid_amounts', 'sum_of_paid_amounts'));
    }

    public function download($id)
    {
        $main_id = $id;
        $main_loop = Invoice::find($main_id);
        $subfleet = $main_loop->subfleet;
        $start = date("Y-m-d", strtotime($main_loop->start_date));
        $end = date("Y-m-d", strtotime($main_loop->end_date));
        $sql2 = "SELECT
            DailyTrans.`Transaction ID`,
            DailyTrans.`Transaction Date`,
            DailyTrans.Invoice,
            DailyTrans.Unit,
            DailyTrans.`Driver ID`,
            DailyTrans.`Driver Name`,
            DailyTrans.`Location ID`,
            DailyTrans.`Location Name`,
            DailyTrans.`Location City`,
            DailyTrans.`Location State/Province`,
            DailyTrans.`Transaction Fee`,
            DailyTrans.`Product Code`,
            DailyTrans.`Price Per Unit`,
            DailyTrans.Retail_PPU,
            DailyTrans.Quantity,
            DailyTrans.Amount,
            DailyTrans.`Card Number`,
            DailyTrans.Chain_Desc,
            DailyTrans.subfleet,
            DailyTrans.PAID_RETAIL,
            DailyTrans.PAID_RETAIL_DATE,
            DailyTrans.PAID_RETAIL_REF,
            DailyTrans.TID,
            (DailyTrans.`Transaction Fee` + DailyTrans.Amount) AS TotalAmount,
            DailyTrans.ChainID,
            CARRIER_PROFILE.BILL_TO,
            CARRIER_PROFILE.EMAIL,
            CARRIER_PROFILE.NAME,
            CARRIER_PROFILE.CONTRACT_ID,
            DailyTrans.`Sub Fleet`,
            DailyTrans.`Discount Type`,
            DailyTrans.`Direct Bill / Funded`
            FROM DailyTrans
            LEFT JOIN `CARRIER_PROFILE` ON CARRIER_PROFILE.`SUBFLEET` =DailyTrans.`Sub Fleet`
            WHERE
            `Transaction Date` BETWEEN '" . date("Y-m-d", strtotime($start)) . "'  AND '" . date("Y-m-d", strtotime($end)) . "'
            AND DailyTrans.`Sub Fleet`='$subfleet'
            ORDER BY DailyTrans.`Transaction Date` DESC";
        $data = DB::select(DB::raw($sql2));

        $invNo = $main_loop->inv_no;
        $invNo = sprintf('%04d', $invNo);
        $day = date("D", strtotime($start));
        $name = "data_" . $day . "_" . $subfleet;
        $onlynamecsv = $name . "_" . date("m_d_Y_H_i") . ".csv";

        $response = response()->stream(
            function () use ($data) {
                $fp = fopen('php://output', 'w');
                $delimiter = ",";
                $fpHeader = [
                    '',
                    'Transaction ID',
                    'Card Number',
                    'Transaction Date',
                    'Invoice',
                    'Unit',
                    'Driver ID',
                    'Driver Name',
                    'Location ID',
                    'Location Name',
                    'Location City',
                    'Location State/Province',
                    'Transaction Fee',
                    'Product Code',
                    'Price Per Unit',
                    'Quantity',
                    'Amount',
                    'Discount Type',
                    'Direct Bill / Funded'
                ];
                fputcsv($fp, $fpHeader, $delimiter);
                fflush($fp);

                $fptotalTransactionFee = 0;
                $fptotalQuantity = 0;
                $fptotalAmount = 0;
                $number = 0;

                foreach ($data as $row) {
                    $fprow = [
                        '',
                        $row->{'Transaction ID'},
                        $row->{'Card Number'},
                        $row->{'Transaction Date'},
                        $row->{'Invoice'},
                        $row->{'Unit'},
                        $row->{'Driver ID'},
                        $row->{'Driver Name'},
                        $row->{'Location ID'},
                        $row->{'Location Name'},
                        $row->{'Location City'},
                        $row->{'Location State/Province'},
                        $row->{'Transaction Fee'},
                        $row->{'Product Code'},
                        $row->{'Price Per Unit'},
                        $row->{'Quantity'},
                        $row->{'Amount'},
                        $row->{'Discount Type'},
                        $row->{'Direct Bill / Funded'}
                    ];
                    fputcsv($fp, $fprow, $delimiter);
                    fflush($fp);
                    $fptotalTransactionFee += $row->{'Transaction Fee'};
                    $fptotalQuantity += $row->{'Quantity'};
                    $fptotalAmount += $row->{'Amount'};
                    $number++;
                }

                $fprow = [
                    'Grand Totals:',
                    '',
                    '',
                    '',
                    '',
                    '',
                    '',
                    '',
                    '',
                    '',
                    '',
                    '',
                    $fptotalTransactionFee,
                    '',
                    '',
                    $fptotalQuantity,
                    $fptotalAmount,
                    '',
                    ''
                ];
                fputcsv($fp, $fprow, $delimiter);
                fflush($fp);
                fclose($fp);
            },
            200,
            [
                'Content-Type' => 'text/csv',
                'Content-Disposition' => 'attachment; filename=' . $onlynamecsv,
            ]
        );

        return $response;
    }

    public function changeStatus(Request $request)
    {
        Invoice::where('id', $request->invoiceId)->update([
            'is_paid' => $request->status
        ]);

        return response()->json(['status' => true, 'message' => 'Invoice Updated Successfully.']);
    }

    public function payNow($id, $amount, $card)
    {
        $invoice = Invoice::whereRaw("MD5(id) = '$id'")->first();
        $carrierInfo = CarrierProfile::where('SUBFLEET', getAuthMemberId())->first();
        $cards = PaymentMethod::where('mem_id', getAuthMemberId())->where('type', 'CC')->get();
        if ($cards->isEmpty()) {
            return redirect()->route('myWallet.index')->with('error', 'Please Add Card First');
        } else {
            return view('invoices.payNow', compact('invoice', 'carrierInfo', 'card', 'amount'));
        }
    }

    public function chargePayment(Request $request)
    {
        $amount = $request->amount;
        $invoice = Invoice::where('id', $request->invoiceId)->first();
        $invoiceAmount = $invoice->amount;
        $payment_type = 'F';
        if ($invoiceAmount < $amount) {
            $payment_type = 'P';
        }
        try {
            $carrierInfo = CarrierProfile::where('SUBFLEET', getAuthMemberId())->first();
            \Stripe\Stripe::setApiKey(config('custom_key')['STRIPE_SECRET']);

            $email = $carrierInfo->EMAIL;
            $stripe_customer_id = $carrierInfo->stripe_customer_id;
            $carrier_name = $carrierInfo->CONTACT_NAME;

            if (isset($stripe_customer_id) && $stripe_customer_id != "") {
            } else {
                $customer = \Stripe\Customer::create(
                    array(
                        "name" => $carrier_name,
                        "email" => strip_tags(trim($email))
                    )
                );
                $stripe_customer_id = $customer->id;
                CarrierProfile::where('SUBFLEET', 'like', '%' . getAuthMemberId() . '%')->update([
                    'stripe_customer_id' => $stripe_customer_id
                ]);
            }

            $finalAMount = $amount + $request->fee;
            $params = [
                'payment_method_types' => 'card',
                'amount' => $finalAMount * 100,
                'currency' => 'usd',
                'customer' => $stripe_customer_id,
                'payment_method' => $request->cardId,
                'confirm' => true,
                'receipt_email' => $carrierInfo->EMAIL
            ];
            $postData = http_build_query($params);
            $userpwd = config('custom_key')['STRIPE_SECRET'] . ":";

            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, 'https://api.stripe.com/v1/payment_intents');
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'POST');
            curl_setopt($ch, CURLOPT_HTTPHEADER, [
                'Content-Type: application/x-www-form-urlencoded',
            ]);
            curl_setopt($ch, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
            curl_setopt($ch, CURLOPT_USERPWD, $userpwd);
            curl_setopt($ch, CURLOPT_POSTFIELDS, $postData);

            $response = curl_exec($ch);
            $result = json_decode($response);

            $serializedResponse = serialize($result);

            curl_close($ch);

            InvoicePayment::create([
                'invoice_id' => $request->invoiceId,
                'amount' => $request->amount,
                'fee' => $request->fee,
                'payment_type' => $payment_type,
                'status' => $result->status,
                'transaction_id' => $result->id,
                'stripe_response' => $serializedResponse,
                'created_date' => date('Y-m-d H:i:s'),
            ]);

            $is_paid = 'N';
            if ($payment_type == 'P') {
                $is_paid = 'N';
            } else {
                $is_paid = 'Y';
            }

            if ($result->status == 'succeeded') {
                Invoice::where('id', $request->invoiceId)->update([
                    'is_paid' => $is_paid
                ]);
            }

            return redirect()->route('invoices.view', $request->invoiceId)->with('success', 'Invoice Paid Successfully');
        } catch (\Exception $e) {
            return redirect()->back()->with('error', $e->getMessage());
        }
    }
}
