<?php

namespace App\Http\Controllers;

use App\Models\TransactionReport;
use Illuminate\Http\Request;
use App\DataTables\InNetworkGallonsDataTable;
use App\DataTables\PilotFJGallonsDataTable;
use App\DataTables\TAPetroGallonsDataTable;
use App\DataTables\LovesGallonsDataTable;
use App\DataTables\OutOfNetworkDataTable;
use App\DataTables\GallonByStateDataTable;
use App\DataTables\MilesByStateDataTable;
use App\DataTables\GallonsByStoreDataTable;
use Carbon\Carbon;
use DB;

class MonthlyReportsController extends Controller
{
    public function index(InNetworkGallonsDataTable $dataTable, Request $request)
    {
        $start = "";
        $end = "";

        if (!empty($request->input('start_date'))) {
            $start = $request->input('start_date');
        } else {
            $start = date('Y-m-01');
        }

        if (!empty($request->input('end_date'))) {
            $end = $request->input('end_date');
        } else {
            $end = date('Y-m-t');
        }
        $stateMapData = json_decode($this->stateMap($start, $end), true);
        $storeMapData = json_decode($this->storeMap($start, $end), true);
        
        $In_Network_Gallons = TransactionReport::select(
            DB::raw('COUNT(*) as Total'),
            DB::raw('SUM(`Quantity`) as TotalGallons'),
            DB::raw('SUM(`Amount`) as TotalCharges'),
            DB::raw('(SUM(`Amount`)/SUM(`Quantity`)) AS AvgPrice'),
            DB::raw('(SUM(`Quantity`)*0.20) AS TotalRebate')
        )
            ->whereBetween('Transaction Date', [$start, $end])
            ->whereIn('ChainID', array(3, 2, 1))
            ->whereIn('Product Code', ['ULSD', 'FUEL', 'ULSR'])
            ->where('Sub Fleet', getAuthMemberId())
            ->first();

        $Out_Network_Gallons = TransactionReport::select(
            DB::raw('COUNT(*) as Total'),
            DB::raw('SUM(`Quantity`) as TotalGallons'),
            DB::raw('SUM(`Amount`) as TotalCharges'),
            DB::raw('(SUM(`Amount`)/SUM(`Quantity`)) AS AvgPrice'),
            DB::raw('(SUM(`Quantity`)*0.20) AS TotalRebate')
        )
            ->whereBetween('Transaction Date', [$start, $end])
            ->where('ChainID', '2')
            ->whereIn('Product Code', array('ULSD', 'FUEL', 'ULSR'))
            ->where('Sub Fleet', getAuthMemberId())
            ->first();
            
        $totalGallons = $In_Network_Gallons->TotalGallons + $Out_Network_Gallons->TotalGallons;
        $totalRebate = $In_Network_Gallons->TotalRebate + $Out_Network_Gallons->TotalRebate;
        $totalAvrage =  !empty($totalGallons) ? $totalRebate / $totalGallons : 0.00; 

        $Pilot_Gallons = TransactionReport::select(
            DB::raw('COUNT(*) as Total'),
            DB::raw('SUM(`Quantity`) as TotalGallons'),
            DB::raw('SUM(`Amount`) as TotalCharges'),
            DB::raw('(SUM(`Amount`)/SUM(`Quantity`)) AS AvgPrice'),
            DB::raw('(SUM(`Quantity`)*0.20) AS TotalRebate')
        )
            ->whereBetween('Transaction Date', [$start, $end])
            ->whereIn('ChainID', ['1'])
            ->whereIn('Product Code', ['ULSD', 'FUEL', 'ULSR'])
            ->where('Sub Fleet', getAuthMemberId())
            ->first();

        $TA_Gallons = TransactionReport::select(
            DB::raw('COUNT(*) as Total'),
            DB::raw('SUM(`Quantity`) as TotalGallons'),
            DB::raw('SUM(`Amount`) as TotalCharges'),
            DB::raw('(SUM(`Amount`)/SUM(`Quantity`)) AS AvgPrice'),
            DB::raw('(SUM(`Quantity`)*0.20) AS TotalRebate')
        )
            ->whereBetween('Transaction Date', [$start, $end])
            ->whereIn('ChainID', ['3'])
            ->whereIn('Product Code', ['ULSD', 'FUEL', 'ULSR'])
            ->where('Sub Fleet', getAuthMemberId())
            ->first();

        $Loves_Gallons = TransactionReport::select(
            DB::raw('COUNT(*) as Total'),
            DB::raw('SUM(`Quantity`) as TotalGallons'),
            DB::raw('SUM(`Amount`) as TotalCharges'),
            DB::raw('(SUM(`Amount`)/SUM(`Quantity`)) AS AvgPrice'),
            DB::raw('(SUM(`Quantity`)*0.20) AS TotalRebate')
        )
            ->whereBetween('Transaction Date', [$start, $end])
            ->whereIn('ChainID', ['2'])
            ->whereIn('Product Code', ['ULSD', 'FUEL', 'ULSR'])
            ->where('Sub Fleet', getAuthMemberId())
            ->first();

        $One9_and_Express_Gallons = TransactionReport::select(
            DB::raw('COUNT(*) as Total'),
            DB::raw('SUM(`Quantity`) as TotalGallons'),
            DB::raw('SUM(`Amount`) as TotalCharges'),
            DB::raw('(SUM(`Amount`)/SUM(`Quantity`)) AS AvgPrice'),
            DB::raw('(SUM(`Quantity`)*0.20) AS TotalRebate')
        )
            ->whereBetween('Transaction Date', [$start, $end])
            ->whereIn('ChainID', ['1'])
            ->whereIn('Product Code', ['ULSD', 'FUEL', 'ULSR'])
            ->where(function ($query) {
                $query->where('Location Name', 'like', '%Express%')
                    ->orWhere('Location Name', 'like', '%One9%');
            })
            ->where('Sub Fleet', getAuthMemberId())
            ->first();

        $PilotFJ = TransactionReport::select(
            DB::raw('COUNT(*) as Total'),
            'Location State/Province',
            DB::raw('SUM(Quantity) as TotalGallons'),
            DB::raw('SUM(Amount) as TotalCharges'),
            DB::raw('(SUM(Amount)/SUM(Quantity)) AS AvgPrice'),
            DB::raw('(SUM(Quantity)*0.20) AS TotalRebate')
        )->whereBetween('Transaction Date', [$start, $end])
            ->where('ChainID', 1)
            ->where('Sub Fleet', getAuthMemberId())
            ->whereIn('Product Code', ['ULSD', 'FUEL', 'ULSR'])
            ->groupBy('Location State/Province')
            ->orderByDesc('TotalGallons')->first();

        $currentUserID = getAuthMemberId();
        if (in_array($currentUserID, ["24", "8", "32", "34", "18", "14", "20", "17", "10", "19", "50", "48"])) {
            $sql = "SELECT SUM(CarrierPay) as Total FROM (
                        SELECT *, GREATEST(( CASE WHEN(DailyTrans.ChainID = 1) THEN( SELECT TRUNCATE ( (TBL_PILOT.`Your Price` + 0.15), 4) AS YourPrice FROM TBL_PILOT INNER JOIN `TABLE 172` ON `TABLE 172`.Site = TBL_PILOT.Site WHERE TBL_PILOT.`Retail Price` != '' AND `TABLE 172`.LocationID = DailyTrans.`Location ID` AND TBL_PILOT.`Effective Date` = DailyTrans.`Transaction Date` LIMIT 1 ) WHEN(DailyTrans.ChainID = 3) THEN( SELECT TRUNCATE ( ( TBL_TA_RETAIL.`Fuel Price` + 0.15 ), 4 ) AS YourPrice FROM TBL_TA_RETAIL WHERE TBL_TA_RETAIL.ST != 'ON' AND DailyTrans.`Location ID` = TBL_TA_RETAIL.`Third Party Merchant ID` AND TBL_TA_RETAIL.`Deal Eff. Date` = DailyTrans.`Transaction Date` LIMIT 1 ) END ),0) AS MikeKeep, GREATEST(( DailyTrans.Retail_PPU -GREATEST(( CASE WHEN(DailyTrans.ChainID = 1) THEN( SELECT TRUNCATE ( (TBL_PILOT.`Your Price` + 0.15), 4 ) AS YourPrice FROM TBL_PILOT INNER JOIN `TABLE 172` ON `TABLE 172`.Site = TBL_PILOT.Site WHERE TBL_PILOT.`Retail Price` != '' AND `TABLE 172`.LocationID = DailyTrans.`Location ID` AND TBL_PILOT.`Effective Date` = DailyTrans.`Transaction Date` LIMIT 1 ) WHEN(DailyTrans.ChainID = 3) THEN( SELECT TRUNCATE ( ( TBL_TA_RETAIL.`Fuel Price` + 0.15 ), 4 ) AS YourPrice FROM TBL_TA_RETAIL WHERE TBL_TA_RETAIL.ST != 'ON' AND DailyTrans.`Location ID` = TBL_TA_RETAIL.`Third Party Merchant ID` AND TBL_TA_RETAIL.`Deal Eff. Date` = DailyTrans.`Transaction Date` LIMIT 1 ) END ),0) ),0) AS retainminusmike, GREATEST(( CASE WHEN( DailyTrans.ChainID = 1 OR DailyTrans.ChainID = 3 ) THEN( DailyTrans.Retail_PPU -GREATEST(( CASE WHEN(DailyTrans.ChainID = 1) THEN( SELECT TRUNCATE ( (TBL_PILOT.`Your Price` + 0.15), 4 ) AS YourPrice FROM TBL_PILOT INNER JOIN `TABLE 172` ON `TABLE 172`.Site = TBL_PILOT.Site WHERE TBL_PILOT.`Retail Price` != '' AND `TABLE 172`.LocationID = DailyTrans.`Location ID` AND TBL_PILOT.`Effective Date` = DailyTrans.`Transaction Date` LIMIT 1 ) WHEN(DailyTrans.ChainID = 3) THEN( SELECT TRUNCATE ( ( TBL_TA_RETAIL.`Fuel Price` + 0.15 ), 4 ) AS YourPrice FROM TBL_TA_RETAIL WHERE TBL_TA_RETAIL.ST != 'ON' AND DailyTrans.`Location ID` = TBL_TA_RETAIL.`Third Party Merchant ID` AND TBL_TA_RETAIL.`Deal Eff. Date` = DailyTrans.`Transaction Date` LIMIT 1 ) END ),0) ) - 0.10 WHEN(DailyTrans.ChainID = 2) THEN 0 END ),0) AS CarrierKeep, GREATEST(( ( CASE WHEN( DailyTrans.ChainID = 1 OR DailyTrans.ChainID = 3 ) THEN( DailyTrans.Retail_PPU -GREATEST(( CASE WHEN(DailyTrans.ChainID = 1) THEN( SELECT TRUNCATE ( (TBL_PILOT.`Your Price` + 0.15), 4 ) AS YourPrice FROM TBL_PILOT INNER JOIN `TABLE 172` ON `TABLE 172`.Site = TBL_PILOT.Site WHERE TBL_PILOT.`Retail Price` != '' AND `TABLE 172`.LocationID = DailyTrans.`Location ID` AND TBL_PILOT.`Effective Date` = DailyTrans.`Transaction Date` LIMIT 1 ) WHEN(DailyTrans.ChainID = 3) THEN( SELECT TRUNCATE ( ( TBL_TA_RETAIL.`Fuel Price` + 0.15 ), 4 ) AS YourPrice FROM TBL_TA_RETAIL WHERE TBL_TA_RETAIL.ST != 'ON' AND DailyTrans.`Location ID` = TBL_TA_RETAIL.`Third Party Merchant ID` AND TBL_TA_RETAIL.`Deal Eff. Date` = DailyTrans.`Transaction Date` ORDER BY DailyTrans.`Transaction Date` DESC LIMIT 1 ) END ),0) ) - 0.10 WHEN(DailyTrans.ChainID = 2) THEN 0 END ) * DailyTrans.Quantity ),0) AS CarrierPay, ( CASE WHEN( DailyTrans.ChainID = 1 OR DailyTrans.ChainID = 3 ) THEN DailyTrans.Quantity * 0.10 WHEN(DailyTrans.ChainID = 2) THEN DailyTrans.Quantity * 0.10 END ) AS MikeKeepOld
                        FROM
                        DailyTrans
                        WHERE
                        DailyTrans.`Transaction Date` BETWEEN '$start' AND '$end'
                        AND DailyTrans.`Sub Fleet` = '$currentUserID'
                        AND DailyTrans.ChainID IN(1, 3)
                        AND( DailyTrans.`Product Code` = 'ULSD' OR DailyTrans.`Product Code` = 'FUEL' )
                        ) as tableA";
            $rowa = DB::select(DB::raw($sql));
            $rowa = !empty($rowa) ? (array) $rowa[0] : [];

            $ytd_rebate_sql = "SELECT SUM(CarrierPay) as Total FROM (
                    SELECT *, GREATEST(( CASE WHEN(DailyTrans.ChainID = 1) THEN( SELECT TRUNCATE ( (TBL_PILOT.`Your Price` + 0.15), 4) AS YourPrice FROM TBL_PILOT INNER JOIN `TABLE 172` ON `TABLE 172`.Site = TBL_PILOT.Site WHERE TBL_PILOT.`Retail Price` != '' AND `TABLE 172`.LocationID = DailyTrans.`Location ID` AND TBL_PILOT.`Effective Date` = DailyTrans.`Transaction Date` LIMIT 1 ) WHEN(DailyTrans.ChainID = 3) THEN( SELECT TRUNCATE ( ( TBL_TA_RETAIL.`Fuel Price` + 0.15 ), 4 ) AS YourPrice FROM TBL_TA_RETAIL WHERE TBL_TA_RETAIL.ST != 'ON' AND DailyTrans.`Location ID` = TBL_TA_RETAIL.`Third Party Merchant ID` AND TBL_TA_RETAIL.`Deal Eff. Date` = DailyTrans.`Transaction Date` LIMIT 1 ) END ),0) AS MikeKeep, GREATEST(( DailyTrans.Retail_PPU -GREATEST(( CASE WHEN(DailyTrans.ChainID = 1) THEN( SELECT TRUNCATE ( (TBL_PILOT.`Your Price` + 0.15), 4 ) AS YourPrice FROM TBL_PILOT INNER JOIN `TABLE 172` ON `TABLE 172`.Site = TBL_PILOT.Site WHERE TBL_PILOT.`Retail Price` != '' AND `TABLE 172`.LocationID = DailyTrans.`Location ID` AND TBL_PILOT.`Effective Date` = DailyTrans.`Transaction Date` LIMIT 1 ) WHEN(DailyTrans.ChainID = 3) THEN( SELECT TRUNCATE ( ( TBL_TA_RETAIL.`Fuel Price` + 0.15 ), 4 ) AS YourPrice FROM TBL_TA_RETAIL WHERE TBL_TA_RETAIL.ST != 'ON' AND DailyTrans.`Location ID` = TBL_TA_RETAIL.`Third Party Merchant ID` AND TBL_TA_RETAIL.`Deal Eff. Date` = DailyTrans.`Transaction Date` LIMIT 1 ) END ),0) ),0) AS retainminusmike, GREATEST(( CASE WHEN( DailyTrans.ChainID = 1 OR DailyTrans.ChainID = 3 ) THEN( DailyTrans.Retail_PPU -GREATEST(( CASE WHEN(DailyTrans.ChainID = 1) THEN( SELECT TRUNCATE ( (TBL_PILOT.`Your Price` + 0.15), 4 ) AS YourPrice FROM TBL_PILOT INNER JOIN `TABLE 172` ON `TABLE 172`.Site = TBL_PILOT.Site WHERE TBL_PILOT.`Retail Price` != '' AND `TABLE 172`.LocationID = DailyTrans.`Location ID` AND TBL_PILOT.`Effective Date` = DailyTrans.`Transaction Date` LIMIT 1 ) WHEN(DailyTrans.ChainID = 3) THEN( SELECT TRUNCATE ( ( TBL_TA_RETAIL.`Fuel Price` + 0.15 ), 4 ) AS YourPrice FROM TBL_TA_RETAIL WHERE TBL_TA_RETAIL.ST != 'ON' AND DailyTrans.`Location ID` = TBL_TA_RETAIL.`Third Party Merchant ID` AND TBL_TA_RETAIL.`Deal Eff. Date` = DailyTrans.`Transaction Date` LIMIT 1 ) END ),0) ) - 0.10 WHEN(DailyTrans.ChainID = 2) THEN 0 END ),0) AS CarrierKeep, GREATEST(( ( CASE WHEN( DailyTrans.ChainID = 1 OR DailyTrans.ChainID = 3 ) THEN( DailyTrans.Retail_PPU -GREATEST(( CASE WHEN(DailyTrans.ChainID = 1) THEN( SELECT TRUNCATE ( (TBL_PILOT.`Your Price` + 0.15), 4 ) AS YourPrice FROM TBL_PILOT INNER JOIN `TABLE 172` ON `TABLE 172`.Site = TBL_PILOT.Site WHERE TBL_PILOT.`Retail Price` != '' AND `TABLE 172`.LocationID = DailyTrans.`Location ID` AND TBL_PILOT.`Effective Date` = DailyTrans.`Transaction Date` LIMIT 1 ) WHEN(DailyTrans.ChainID = 3) THEN( SELECT TRUNCATE ( ( TBL_TA_RETAIL.`Fuel Price` + 0.15 ), 4 ) AS YourPrice FROM TBL_TA_RETAIL WHERE TBL_TA_RETAIL.ST != 'ON' AND DailyTrans.`Location ID` = TBL_TA_RETAIL.`Third Party Merchant ID` AND TBL_TA_RETAIL.`Deal Eff. Date` = DailyTrans.`Transaction Date` ORDER BY DailyTrans.`Transaction Date` DESC LIMIT 1 ) END ),0) ) - 0.10 WHEN(DailyTrans.ChainID = 2) THEN 0 END ) * DailyTrans.Quantity ),0) AS CarrierPay, ( CASE WHEN( DailyTrans.ChainID = 1 OR DailyTrans.ChainID = 3 ) THEN DailyTrans.Quantity * 0.10 WHEN(DailyTrans.ChainID = 2) THEN DailyTrans.Quantity * 0.10 END ) AS MikeKeepOld
                    FROM
                    DailyTrans
                    WHERE
                    YEAR(DailyTrans.`Transaction Date`)=YEAR(NOW())
                    AND DailyTrans.`Sub Fleet` = '$currentUserID'
                    AND DailyTrans.ChainID IN(1, 3)
                    AND( DailyTrans.`Product Code` = 'ULSD' OR DailyTrans.`Product Code` = 'FUEL' )
                    ) as tableA";
            $ytd_rebate = DB::select(DB::raw($ytd_rebate_sql));
            $ytd_rebate = !empty($ytd_rebate) ? (array) $ytd_rebate[0] : [];

            $lifetime_rebate_sql = "SELECT SUM(CarrierPay) as Total FROM (
                    SELECT *, GREATEST(( CASE WHEN(DailyTrans.ChainID = 1) THEN( SELECT TRUNCATE ( (TBL_PILOT.`Your Price` + 0.15), 4) AS YourPrice FROM TBL_PILOT INNER JOIN `TABLE 172` ON `TABLE 172`.Site = TBL_PILOT.Site WHERE TBL_PILOT.`Retail Price` != '' AND `TABLE 172`.LocationID = DailyTrans.`Location ID` AND TBL_PILOT.`Effective Date` = DailyTrans.`Transaction Date` LIMIT 1 ) WHEN(DailyTrans.ChainID = 3) THEN( SELECT TRUNCATE ( ( TBL_TA_RETAIL.`Fuel Price` + 0.15 ), 4 ) AS YourPrice FROM TBL_TA_RETAIL WHERE TBL_TA_RETAIL.ST != 'ON' AND DailyTrans.`Location ID` = TBL_TA_RETAIL.`Third Party Merchant ID` AND TBL_TA_RETAIL.`Deal Eff. Date` = DailyTrans.`Transaction Date` LIMIT 1 ) END ),0) AS MikeKeep, GREATEST(( DailyTrans.Retail_PPU -GREATEST(( CASE WHEN(DailyTrans.ChainID = 1) THEN( SELECT TRUNCATE ( (TBL_PILOT.`Your Price` + 0.15), 4 ) AS YourPrice FROM TBL_PILOT INNER JOIN `TABLE 172` ON `TABLE 172`.Site = TBL_PILOT.Site WHERE TBL_PILOT.`Retail Price` != '' AND `TABLE 172`.LocationID = DailyTrans.`Location ID` AND TBL_PILOT.`Effective Date` = DailyTrans.`Transaction Date` LIMIT 1 ) WHEN(DailyTrans.ChainID = 3) THEN( SELECT TRUNCATE ( ( TBL_TA_RETAIL.`Fuel Price` + 0.15 ), 4 ) AS YourPrice FROM TBL_TA_RETAIL WHERE TBL_TA_RETAIL.ST != 'ON' AND DailyTrans.`Location ID` = TBL_TA_RETAIL.`Third Party Merchant ID` AND TBL_TA_RETAIL.`Deal Eff. Date` = DailyTrans.`Transaction Date` LIMIT 1 ) END ),0) ),0) AS retainminusmike, GREATEST(( CASE WHEN( DailyTrans.ChainID = 1 OR DailyTrans.ChainID = 3 ) THEN( DailyTrans.Retail_PPU -GREATEST(( CASE WHEN(DailyTrans.ChainID = 1) THEN( SELECT TRUNCATE ( (TBL_PILOT.`Your Price` + 0.15), 4 ) AS YourPrice FROM TBL_PILOT INNER JOIN `TABLE 172` ON `TABLE 172`.Site = TBL_PILOT.Site WHERE TBL_PILOT.`Retail Price` != '' AND `TABLE 172`.LocationID = DailyTrans.`Location ID` AND TBL_PILOT.`Effective Date` = DailyTrans.`Transaction Date` LIMIT 1 ) WHEN(DailyTrans.ChainID = 3) THEN( SELECT TRUNCATE ( ( TBL_TA_RETAIL.`Fuel Price` + 0.15 ), 4 ) AS YourPrice FROM TBL_TA_RETAIL WHERE TBL_TA_RETAIL.ST != 'ON' AND DailyTrans.`Location ID` = TBL_TA_RETAIL.`Third Party Merchant ID` AND TBL_TA_RETAIL.`Deal Eff. Date` = DailyTrans.`Transaction Date` LIMIT 1 ) END ),0) ) - 0.10 WHEN(DailyTrans.ChainID = 2) THEN 0 END ),0) AS CarrierKeep, GREATEST(( ( CASE WHEN( DailyTrans.ChainID = 1 OR DailyTrans.ChainID = 3 ) THEN( DailyTrans.Retail_PPU -GREATEST(( CASE WHEN(DailyTrans.ChainID = 1) THEN( SELECT TRUNCATE ( (TBL_PILOT.`Your Price` + 0.15), 4 ) AS YourPrice FROM TBL_PILOT INNER JOIN `TABLE 172` ON `TABLE 172`.Site = TBL_PILOT.Site WHERE TBL_PILOT.`Retail Price` != '' AND `TABLE 172`.LocationID = DailyTrans.`Location ID` AND TBL_PILOT.`Effective Date` = DailyTrans.`Transaction Date` LIMIT 1 ) WHEN(DailyTrans.ChainID = 3) THEN( SELECT TRUNCATE ( ( TBL_TA_RETAIL.`Fuel Price` + 0.15 ), 4 ) AS YourPrice FROM TBL_TA_RETAIL WHERE TBL_TA_RETAIL.ST != 'ON' AND DailyTrans.`Location ID` = TBL_TA_RETAIL.`Third Party Merchant ID` AND TBL_TA_RETAIL.`Deal Eff. Date` = DailyTrans.`Transaction Date` ORDER BY DailyTrans.`Transaction Date` DESC LIMIT 1 ) END ),0) ) - 0.10 WHEN(DailyTrans.ChainID = 2) THEN 0 END ) * DailyTrans.Quantity ),0) AS CarrierPay, ( CASE WHEN( DailyTrans.ChainID = 1 OR DailyTrans.ChainID = 3 ) THEN DailyTrans.Quantity * 0.10 WHEN(DailyTrans.ChainID = 2) THEN DailyTrans.Quantity * 0.10 END ) AS MikeKeepOld
                    FROM
                    DailyTrans
                    WHERE
                    DailyTrans.`Sub Fleet` = '$currentUserID'
                    AND DailyTrans.ChainID IN(1, 3)
                    AND( DailyTrans.`Product Code` = 'ULSD' OR DailyTrans.`Product Code` = 'FUEL' )
                    ) as tableA";
            $lifetime_rebate = DB::select(DB::raw($lifetime_rebate_sql));
            $lifetime_rebate = !empty($lifetime_rebate) ? (array) $lifetime_rebate[0] : [];

            $SQL = "SELECT *,
                        (CASE WHEN (DailyTrans.ChainID = 1) THEN (SELECT TRUNCATE((TBL_PILOT.`Your Price` + 0.15), 4) AS YourPrice
                        FROM TBL_PILOT
                        INNER JOIN `TABLE 172` ON `TABLE 172`.Site = TBL_PILOT.Site
                        WHERE TBL_PILOT.`Retail Price` != '' AND `TABLE 172`.LocationID = DailyTrans.`Location ID`
                        AND TBL_PILOT.`Effective Date` = DailyTrans.`Transaction Date` LIMIT 1)
                        WHEN (DailyTrans.ChainID = 3) THEN (SELECT TRUNCATE((TBL_TA_RETAIL.`Fuel Price` + 0.15), 4) AS YourPrice FROM TBL_TA_RETAIL WHERE TBL_TA_RETAIL.ST != 'ON' AND DailyTrans.`Location ID` = TBL_TA_RETAIL.`Third Party Merchant ID` AND TBL_TA_RETAIL.`Deal Eff. Date` = DailyTrans.`Transaction Date` LIMIT 1) END) AS MikeKeep, (DailyTrans.Retail_PPU - (CASE WHEN (DailyTrans.ChainID = 1) THEN (SELECT TRUNCATE((TBL_PILOT.`Your Price` + 0.15), 4) AS YourPrice FROM TBL_PILOT INNER JOIN `TABLE 172` ON `TABLE 172`.Site = TBL_PILOT.Site WHERE TBL_PILOT.`Retail Price` != '' AND `TABLE 172`.LocationID = DailyTrans.`Location ID` AND TBL_PILOT.`Effective Date` = DailyTrans.`Transaction Date` LIMIT 1) WHEN (DailyTrans.ChainID = 3) THEN (SELECT TRUNCATE((TBL_TA_RETAIL.`Fuel Price` + 0.15), 4) AS YourPrice FROM TBL_TA_RETAIL WHERE TBL_TA_RETAIL.ST != 'ON' AND DailyTrans.`Location ID` = TBL_TA_RETAIL.`Third Party Merchant ID` AND TBL_TA_RETAIL.`Deal Eff. Date` = DailyTrans.`Transaction Date` LIMIT 1) END)) AS retainminusmike, (CASE WHEN (DailyTrans.ChainID = 1 OR DailyTrans.ChainID = 3) THEN (DailyTrans.Retail_PPU - (CASE WHEN (DailyTrans.ChainID = 1) THEN (SELECT TRUNCATE((TBL_PILOT.`Your Price` + 0.15), 4) AS YourPrice FROM TBL_PILOT INNER JOIN `TABLE 172` ON `TABLE 172`.Site = TBL_PILOT.Site WHERE TBL_PILOT.`Retail Price` != '' AND `TABLE 172`.LocationID = DailyTrans.`Location ID` AND TBL_PILOT.`Effective Date` = DailyTrans.`Transaction Date` LIMIT 1) WHEN (DailyTrans.ChainID = 3) THEN (SELECT TRUNCATE((TBL_TA_RETAIL.`Fuel Price` + 0.15), 4) AS YourPrice FROM TBL_TA_RETAIL WHERE TBL_TA_RETAIL.ST != 'ON' AND DailyTrans.`Location ID` = TBL_TA_RETAIL.`Third Party Merchant ID` AND TBL_TA_RETAIL.`Deal Eff. Date` = DailyTrans.`Transaction Date` LIMIT 1) END)) - 0.10 WHEN (DailyTrans.ChainID = 2) THEN 0 END) AS CarrierKeep, ((CASE WHEN (DailyTrans.ChainID = 1 OR DailyTrans.ChainID = 3) THEN (DailyTrans.Retail_PPU - (CASE WHEN (DailyTrans.ChainID = 1) THEN (SELECT TRUNCATE((TBL_PILOT.`Your Price` + 0.15), 4) AS YourPrice FROM TBL_PILOT INNER JOIN `TABLE 172` ON `TABLE 172`.Site = TBL_PILOT.Site WHERE TBL_PILOT.`Retail Price` != '' AND `TABLE 172`.LocationID = DailyTrans.`Location ID` AND TBL_PILOT.`Effective Date` = DailyTrans.`Transaction Date` LIMIT 1) WHEN (DailyTrans.ChainID = 3) THEN (SELECT TRUNCATE((TBL_TA_RETAIL.`Fuel Price` + 0.15), 4) AS YourPrice FROM TBL_TA_RETAIL WHERE TBL_TA_RETAIL.ST != 'ON' AND DailyTrans.`Location ID` = TBL_TA_RETAIL.`Third Party Merchant ID` AND TBL_TA_RETAIL.`Deal Eff. Date` = DailyTrans.`Transaction Date` ORDER BY DailyTrans.`Transaction Date` DESC LIMIT 1) END)) - 0.10 WHEN (DailyTrans.ChainID = 2) THEN 0 END) * DailyTrans.Quantity) AS CarrierPay, (CASE WHEN (DailyTrans.ChainID = 1 OR DailyTrans.ChainID = 3) THEN DailyTrans.Quantity * 0.10 WHEN (DailyTrans.ChainID = 2) THEN DailyTrans.Quantity * 0.10 END) AS MikeKeepOld
                        FROM DailyTrans
                        WHERE DailyTrans.ChainID IN (1, 3)
                        AND (DailyTrans.`Product Code` = 'ULSD' OR DailyTrans.`Product Code` = 'FUEL')
                        AND MONTH(DailyTrans.`Transaction Date`) = MONTH(NOW() - INTERVAL 1 MONTH)
                        AND YEAR(DailyTrans.`Transaction Date`)=YEAR(NOW() - INTERVAL 1 MONTH)
                        AND (`Sub Fleet` IN ('" . getAuthMemberId() . "'))
                        ";
            $dataRebatesByPMonth = DB::select(DB::raw($SQL));
            $sum_CarrierPay = [];

            foreach ($dataRebatesByPMonth as $row) {
                if ($row->ChainID == 2) {
                    $row->CarrierPay = 0;
                    $row->MarginRebate = (0.10 * $row->Quantity);
                    $row->CJORebate = $row->MarginRebate;
                    $row->CarrierRebate = 0;
                    $row->CarrierKeep = 0;
                }
                if ($row->CarrierPay <= 0 && $row->ChainID != 2) {
                    $row->CarrierPay = 0;
                    $row->CarrierKeep = 0;
                }
                $sum_CarrierPay[] = isset($row->CarrierPay) ? $row->CarrierPay : 0;
            }
            $rowaP['Total'] = array_sum($sum_CarrierPay);
        } else if (in_array($currentUserID, ["5", "28", "13", "2", "25", "11", "19", "37", "15"])) {
            $sql = "SELECT SUM(REBATE*QTY) AS Total
                        FROM FUEL_PROCESS
                        WHERE TRANS_DATE BETWEEN '$start' AND '$end' and SUBFLEET = '$currentUserID' AND `LocationID` IN(1,3) AND `PRODUCT_CODE` = 'ULSD'";
            $rowa = DB::select(DB::raw($sql));
            $rowa = !empty($rowa) ? (array) $rowa[0] : [];

            $ytd_rebate_sql = "SELECT SUM(REBATE*QTY) AS Total
                        FROM FUEL_PROCESS
                        WHERE YEAR(TRANS_DATE)=YEAR(NOW()) and SUBFLEET = '$currentUserID' AND `LocationID` IN(1,3) AND `PRODUCT_CODE` = 'ULSD'";
            $ytd_rebate = DB::select(DB::raw($ytd_rebate_sql));
            $ytd_rebate = !empty($ytd_rebate) ? (array) $ytd_rebate[0] : [];

            $lifetime_rebate_sql = "SELECT SUM(REBATE*QTY) AS Total
                        FROM FUEL_PROCESS
                        WHERE SUBFLEET = '$currentUserID' AND `LocationID` IN(1,3) AND `PRODUCT_CODE` = 'ULSD'";
            $lifetime_rebate = DB::select(DB::raw($lifetime_rebate_sql));
            $lifetime_rebate = !empty($lifetime_rebate) ? (array) $lifetime_rebate[0] : [];

            $sql = "SELECT SUM(REBATE*QTY) AS Total
                        FROM FUEL_PROCESS
                        WHERE MONTH(TRANS_DATE) = MONTH(NOW()-INTERVAL 1 MONTH) AND YEAR(TRANS_DATE)=YEAR(NOW()-INTERVAL 1 MONTH)and SUBFLEET = '$currentUserID' AND `LocationID` IN(1,3) AND `PRODUCT_CODE` = 'ULSD'";
            $rowaP = DB::select(DB::raw($sql));
            $rowaP = !empty($rowaP) ? (array) $rowaP[0] : [];
        } else if (in_array($currentUserID, ["40", "41", "42", "43", "44", "45", "46", "47", "48", "50"])) {
            $sql = "SELECT
                            SUM(CASE WHEN (`DailyTrans`.`Product Code` = 'ULSD' OR `DailyTrans`.`Product Code` = 'FUEL') AND `DailyTrans`.`ChainID` != '2' AND `DailyTrans`.`Location Name` NOT LIKE '%ONE9%' THEN (`CARRIER_PROFILE`.`MARGIN_RATE` * `DailyTrans`.`Quantity`) ELSE 0.00 END) AS Total
                            FROM
                            `DailyTrans`
                            LEFT JOIN `CARRIER_PROFILE` ON `DailyTrans`.`Sub Fleet` = `CARRIER_PROFILE`.`SUBFLEET`
                            DailyTrans.`Transaction Date` BETWEEN '$start' AND '$end'
                            AND `DailyTrans`.`Sub Fleet` = '$currentUserID'
                            ";
            $rowa = DB::select(DB::raw($sql));
            $rowa = !empty($rowa) ? (array) $rowa[0] : [];

            $ytd_rebate_sql = "SELECT
                            SUM(CASE WHEN (`DailyTrans`.`Product Code` = 'ULSD' OR `DailyTrans`.`Product Code` = 'FUEL') AND `DailyTrans`.`ChainID` != '2' AND `DailyTrans`.`Location Name` NOT LIKE '%ONE9%' THEN (`CARRIER_PROFILE`.`MARGIN_RATE` * `DailyTrans`.`Quantity`) ELSE 0.00 END) AS Total
                            FROM
                            `DailyTrans`
                            LEFT JOIN `CARRIER_PROFILE` ON `DailyTrans`.`Sub Fleet` = `CARRIER_PROFILE`.`SUBFLEET`
                            WHERE YEAR(`DailyTrans`.`Transaction Date`)=YEAR(NOW())
                            AND `DailyTrans`.`Sub Fleet` = '$currentUserID'
                            ";
            $ytd_rebate = DB::select(DB::raw($ytd_rebate_sql));
            $ytd_rebate = !empty($ytd_rebate) ? (array) $ytd_rebate[0] : [];

            $lifetime_rebate_sql = "SELECT
                            SUM(CASE WHEN (`DailyTrans`.`Product Code` = 'ULSD' OR `DailyTrans`.`Product Code` = 'FUEL') AND `DailyTrans`.`ChainID` != '2' AND `DailyTrans`.`Location Name` NOT LIKE '%ONE9%' THEN (`CARRIER_PROFILE`.`MARGIN_RATE` * `DailyTrans`.`Quantity`) ELSE 0.00 END) AS Total
                            FROM
                            `DailyTrans`
                            LEFT JOIN `CARRIER_PROFILE` ON `DailyTrans`.`Sub Fleet` = `CARRIER_PROFILE`.`SUBFLEET`
                            WHERE `DailyTrans`.`Sub Fleet` = '$currentUserID'
                            ";
            $lifetime_rebate = DB::select(DB::raw($lifetime_rebate_sql));
            $lifetime_rebate = !empty($lifetime_rebate) ? (array) $lifetime_rebate[0] : [];

            $sql = "SELECT
                        SUM(CASE WHEN (`DailyTrans`.`Product Code` = 'ULSD' OR `DailyTrans`.`Product Code` = 'FUEL') AND `DailyTrans`.`ChainID` != '2' AND `DailyTrans`.`Location Name` NOT LIKE '%ONE9%' THEN (`CARRIER_PROFILE`.`MARGIN_RATE` * `DailyTrans`.`Quantity`) ELSE 0.00 END) AS Total
                        FROM
                        `DailyTrans`
                        LEFT JOIN `CARRIER_PROFILE` ON `DailyTrans`.`Sub Fleet` = `CARRIER_PROFILE`.`SUBFLEET`
                        WHERE MONTH(`DailyTrans`.`Transaction Date`) = MONTH(NOW() -INTERVAL 1 MONTH)
                        AND YEAR(`DailyTrans`.`Transaction Date`)=YEAR(NOW() -INTERVAL 1 MONTH)
                        AND `DailyTrans`.`Sub Fleet` = '$currentUserID'
                        ";
            $rowaP = DB::select(DB::raw($sql));
            $rowaP = !empty($rowaP) ? (array) $rowaP[0] : [];
        } else {
            $sql = "SELECT SUM(`REBATE`*`QTY`) AS Total
                        FROM `FUEL_PROCESS`
                        WHERE TRANS_DATE BETWEEN '$start' AND '$end' and `SUBFLEET` = '" . getAuthMemberId() . "' AND `LocationID` IN(1,3) AND `PRODUCT_CODE` = 'ULSD'";
            $rowa = DB::select(DB::raw($sql));
            $rowa = !empty($rowa) ? (array) $rowa[0] : [];

            $ytd_rebate_sql = "SELECT SUM(`REBATE`*`QTY`) AS Total
                        FROM `FUEL_PROCESS`
                        WHERE YEAR(`TRANS_DATE`)=YEAR(NOW()) and `SUBFLEET` = '" . getAuthMemberId() . "' AND `LocationID` IN(1,3) AND `PRODUCT_CODE` = 'ULSD'";
            $ytd_rebate = DB::select(DB::raw($ytd_rebate_sql));
            $ytd_rebate = !empty($ytd_rebate) ? (array) $ytd_rebate[0] : [];

            $lifetime_rebate_sql = "SELECT SUM(`REBATE`*`QTY`) AS Total
                        FROM `FUEL_PROCESS`
                        WHERE `SUBFLEET` = '" . getAuthMemberId() . "' AND `LocationID` IN(1,3) AND `PRODUCT_CODE` = 'ULSD'";
            $lifetime_rebate = DB::select(DB::raw($lifetime_rebate_sql));
            $lifetime_rebate = !empty($lifetime_rebate) ? (array) $lifetime_rebate[0] : [];

            $sql = "SELECT SUM(`REBATE`*`QTY`) AS Total
                        FROM `FUEL_PROCESS`
                        WHERE MONTH(`TRANS_DATE`) = MONTH(NOW()-INTERVAL 1 MONTH) AND YEAR(`TRANS_DATE`)=YEAR(NOW()-INTERVAL 1 MONTH)and `SUBFLEET` = '" . getAuthMemberId() . "' AND `LocationID` IN(1,3) AND `PRODUCT_CODE` = 'ULSD'";
            $rowaP = DB::select(DB::raw($sql));
            $rowaP = !empty($rowaP) ? (array) $rowaP[0] : [];
        }

        $sqls = "SELECT sum(`Quantity`) AS Total, `Location State/Province` FROM `DailyTrans` WHERE
        `Location State/Province` != '' AND `Transaction Date` BETWEEN '$start' AND '$end'
         AND `Sub Fleet` = " . getAuthMemberId() . "
        GROUP BY `Location State/Province` ORDER BY `Total`  DESC limit 20";
        $data1 = DB::select(DB::raw($sqls));
        $data1 = !empty($data1) ? (array) $data1 : [];
        $gallonsByState = [];
        $states = [];
        foreach ($data1 as $rows) {
            $rows = (array) $rows;
            $gallonsByState[] = $rows['Total'];
            $states[] = $rows['Location State/Province'];
        }

        $sqlGallonsByDays = "SELECT `Transaction Date`, SUM(`Quantity`) AS TOTAL_GALS  FROM `DailyTrans` WHERE `Product Code` = 'ULSD' AND `Transaction Date` BETWEEN '$start' AND '$end' AND `Sub Fleet` = " . getAuthMemberId() . " GROUP BY `Transaction Date` ORDER BY `DailyTrans`.`Transaction Date` ASC";
        $dataGallonsByDays = DB::select(DB::raw($sqlGallonsByDays));
        $dataGallonsByDays = !empty($dataGallonsByDays) ? (array) $dataGallonsByDays : [];
        $gallonsDaysDates = [];
        $gallonsDays = [];
        foreach ($dataGallonsByDays as $rowsGallonsByDays) {
            $rowsGallonsByDays = (array) $rowsGallonsByDays;
            $gallonsDaysDates[] = date('M-Y-D', strtotime($rowsGallonsByDays['Transaction Date']));
            $gallonsDays[] = $rowsGallonsByDays['TOTAL_GALS'];
        }

        $sqls = "SELECT SUM(`Quantity`) AS Total, `Chain_Desc`
                    FROM `DailyTrans`
                    WHERE `Transaction Date` BETWEEN '$start' AND '$end'
                    AND `Sub Fleet` = " . getAuthMemberId() . "
                    GROUP BY `Chain_Desc`
                    ORDER BY Total DESC";
        $data1 = DB::select(DB::raw($sqls));
        $data1 = !empty($data1) ? (array) $data1 : [];
        $gallonsByChain = [];
        $chains = [];
        $colors = [];
        $colorsAll = ['#c6df5f', '#ffd000', '#a5435c', '#146683', '#1fddff', '#cda441', '#161114', '#4f8240'];
        $i = 0;
        foreach ($data1 as $rows) {
            $rows = (array) $rows;
            $gallonsByChain[] = $rows['Total'];
            $chains[] = stripslashes($rows['Chain_Desc']);
            $d = array_rand($colorsAll);
            $colors[] = $colorsAll[$i];
            $i++;
        }
        
       
        return $dataTable->render('monthlyReports.monthlyReports', compact('totalGallons', 'totalAvrage', 'In_Network_Gallons', 'Out_Network_Gallons', 'Pilot_Gallons', 'TA_Gallons', 'Loves_Gallons', 'One9_and_Express_Gallons', 'rowa', 'rowaP', 'ytd_rebate', 'lifetime_rebate', 'gallonsByState', 'states', 'gallonsDaysDates', 'gallonsDays', 'gallonsByChain', 'chains', 'colors', 'stateMapData', 'storeMapData'));
    }

    public function TAPetroGallonsGallons(TAPetroGallonsDataTable $dataTable)
    {
        return $dataTable->render('monthlyReports.monthlyReports');
    }

    public function PetroGallonsGallons(PilotFJGallonsDataTable $dataTable)
    {
        return $dataTable->render('monthlyReports.monthlyReports');
    }

    public function lovesGallons(LovesGallonsDataTable $dataTable)
    {
        return $dataTable->render('monthlyReports.monthlyReports');
    }

    public function outOfNetworkGallons(OutOfNetworkDataTable $dataTable)
    {
        return $dataTable->render('monthlyReports.monthlyReports');
    }

    public function gallonsByState(GallonByStateDataTable $dataTable)
    {
        return $dataTable->render('monthlyReports.monthlyReports');
    }

    public function milesByState(MilesByStateDataTable $dataTable)
    {
        return $dataTable->render('monthlyReports.monthlyReports');
    }

    public function gallonsByStore(GallonsByStoreDataTable $dataTable)
    {
        return $dataTable->render('monthlyReports.monthlyReports');
    }

    public function stateMap($start, $end)
    {
        // $start = '2020-06-02';
        // $end = '2020-06-30';

        if (!empty($start)) {
            $start_date = $start;
        } else {
            $start_date = Carbon::now()->startOfMonth();
        }
        if (!empty($end)) {
            $end_date = $end;
        } else {
            $end_date = Carbon::now()->endOfMonth();
        }


        $results =TransactionReport::select(DB::raw('SUM(`Quantity`) as gallons'),'DailyTrans.Location State/Province')        
        ->where('Sub Fleet', getAuthMemberId())
        ->whereBetween('Transaction Date', [$start_date, $end_date])  
        ->groupBy('DailyTrans.Location State/Province')
        ->orderBy('gallons', 'DESC')               
        ->get()->toArray();
        $data=[];
        $newData=[];

         foreach ($results as $result) {
            $data['state'] = $result['Location State/Province'];
            $data['gallon'] = $result['gallons'];
            $newData[] = $data;
        }
       
        return json_encode($newData);



    }

    public function storeMap($start, $end)
    {

        if (!empty($start)) {
            $start_date = $start;
        } else {
            $start_date = Carbon::now()->startOfMonth();
        }
        if (!empty($end)) {
            $end_date = $end;
        } else {
            $end_date = Carbon::now()->endOfMonth();
        }


        $results = DB::table('FUEL_PROCESS')->select('FUEL_PROCESS.LOCATION', DB::raw('sum(`QTY`) as QTY'))        
        ->where('FUEL_PROCESS.SUBFLEET', '=', getAuthMemberId())
        ->where('FUEL_PROCESS.PRODUCT_CODE', '=', 'ULSD')
        ->whereBetween('FUEL_PROCESS.TRANS_DATE', [$start_date, $end_date])
        ->groupBy('StoreID')
        ->orderBy('QTY', 'DESC')
        ->get()->toArray();

        $data=[];
        $newData=[];
         foreach ($results as $result) {
            $location = explode(',',$result->LOCATION);
            $loc = trim($location[1]);   
            $data['states'] = $loc;
            $data['quantity'] = $result->QTY;
            $newData[] = $data;           
        }

        $totals = [];
        foreach ($newData as $item) {
            $state = $item['states'];
            $quantity = $item['quantity'];
            
            // Check if the state already exists in the totals array
            if (isset($totals[$state])) {
                // If it exists, add the quantity to the existing total
                $totals[$state] += $quantity;
            } else {
                // If it doesn't exist, initialize the total with the quantity
                $totals[$state] = $quantity;
            }            
        }

        $qtyData = [];
        $finalData = [];
        foreach ($totals as $state => $totalQuantity) {            
            $qtyData['state'] = $state;
            $qtyData['QTY'] = $totalQuantity;
            $finalData[] = $qtyData;
        }
        return json_encode($finalData);
    }
}
