<?php

namespace App\Http\Controllers;

use App\Models\Invoice;
use App\Models\Member;
use App\Models\CarrierProfile;
use App\Models\PlaidBalanceHistory;
use Illuminate\Http\Request;
use App\Models\PaymentMethod;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Stripe\Stripe;

require public_path() . '/setting/sendgrid/vendor/autoload.php';

class MyWalletController extends Controller
{
    public function index()
    {
        $bankDetail = PaymentMethod::where('mem_id', getAuthMemberId())->where('type', 'BANK')->where('default_option', '1')->first();
        $userData = Member::where('username', Auth::user()->username)
            ->where('mem_id', getAuthMemberId())
            ->first();
        $carrierData = CarrierProfile::where('SUBFLEET', getAuthMemberId())->first();
        $getPlaidBalHistory = DB::select(DB::raw("
            SELECT plaid_bal_history.*
            FROM plaid_bal_history
            LEFT JOIN `payment_methods` ON plaid_bal_history.`pay_method_id` = payment_methods.`id`
            WHERE payment_methods.mem_id = '".getAuthMemberId()."'
            GROUP BY DATE_FORMAT(plaid_bal_history.`created_at`, '%Y-%m-%d %H') ORDER BY plaid_bal_history.id DESC LIMIT 10
            "));
        $invoices = Invoice::where('subfleet', getAuthMemberId())->withCount([
            'invoicePayments as total_paid_amount' => function ($query) {
                $query->where('payment_type', 'P')
                    ->where('status', 'succeeded')
                    ->select(DB::raw('SUM(amount)'));
            }
        ])->groupBy('start_date', 'end_date')
        ->orderBy('id', 'desc')->limit(10)->get();
        $totalInvoices = $invoices->sum('amount');
        $amount =
            $email = $carrierData->EMAIL;
        $stripe_customer_id = $carrierData->stripe_customer_id;
        $carrier_name = $carrierData->CONTACT_NAME;
        $link_token = "";

        \Stripe\Stripe::setApiKey(config('custom_key')['STRIPE_SECRET']);

        if (isset($stripe_customer_id) && $stripe_customer_id != "") {
        } else {
            $customer = \Stripe\Customer::create(
                array(
                    "name" => $carrier_name,
                    "email" => strip_tags(trim($email))
                )
            );
            $stripe_customer_id = $customer->id;
            CarrierProfile::where('SUBFLEET', getAuthMemberId())->update([
                'stripe_customer_id' => $stripe_customer_id
            ]);
        }

        $card_data = [];

        $stripe_token_get = PaymentMethod::where('mem_id', getAuthMemberId())->where('type', 'CC')->where('default_option', '1')->first();
        if (empty($stripe_token_get)) {
            $stripe_token_get = PaymentMethod::where('mem_id', getAuthMemberId())->where('type', 'CC')->orderBy('id', 'DESC')->first();
        }
        if (!empty($stripe_token_get)) {
            $userpwd = config('custom_key')['STRIPE_SECRET'] . ":" . "";
            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, "https://api.stripe.com/v1/tokens/$stripe_token_get->source");
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($ch, CURLOPT_USERPWD, $userpwd);

            $headers = array();
            $headers[] = 'Content-Type: application/x-www-form-urlencoded';
            curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

            $result = curl_exec($ch);
            $card_data = json_decode($result, true);
            if (curl_errno($ch)) {
                echo 'Error:' . curl_error($ch);
                return response()->json(['status' => false, 'message' => curl_error($ch)]);
            }
            curl_close($ch);
        }


        // TMSCloud
        $client_name = Auth::user()->name;
        // $client_name = $carrierData->NAME;
        $link_token = "";
        $accountID = "";
        // $plaidURL = "https://production.plaid.com";
        $plaidURL = "https://development.plaid.com";
        // $plaidURL = "https://sandbox.plaid.com";
        $curl = curl_init();
        if ($client_name != "") {
            curl_setopt_array(
                $curl,
                array(
                    CURLOPT_URL => $plaidURL . '/link/token/create',
                    CURLOPT_RETURNTRANSFER => true,
                    CURLOPT_ENCODING => '',
                    CURLOPT_MAXREDIRS => 10,
                    CURLOPT_TIMEOUT => 0,
                    CURLOPT_FOLLOWLOCATION => true,
                    CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                    CURLOPT_CUSTOMREQUEST => 'POST',
                    CURLOPT_POSTFIELDS => '{
                        "client_id": "' . config('custom_key')['CLIENT_ID'] . '",
                        "secret": "' . config('custom_key')['PLAID_SECRET_DEVELOPMENT'] . '",
                        "client_name": "TMSCloud",
                        "country_codes": [
                            "US"
                        ],
                        "language": "en",
                        "user": {
                            "client_user_id": "' . $client_name . '"
                        },
                        "products": [
                            "auth"
                        ]
                    }',
                    CURLOPT_HTTPHEADER => array(
                        'Content-Type: application/json'
                    ),
                )
            );

            $response = curl_exec($curl);
            curl_close($curl);
            $rs = json_decode($response);
            $link_token = @$rs->link_token;
        }
        return view(
            'myWallet.index',
            compact(
                'bankDetail',
                'userData',
                'carrierData',
                'link_token',
                'email',
                'getPlaidBalHistory',
                'invoices',
                'totalInvoices',
                'card_data'
            )
        );
    }

    public function newIndex()
    {
        $bankDetail = PaymentMethod::where('mem_id', getAuthMemberId())->where('type', 'BANK')->where('default_option', '1')->first();
        $userData = Member::where('username', Auth::user()->username)
            ->where('mem_id', getAuthMemberId())
            ->first();
        $carrierData = CarrierProfile::where('SUBFLEET', getAuthMemberId())->first();
        $getPlaidBalHistory = DB::select(DB::raw("
            SELECT plaid_bal_history.*
            FROM plaid_bal_history
            LEFT JOIN `payment_methods` ON plaid_bal_history.`pay_method_id` = payment_methods.`id`
            WHERE payment_methods.mem_id = '".getAuthMemberId()."'
            GROUP BY DATE_FORMAT(plaid_bal_history.`created_at`, '%Y-%m-%d %H') ORDER BY plaid_bal_history.id DESC LIMIT 10
            "));
        $invoices = Invoice::where('subfleet', getAuthMemberId())->withCount([
            'invoicePayments as total_paid_amount' => function ($query) {
                $query->where('payment_type', 'P')
                    ->where('status', 'succeeded')
                    ->select(DB::raw('SUM(amount)'));
            }
        ])->groupBy('start_date', 'end_date')
        ->orderBy('id', 'desc')->limit(10)->get();
        $totalInvoices = $invoices->sum('amount');
        $amount =
            $email = $carrierData->EMAIL;
        $stripe_customer_id = $carrierData->stripe_customer_id;
        $carrier_name = $carrierData->CONTACT_NAME;
        $link_token = "";

        \Stripe\Stripe::setApiKey(config('custom_key')['STRIPE_SECRET']);

        if (isset($stripe_customer_id) && $stripe_customer_id != "") {
        } else {
            $customer = \Stripe\Customer::create(
                array(
                    "name" => $carrier_name,
                    "email" => strip_tags(trim($email))
                )
            );
            $stripe_customer_id = $customer->id;
            CarrierProfile::where('SUBFLEET', getAuthMemberId())->update([
                'stripe_customer_id' => $stripe_customer_id
            ]);
        }

        $card_data = [];

        $stripe_token_get = PaymentMethod::where('mem_id', getAuthMemberId())->where('type', 'CC')->where('default_option', '1')->first();
        if (empty($stripe_token_get)) {
            $stripe_token_get = PaymentMethod::where('mem_id', getAuthMemberId())->where('type', 'CC')->orderBy('id', 'DESC')->first();
        }
        if (!empty($stripe_token_get)) {
            $userpwd = config('custom_key')['STRIPE_SECRET'] . ":" . "";
            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, "https://api.stripe.com/v1/tokens/$stripe_token_get->source");
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($ch, CURLOPT_USERPWD, $userpwd);

            $headers = array();
            $headers[] = 'Content-Type: application/x-www-form-urlencoded';
            curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

            $result = curl_exec($ch);
            $card_data = json_decode($result, true);
            if (curl_errno($ch)) {
                echo 'Error:' . curl_error($ch);
                return response()->json(['status' => false, 'message' => curl_error($ch)]);
            }
            curl_close($ch);
        }


        // TMSCloud
        $client_name = Auth::user()->name;
        // $client_name = $carrierData->NAME;
        $link_token = "";
        $accountID = "";
        // $plaidURL = "https://production.plaid.com";
        $plaidURL = "https://development.plaid.com";
        // $plaidURL = "https://sandbox.plaid.com";
        $curl = curl_init();
        if ($client_name != "") {
            curl_setopt_array(
                $curl,
                array(
                    CURLOPT_URL => $plaidURL . '/link/token/create',
                    CURLOPT_RETURNTRANSFER => true,
                    CURLOPT_ENCODING => '',
                    CURLOPT_MAXREDIRS => 10,
                    CURLOPT_TIMEOUT => 0,
                    CURLOPT_FOLLOWLOCATION => true,
                    CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                    CURLOPT_CUSTOMREQUEST => 'POST',
                    CURLOPT_POSTFIELDS => '{
                        "client_id": "' . config('custom_key')['CLIENT_ID'] . '",
                        "secret": "' . config('custom_key')['PLAID_SECRET_DEVELOPMENT'] . '",
                        "client_name": "TMSCloud",
                        "country_codes": [
                            "US"
                        ],
                        "language": "en",
                        "user": {
                            "client_user_id": "' . $client_name . '"
                        },
                        "products": [
                            "auth"
                        ]
                    }',
                    CURLOPT_HTTPHEADER => array(
                        'Content-Type: application/json'
                    ),
                )
            );

            $response = curl_exec($curl);
            curl_close($curl);
            $rs = json_decode($response);
            $link_token = @$rs->link_token;
        }
        return view(
            'myWallet.newIndex',
            compact(
                'bankDetail',
                'userData',
                'carrierData',
                'link_token',
                'email',
                'getPlaidBalHistory',
                'invoices',
                'totalInvoices',
                'card_data'
            )
        );
    }

    public function plaid_success(Request $requet)
    {
        $PLAID_LINK_PUBLIC_TOKEN = $requet->public_token;
        $ACCOUNT_ID = $requet->account_id;
        $link_token = $requet->link_token;
        $amount = $requet->amount;
        $email = $requet->email;
        $account_lastfour = "";
        // $plaidURL = "https://sandbox.plaid.com";
        $plaidURL = "https://development.plaid.com";
        // $plaidURL = "https://production.plaid.com";
        $mainData = [];

        $mainResponse = [
            'error' => '',
            'status' => false,
            'data' => $mainData
        ];

        $params = [
            'client_id' => config('custom_key')['CLIENT_ID'],
            'secret' => config('custom_key')['PLAID_SECRET_DEVELOPMENT'],
            'public_token' => $PLAID_LINK_PUBLIC_TOKEN,
        ];

        $headers[] = 'Content-Type: application/json';

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $plaidURL . "/item/public_token/exchange");
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($params));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 30);
        curl_setopt($ch, CURLOPT_TIMEOUT, 80);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        if (!$result = curl_exec($ch)) {
            trigger_error(curl_error($ch));
        }
        curl_close($ch);
        $jsonParsed = json_decode($result);
        $access_token = $jsonParsed->access_token;

        $curl = curl_init();
        curl_setopt_array(
            $curl,
            array(
                CURLOPT_URL => $plaidURL . '/accounts/balance/get',
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_ENCODING => '',
                CURLOPT_MAXREDIRS => 10,
                CURLOPT_TIMEOUT => 0,
                CURLOPT_FOLLOWLOCATION => true,
                CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                CURLOPT_CUSTOMREQUEST => 'POST',
                CURLOPT_POSTFIELDS => '{
                "client_id": "' . config('custom_key')['CLIENT_ID'] . '",
                "secret": "' . config('custom_key')['PLAID_SECRET_DEVELOPMENT'] . '",
                "access_token": "' . $access_token . '",
                "options": {
                    "account_ids": [
                        "' . $ACCOUNT_ID . '"
                    ]
                }
            }',
                CURLOPT_HTTPHEADER => array(
                    'Content-Type: application/json'
                ),
            )
        );
        $response = curl_exec($curl);
        curl_close($curl);
        $baledata = json_decode($response, true);
        $getAccountBalace = $baledata['accounts'][0]['balances']['available'];
        $account_lastfour = $baledata['accounts'][0]['mask'];
        $account_name = $baledata['accounts'][0]['official_name'];

        $btok_params = [
            'client_id' => config('custom_key')['CLIENT_ID'],
            'secret' => config('custom_key')['PLAID_SECRET_DEVELOPMENT'],
            'access_token' => $jsonParsed->access_token,
            'account_id' => $ACCOUNT_ID
        ];

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $plaidURL . "/processor/stripe/bank_account_token/create");
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($btok_params));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 30);
        curl_setopt($ch, CURLOPT_TIMEOUT, 80);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        if (!$result = curl_exec($ch)) {
            trigger_error(curl_error($ch));
        }
        curl_close($ch);

        $btok_parsed = json_decode($result, true);
        $bank_token = $btok_parsed['stripe_bank_account_token'];

        $mainData['token'] = $btok_parsed;
        $mainData['account_id'] = $ACCOUNT_ID;
        $mainData['balance'] = $getAccountBalace;
        $mainData['mask'] = $account_lastfour;
        $mainData['account_name'] = $account_name;
        $mainData['public_token'] = $access_token;
        $mainResponse = [
            'error' => '',
            'status' => true,
            'data' => $mainData
        ];

        return json_encode($mainResponse);
    }

    public function addbankach(Request $request)
    {
        \Stripe\Stripe::setApiKey(config('custom_key')['STRIPE_SECRET']);

        $userData = Member::where('username', Auth::user()->username)
            ->where('mem_id', getAuthMemberId())
            ->first();
        $carrierData = CarrierProfile::where('SUBFLEET', getAuthMemberId())->first();

        $email = $carrierData->EMAIL;
        $stripe_customer_id = $carrierData->stripe_customer_id;
        $carrier_name = $carrierData->CONTACT_NAME;
        $CARRIERID = $carrierData->ID;

        if (isset($stripe_customer_id) && $stripe_customer_id != "") {
        } else {
            $customer = \Stripe\Customer::create(
                array(
                    "name" => $carrier_name,
                    "email" => strip_tags(trim($email))
                )
            );
            $stripe_customer_id = $customer->id;
            CarrierProfile::where('SUBFLEET', getAuthMemberId())->update([
                'stripe_customer_id' => $stripe_customer_id
            ]);
        }

        $stripeToken = $_POST['stripeBankToken'];
        $account_id = $_POST['account_id'];
        $balance = $_POST['balance'];

        $mask = $_POST['mask'];
        $account_name = $_POST['account_name'];
        $public_token = $_POST['public_token'];

        PaymentMethod::where('mem_id', getAuthMemberId())->update([
            'default_option' => 0
        ]);

        $sourceData = PaymentMethod::where('mem_id', getAuthMemberId())->where('source', $stripeToken)->where('type', 'BANK')->first();
        if (empty($sourceData)) {
            PaymentMethod::create([
                'mem_id' => getAuthMemberId(),
                'source' => $stripeToken,
                'type' => 'BANK',
                'account_id' => $account_id,
                'balance' => $balance,
                'account_name' => $account_name,
                'mask' => $mask,
                'public_token' => $public_token,
                'default_option' => 1,
            ]);
        } else {
            PaymentMethod::where('mem_id', getAuthMemberId())->where('type', 'BANK')->update([
                'source' => $stripeToken,
                'balance' => $balance,
                'account_id' => $account_id,
                'default_option' => 1,
            ]);
        }

        $userpwd = config('custom_key')['STRIPE_SECRET'] . ":" . "";
        $ch = curl_init();

        curl_setopt($ch, CURLOPT_URL, 'https://api.stripe.com/v1/customers/' . $stripe_customer_id . '/sources');
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, "source=" . $stripeToken);
        curl_setopt($ch, CURLOPT_USERPWD, $userpwd);

        $headers = array();
        $headers[] = 'Content-Type: application/x-www-form-urlencoded';
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

        $result = curl_exec($ch);
        curl_close($ch);

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, 'https://api.stripe.com/v1/customers/' . $stripe_customer_id);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_POST, 1);
        curl_setopt($ch, CURLOPT_POSTFIELDS, "default_source=$stripeToken");
        curl_setopt($ch, CURLOPT_USERPWD, $userpwd);
        $headers = array();
        $headers[] = 'Content-Type: application/x-www-form-urlencoded';
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        $result = curl_exec($ch);
        curl_close($ch);

        // file_get_contents("https://efs2.tmdashboard.com/sendgrid/plaid_connected.php?ID=" . $CARRIERID);
        // Mail code starts
        $resultset = CarrierProfile::where('ID', $CARRIERID)->first();
        $status = [
            1 => "Active",
            0 => "Inactive",
        ];
        $table = '<h1>Plaid Connected # <a href="https://efspro.tmscloud.com/CARRIER_PROFILEedit.php?ID=' . $resultset->ID . '" target="_blank">' . $resultset->ID . '</a></h1> <table width="750" border="1" cellpading="3" cellspacing="0">
                    <thead>
                    </thead>
                     <tbody>';

        $table = $table .
                    '<tr>
                        <th>Balance</th>
                        <td>' . $balance . '</td>
                    </tr>
                    <tr>
                        <th>DOT #</th>
                        <td><a href="https://ai.fmcsa.dot.gov/SMS/Carrier/' . $resultset->DOT . '/8/Overview.aspx?FirstView=True" target="_blank">' . $resultset->DOT . '</a></td>
                    </tr>
                    <tr>
                        <th>Bisness Name</th>
                        <td>' . $resultset->NAME . '</td>
                    </tr>
                    <tr>
                        <th>Phone</th>
                        <td><a href="tel:' . $resultset->PHONE . '" target="_blank">' . $resultset->PHONE . '</a></td>
                    </tr>
                    <tr>
                        <th>Email</th>
                        <td>' . $resultset->EMAIL . '</td>
                    </tr>
                    <tr>
                        <th>Subfleet</th>
                        <td>' . $resultset->SUBFLEET . '</td>
                    </tr>
                    <tr>
                        <th>Status</th>
                        <td>' . $status[$resultset->ACTIVE] . '</td>
                    </tr>
                </tr>
                </tbody>';

        $table = $table . '</table>';

        $email = new \SendGrid\Mail\Mail();
        $email->setFrom("reports@tmscloud.com", "Diesel Buddy");
        $email->setSubject("Plaid Connected - " . $resultset->NAME);
        $email->addTo("daryn@tmscloud.com", "Daryn Duff");
        $email->addTo("getfuel@dieselbuddy.app");

        $email->addContent(
            "text/html",
            $table
        );
        $sendgrid = new \SendGrid('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');
        $response = $sendgrid->send($email);

        return response()->json(['status' => true, 'message' => 'Bank(ACH) Added Successfully']);
    }

    public function fetchPlaidBalance(Request $request)
    {
        $bankDetail = PaymentMethod::where('id', $request->id)->first();
        if (!empty($bankDetail)) {
            $headers[] = 'Content-Type: application/json';
            $client_id = config('custom_key')['CLIENT_ID'];
            $secret_key = config('custom_key')['PLAID_SECRET_DEVELOPMENT'];

            $PLAID_LINK_PUBLIC_TOKEN = $bankDetail->public_token;
            $ACCOUNT_ID = $bankDetail->account_id;
            $account_lastfour = "";

            $plaidURL = "https://development.plaid.com";

            $mainData = [];

            $mainResponse = [
                'error' => '',
                'status' => false,
                'data' => $mainData
            ];

            $curl = curl_init();
            curl_setopt_array(
                $curl,
                array(
                    CURLOPT_URL => $plaidURL . '/accounts/balance/get',
                    CURLOPT_RETURNTRANSFER => true,
                    CURLOPT_ENCODING => '',
                    CURLOPT_MAXREDIRS => 10,
                    CURLOPT_TIMEOUT => 0,
                    CURLOPT_FOLLOWLOCATION => true,
                    CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
                    CURLOPT_CUSTOMREQUEST => 'POST',
                    CURLOPT_POSTFIELDS => '{
                        "client_id": "' . $client_id . '",
                        "secret": "' . $secret_key . '",
                        "access_token": "' . $PLAID_LINK_PUBLIC_TOKEN . '",
                        "options": {
                            "account_ids": [
                                "' . $ACCOUNT_ID . '"
                            ]
                        }
                    }',
                    CURLOPT_HTTPHEADER => array(
                        'Content-Type: application/json'
                    ),
                )
            );
            $response = curl_exec($curl);
            curl_close($curl);
            $baledata = json_decode($response, true);
            $getAccountBalace = $baledata['accounts'][0]['balances']['available'];
            $pay_method_id = $request->id;
            $balance = $getAccountBalace;
            $created_at = date('Y-m-d H:i:s');
            $account_number = $bankDetail['account_name'];

            PlaidBalanceHistory::create([
                'pay_method_id' => $pay_method_id,
                'account_number' => $account_number,
                'balance' => $balance,
                'created_at' => $created_at,
            ]);

            PaymentMethod::where('id', $pay_method_id)->update([
                'balance' => $balance,
            ]);

            return $getAccountBalace;
        }
    }

    public function getStripeCards()
    {
        \Stripe\Stripe::setApiKey(config('custom_key')['STRIPE_SECRET']);

        $userData = Member::where('username', Auth::user()->username)
            ->where('mem_id', getAuthMemberId())
            ->first();
        $carrierData = CarrierProfile::where('SUBFLEET', getAuthMemberId())->first();
        $default_card = PaymentMethod::where('mem_id', getAuthMemberId())->where('type', 'CC')->where('default_option', 1)->first();
        $cardId = '';
        $userpwd = config('custom_key')['STRIPE_SECRET'] . ":";
        if (!empty($default_card)) {
            $ch = curl_init();

            curl_setopt($ch, CURLOPT_URL, "https://api.stripe.com/v1/tokens/$default_card->source");
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($ch, CURLOPT_USERPWD, $userpwd);

            $headers = array();
            $headers[] = 'Content-Type: application/x-www-form-urlencoded';
            curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

            $result = curl_exec($ch);
            $card_data = json_decode($result, true);
            $cardId = $card_data['card']['id'];
        }

        $email = $carrierData->EMAIL;
        $stripe_customer_id = $carrierData->stripe_customer_id;
        $carrier_name = $carrierData->CONTACT_NAME;

        if (isset($stripe_customer_id) && $stripe_customer_id != "") {
        } else {
            $customer = \Stripe\Customer::create(
                array(
                    "name" => $carrier_name,
                    "email" => strip_tags(trim($email))
                )
            );
            $stripe_customer_id = $customer->id;
            CarrierProfile::where('SUBFLEET', getAuthMemberId())->update([
                'stripe_customer_id' => $stripe_customer_id
            ]);
        }

        $ch = curl_init();

        curl_setopt($ch, CURLOPT_URL, 'https://api.stripe.com/v1/customers/' . $stripe_customer_id . '/cards');
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'GET');
        curl_setopt($ch, CURLOPT_USERPWD, $userpwd);

        $headers = array();
        $headers[] = 'Content-Type: application/x-www-form-urlencoded';
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

        $result = curl_exec($ch);

        if (curl_errno($ch)) {
            $response = array('error' => curl_error($ch), 'html' => '');
        } else {
            $cards = json_decode($result);
            $html = '';
            if (!empty($cards->data)) {
                foreach ($cards->data as $card) {
                    $expDate = $card->exp_month . '/' . $card->exp_year;
                    $html .= '<tr class="fs-6">';
                    $html .= '<td>' . $card->brand . '</td>';
                    $html .= '<td>' . $card->last4 . '</td>';
                    $html .= '<td>' . $expDate . '</td>';
                    $html .= '<td class="w-50">';
                    if ($card->id == $cardId) {
                        $html .= '<span class="me-2">Primary</span>';
                    } else {
                        $html .= '<button type="button" class="btn btn-primary btn-sm me-2" id="makePrimaryBtn' . $card->id . '" onclick=make_primary("' . $card->id . '");>Make Primary</button>';
                    }
                    $html .= '<button type="button" class="btn btn-dark btn-sm" id="deleteBtn' . $card->id . '" onclick=delete_card("' . $card->id . '");>Delete</button>';
                    $html .= '</td>';
                    $html .= '</tr>';
                }
            } else {
                $html .= '<tr>';
                $html .= '<td colspan="6" align="center">No Data Found</td>';
                $html .= '</tr>';
            }

            return $html;
        }

        curl_close($ch);

        return json_encode($response);
    }

    public function addcardsccdc(Request $request)
    {
        try {
            \Stripe\Stripe::setApiKey(config('custom_key')['STRIPE_SECRET']);

            $userData = Member::where('username', Auth::user()->username)
                ->where('mem_id', getAuthMemberId())
                ->first();
            $carrierData = CarrierProfile::where('SUBFLEET', getAuthMemberId())->first();

            $email = $carrierData->EMAIL;
            $stripe_customer_id = $carrierData->stripe_customer_id;
            $carrier_name = $carrierData->CONTACT_NAME;

            if (isset($stripe_customer_id) && $stripe_customer_id != "") {
            } else {
                $customer = \Stripe\Customer::create(
                    array(
                        "name" => $carrier_name,
                        "email" => strip_tags(trim($email))
                    )
                );
                $stripe_customer_id = $customer->id;
                CarrierProfile::where('SUBFLEET', getAuthMemberId())->update([
                    'stripe_customer_id' => $stripe_customer_id
                ]);
            }

            $stripeToken = $request->stripeToken;
            $userpwd = config('custom_key')['STRIPE_SECRET'] . ":" . "";
            $ch = curl_init();

            curl_setopt($ch, CURLOPT_URL, 'https://api.stripe.com/v1/customers/' . $stripe_customer_id . '/sources');
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
            curl_setopt($ch, CURLOPT_POST, 1);
            curl_setopt($ch, CURLOPT_POSTFIELDS, "source=" . $stripeToken);
            curl_setopt($ch, CURLOPT_USERPWD, $userpwd);

            $headers = array();
            $headers[] = 'Content-Type: application/x-www-form-urlencoded';
            curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

            $result = curl_exec($ch);
            if (curl_errno($ch)) {
                echo 'Error:' . curl_error($ch);
                return response()->json(['status' => false, 'message' => curl_error($ch)]);
            }
            curl_close($ch);

            $decodedJson = json_decode($result);
            if(!empty($decodedJson->error)){
                return response()->json(['status' => false, 'message' => $decodedJson->error->message]);
            }else{
                $sourceData = PaymentMethod::where('mem_id', getAuthMemberId())->where('source', $stripeToken)->where('type', 'CC')->first();
                if (empty($sourceData)) {
                    PaymentMethod::create([
                        'mem_id' => getAuthMemberId(),
                        'source' => $stripeToken,
                        'type' => 'CC',
                    ]);
                } else {
                    PaymentMethod::where('mem_id', getAuthMemberId())->where('type', 'CC')->update([
                        'source' => $stripeToken
                    ]);
                }
                return response()->json(['status' => true, 'message' => 'Card Added Successfully.']);
            }
        } catch (\Exception $e) {
            return response()->json(['status' => false, 'message' => $e->getMessage()]);
        }
    }

    public function delete_card($card)
    {
        try {
            $carrierInfo = CarrierProfile::where('SUBFLEET', getAuthMemberId())->first();
            \Stripe\Stripe::setApiKey(config('custom_key')['STRIPE_SECRET']);

            $email = $carrierInfo->EMAIL;
            $stripe_customer_id = $carrierInfo->stripe_customer_id;
            $carrier_name = $carrierInfo->CONTACT_NAME;

            if (isset($stripe_customer_id) && $stripe_customer_id != "") {
            } else {
                $customer = \Stripe\Customer::create(
                    array(
                        "name" => $carrier_name,
                        "email" => strip_tags(trim($email))
                    )
                );
                $stripe_customer_id = $customer->id;
                CarrierProfile::where('SUBFLEET', getAuthMemberId())->update([
                    'stripe_customer_id' => $stripe_customer_id
                ]);
            }

            $userpwd = config('custom_key')['STRIPE_SECRET'] . ":" . "";
            $ch = curl_init();

            curl_setopt($ch, CURLOPT_URL, "https://api.stripe.com/v1/customers/$stripe_customer_id/sources/$card");
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'DELETE');
            curl_setopt($ch, CURLOPT_HTTPAUTH, CURLAUTH_BASIC);
            curl_setopt($ch, CURLOPT_USERPWD, $userpwd);

            $response = curl_exec($ch);
            $res = json_decode($response);

            curl_close($ch);

            $source = '';
            $stripe_token_get = PaymentMethod::where('mem_id', getAuthMemberId())->where('type', 'CC')->get();
            if (!empty($stripe_token_get)) {
                foreach ($stripe_token_get as $token) {
                    $ch = curl_init();
                    curl_setopt($ch, CURLOPT_URL, "https://api.stripe.com/v1/tokens/$token->source");
                    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
                    curl_setopt($ch, CURLOPT_USERPWD, $userpwd);

                    $headers = array();
                    $headers[] = 'Content-Type: application/x-www-form-urlencoded';
                    curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

                    $result = curl_exec($ch);
                    $card_data = json_decode($result, true);
                    if ($card_data['card']['id'] == $card) {
                        $source = $card_data['id'];
                        break;
                    }
                    if (curl_errno($ch)) {
                        echo 'Error:' . curl_error($ch);
                        return response()->json(['status' => false, 'message' => curl_error($ch)]);
                    }
                    curl_close($ch);
                }
            }

            $card_delete = PaymentMethod::where('mem_id', getAuthMemberId())->where('type', 'CC')->where('source', $source)->first();
            if (!empty($card_delete)) {
                $card_delete->delete();
            }

            return response()->json(['status' => true, 'message' => 'Card Deleted Successfully']);
        } catch (\Exception $e) {
            return response()->json(['status' => false, 'message' => $e->getMessage()]);
        }
    }

    public function make_primary($card)
    {
        try {
            $carrierInfo = CarrierProfile::where('SUBFLEET', getAuthMemberId())->first();
            \Stripe\Stripe::setApiKey(config('custom_key')['STRIPE_SECRET']);

            $email = $carrierInfo->EMAIL;
            $stripe_customer_id = $carrierInfo->stripe_customer_id;
            $carrier_name = $carrierInfo->CONTACT_NAME;

            if (isset($stripe_customer_id) && $stripe_customer_id != "") {
            } else {
                $customer = \Stripe\Customer::create(
                    array(
                        "name" => $carrier_name,
                        "email" => strip_tags(trim($email))
                    )
                );
                $stripe_customer_id = $customer->id;
                CarrierProfile::where('SUBFLEET', getAuthMemberId())->update([
                    'stripe_customer_id' => $stripe_customer_id
                ]);
            }

            $userpwd = config('custom_key')['STRIPE_SECRET'] . ":" . "";
            $source = '';
            $stripe_token_get = PaymentMethod::where('mem_id', getAuthMemberId())->where('type', 'CC')->get();
            if (!empty($stripe_token_get)) {
                foreach ($stripe_token_get as $token) {
                    $ch = curl_init();
                    curl_setopt($ch, CURLOPT_URL, "https://api.stripe.com/v1/tokens/$token->source");
                    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
                    curl_setopt($ch, CURLOPT_USERPWD, $userpwd);

                    $headers = array();
                    $headers[] = 'Content-Type: application/x-www-form-urlencoded';
                    curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);

                    $result = curl_exec($ch);
                    $card_data = json_decode($result, true);
                    if ($card_data['card']['id'] == $card) {
                        $source = $card_data['id'];
                        break;
                    }
                    if (curl_errno($ch)) {
                        echo 'Error:' . curl_error($ch);
                        return response()->json(['status' => false, 'message' => curl_error($ch)]);
                    }
                    curl_close($ch);
                }
            }

            $customer = \Stripe\Customer::retrieve($stripe_customer_id);
            $customer->default_source = $card;
            $customer->save();

            PaymentMethod::where('mem_id', getAuthMemberId())->where('type', 'CC')->where('default_option', '1')->update([
                'default_option' => 0
            ]);

            PaymentMethod::where('mem_id', getAuthMemberId())->where('type', 'CC')->where('source', $source)->update([
                'default_option' => 1
            ]);

            return response()->json(['status' => true, 'message' => 'Primary Updated Successfully']);
        } catch (\Exception $e) {
            return response()->json(['status' => false, 'message' => $e->getMessage()]);
        }
    }

    public function removePlaid($bankId)
    {
        PaymentMethod::where('id', $bankId)->delete();
        return redirect()->back()->with('success', 'Plaid Removed Successfully');
    }
}
