<?php

namespace App\Http\Controllers;

use App\Models\Trailer;
use Illuminate\Http\Request;
use App\DataTables\TrailerDataTable;

class TrailerController extends Controller
{
    public function index(TrailerDataTable $dataTable)
    {
        return $dataTable->render('assets.trailers.index');
    }

    public function add()
    {
        return view('assets.trailers.add');
    }

    public function store(Request $request)
    {
        $this->validate($request, [
            'TRAILER_ID' => 'required',
            'USER1' => 'required',
            'PLATE' => 'required',
            'YEAR' => 'required',
            'TRAILER_TYPE' => 'required',
            'LENGTH_1' => 'required',
            'VIN' => 'required',
            'MAXIMUM_WEIGHT' => 'required',
            'AXLES' => 'required',
            'EMPTY_WEIGHT' => 'required',
        ],
        [
            'TRAILER_ID.required' => 'The Trailer Number field is Required',
            'USER1.required' => 'The Make field is Required',
            'PLATE.required' => 'The Trailer Plate field is Required',
            'YEAR.required' => 'The Year field is Required',
            'TRAILER_TYPE.required' => 'The Type field is Required',
            'LENGTH_1.required' => 'The Length field is Required',
            'VIN.required' => 'The VIN # field is Required',
            'MAXIMUM_WEIGHT.required' => 'The Max Weight field is Required',
            'AXLES.required' => 'The Number of Axles field is Required',
            'EMPTY_WEIGHT.required' => 'The Empty Weight field is Required',
        ]);

        Trailer::create([
            'TRAILER_ID' => $request->TRAILER_ID,
            'USER1' => $request->USER1,
            'PLATE' => $request->PLATE,
            'YEAR' => $request->YEAR,
            'TRAILER_TYPE' => $request->TRAILER_TYPE,
            'LENGTH_1' => $request->LENGTH_1,
            'VIN' => $request->VIN,
            'MAXIMUM_WEIGHT' => $request->MAXIMUM_WEIGHT,
            'AXLES' => $request->AXLES,
            'EMPTY_WEIGHT' => $request->EMPTY_WEIGHT,
            'CREATED_DATE' => date('Y-m-d H:i:s'),
            'CREATED_BY' => getAuthMemberId(),
            'mem_id' => getAuthMemberId(),
        ]);

        return redirect()->route('assets.trailers.index')->with('success', 'Trailer Added Successfully');
    }

    public function edit($trailerId)
    {
        $trailer = Trailer::find($trailerId);
        return view('assets.trailers.edit', compact('trailer'));
    }

    public function update(Request $request)
    {
        $this->validate($request, [
            'TRAILER_ID' => 'required',
            'USER1' => 'required',
            'PLATE' => 'required',
            'YEAR' => 'required',
            'TRAILER_TYPE' => 'required',
            'LENGTH_1' => 'required',
            'VIN' => 'required',
            'MAXIMUM_WEIGHT' => 'required',
            'AXLES' => 'required',
            'EMPTY_WEIGHT' => 'required',
        ],
        [
            'TRAILER_ID.required' => 'The Trailer Number field is Required',
            'USER1.required' => 'The Make field is Required',
            'PLATE.required' => 'The Trailer Plate field is Required',
            'YEAR.required' => 'The Year field is Required',
            'TRAILER_TYPE.required' => 'The Type field is Required',
            'LENGTH_1.required' => 'The Length field is Required',
            'VIN.required' => 'The VIN # field is Required',
            'MAXIMUM_WEIGHT.required' => 'The Max Weight field is Required',
            'AXLES.required' => 'The Number of Axles field is Required',
            'EMPTY_WEIGHT.required' => 'The Empty Weight field is Required',
        ]);

        Trailer::where('ID', $request->ID)->update([
            'TRAILER_ID' => $request->TRAILER_ID,
            'USER1' => $request->USER1,
            'PLATE' => $request->PLATE,
            'YEAR' => $request->YEAR,
            'TRAILER_TYPE' => $request->TRAILER_TYPE,
            'LENGTH_1' => $request->LENGTH_1,
            'VIN' => $request->VIN,
            'MAXIMUM_WEIGHT' => $request->MAXIMUM_WEIGHT,
            'AXLES' => $request->AXLES,
            'EMPTY_WEIGHT' => $request->EMPTY_WEIGHT,
        ]);

        return redirect()->route('assets.trailers.index')->with('success', 'Trailer Updated Successfully');
    }

    public function delete($trailerId)
    {
        Trailer::where('ID', $trailerId)->delete();
        return redirect()->back()->with('success', 'Trailer Deleted Successfully.');
    }
}
