<?php

use Illuminate\Support\Facades\Route;
use App\Http\Controllers\Auth\LoginController;
use App\Http\Controllers\DashboardController;
use App\Http\Controllers\TransactionReportController;
use App\Http\Controllers\FuelCardController;
use App\Http\Controllers\RebateReportController;
use App\Http\Controllers\CarrierPricingController;
use App\Http\Controllers\InvoiceController;
use App\Http\Controllers\MyWalletController;
use App\Http\Controllers\DisputesController;
use App\Http\Controllers\CardOrderController;
use App\Http\Controllers\RouteSearchController;
use App\Http\Controllers\PriceListDistanceController;
use App\Http\Controllers\IFTAController;
use App\Http\Controllers\MyProfileController;
use App\Http\Controllers\MonthlyReportsController;
use App\Http\Controllers\IFTAMilesController;
use App\Http\Controllers\GenerateIFTAController;
use App\Http\Controllers\MilesCalculatorController;
use App\Http\Controllers\DriverController;
use App\Http\Controllers\TruckController;
use App\Http\Controllers\TrailerController;
use App\Http\Controllers\CalendarController;
use App\Http\Controllers\CronJobController;

Route::get('/clear-cache', function () {
    Artisan::call('cache:clear');
    Artisan::call('config:clear');
    Artisan::call('config:cache');

    return "Cache cleared successfully";
});

Route::get('/login', [LoginController::class, 'login'])->name('login');
Route::post('/loginCheck', [LoginController::class, 'loginCheck'])->name('loginCheck');
Route::get('/two_fator/{id?}', [LoginController::class, 'two_fator'])->name('two_fator');
Route::post('/two_factor_check', [LoginController::class, 'two_factor_check'])->name('two_factor_check');
Route::get('/resend_login_token/{id?}', [LoginController::class, 'resend_login_token'])->name('resend_login_token');
Route::get('/gallon-cron', [CronJobController::class, 'gallonCron'])->name('gallonCron');
Route::get('/send-gallon-cron/{subfleet?}/{email?}/{mem_id?}/{start_date?}/{end_date?}/{q?}', [CronJobController::class, 'sendGallon'])->name('sendGallon');

Route::middleware(['auth'])->group(function () {
    Route::get('/', [DashboardController::class, 'dashboard'])->name('dashboard');
    Route::post('/addEmailAlert', [DashboardController::class, 'addEmailAlert'])->name('addEmailAlert');
    Route::get('/removeEmailAlert', [DashboardController::class, 'removeEmailAlert'])->name('removeEmailAlert');
    Route::get('/wireInstructions', [DashboardController::class, 'wireInstructions'])->name('wireInstructions');
    Route::get('/getCreditAvailable/{memId?}', [DashboardController::class, 'getCreditAvailable'])->name('getCreditAvailable');
    Route::get('/routeSearch', [RouteSearchController::class, 'index'])->name('routeSearch');
    Route::get('/logout', [LoginController::class, 'logout'])->name('logout');

    Route::prefix('transactionsReport')->group(function () {
        Route::get('/', [TransactionReportController::class, 'index'])->name('transactionsReport.index');
        Route::get('/newIndex', [TransactionReportController::class, 'newIndex'])->name('transactionsReport.newIndex');
        Route::get('/priceListDistance/{StoreID}/{distance}/{DAY}/{retail}/{cid}', [PriceListDistanceController::class, 'index'])->name('transactionsReport.priceListDistance');
    });

    Route::prefix('fuelCards')->group(function () {
        Route::get('/', [FuelCardController::class, 'index'])->name('fuelCards.index');
        Route::get('/newIndex', [FuelCardController::class, 'newIndex'])->name('fuelCards.newIndex');
        Route::get('/edit/{id}', [FuelCardController::class, 'edit'])->name('fuelCards.edit');
        Route::get('/get_efs_data/{id}', [FuelCardController::class, 'get_efs_data'])->name('fuelCards.get_efs_data');
        Route::get('/efs_rejected_transactions/{id}', [FuelCardController::class, 'efs_rejected_transactions'])->name('fuelCards.efs_rejected_transactions');
        Route::get('/efs_transactions', [FuelCardController::class, 'efs_transactions'])->name('fuelCards.efs_transactions');
        Route::post('/update', [FuelCardController::class, 'update'])->name('fuelCards.update');
    });

    Route::prefix('Disputes')->group(function () {
        Route::get('/', [DisputesController::class, 'index'])->name('Disputes.index');
        Route::get('/newIndex', [DisputesController::class, 'newIndex'])->name('Disputes.newIndex');
        Route::get('/edit/{id}', [DisputesController::class, 'edit'])->name('Disputes.edit');
        Route::get('/add', [DisputesController::class, 'create'])->name('Disputes.add');
        Route::get('/view/{id}', [DisputesController::class, 'view'])->name('Disputes.view');
        Route::post('/store', [DisputesController::class, 'store'])->name('Disputes.store');
        Route::post('/update', [DisputesController::class, 'update'])->name('Disputes.update');
        Route::get('/delete/{id}', [DisputesController::class, 'delete'])->name('Disputes.delete');
    });

    Route::prefix('card_orders')->group(function () {
        Route::get('/', [CardOrderController::class, 'index'])->name('card_orders.index');
        Route::get('/add', [CardOrderController::class, 'add'])->name('card_orders.add');
        Route::get('/view/{id}', [CardOrderController::class, 'view'])->name('card_orders.view');
        Route::post('/store', [CardOrderController::class, 'store'])->name('card_orders.store');
    });

    Route::prefix('currentRebateReport')->group(function () {
        Route::get('/', [RebateReportController::class, 'currentReportIndex'])->name('currentRebateReport.index');
        Route::get('/newIndex', [RebateReportController::class, 'currentReportNewIndex'])->name('currentRebateReport.newIndex');
    });

    Route::prefix('carrierPricing')->group(function () {
        Route::get('/', [CarrierPricingController::class, 'index'])->name('carrierPricing.index');
        Route::get('/newIndex', [CarrierPricingController::class, 'newIndex'])->name('carrierPricing.newIndex');
    });

    Route::prefix('invoices')->group(function () {
        Route::get('/', [InvoiceController::class, 'index'])->name('invoices.index');
        Route::get('/newIndex', [InvoiceController::class, 'newIndex'])->name('invoices.newIndex');
        Route::get('/view/{id}', [InvoiceController::class, 'view'])->name('invoices.view');
        Route::get('/download/{id}', [InvoiceController::class, 'download'])->name('invoices.download');
        Route::get('/changeStatus', [InvoiceController::class, 'changeStatus'])->name('invoices.changeStatus');
        Route::get('/payNow/{id}/{amount}/{card}', [InvoiceController::class, 'payNow'])->name('invoices.payNow');
        Route::post('/chargePayment', [InvoiceController::class, 'chargePayment'])->name('invoices.chargePayment');
    });

    Route::prefix('myWallet')->group(function () {
        Route::get('/', [MyWalletController::class, 'index'])->name('myWallet.index');
        Route::get('/newIndex', [MyWalletController::class, 'newIndex'])->name('myWallet.newIndex');
        Route::get('/plaid_success', [MyWalletController::class, 'plaid_success'])->name('myWallet.plaid_success');
        Route::post('/addbankach', [MyWalletController::class, 'addbankach'])->name('myWallet.addbankach');
        Route::get('/fetchPlaidBalance', [MyWalletController::class, 'fetchPlaidBalance'])->name('myWallet.fetchPlaidBalance');
        Route::get('/removePlaid/{bankId}', [MyWalletController::class, 'removePlaid'])->name('myWallet.removePlaid');
        Route::get('/getStripeCards', [MyWalletController::class, 'getStripeCards'])->name('myWallet.getStripeCards');
        Route::post('/addcardsccdc', [MyWalletController::class, 'addcardsccdc'])->name('myWallet.addcardsccdc');
        Route::get('/delete_card/{card}', [MyWalletController::class, 'delete_card'])->name('myWallet.delete_card');
        Route::get('/make_primary/{card}', [MyWalletController::class, 'make_primary'])->name('myWallet.make_primary');
    });

    Route::prefix('ifta')->group(function () {
        Route::get('/', [IFTAController::class, 'index'])->name('ifta.index');
        Route::get('/newIndex', [IFTAController::class, 'newIndex'])->name('ifta.newIndex');

        Route::prefix('miles')->group(function () {
            Route::get('/index', [IFTAMilesController::class, 'index'])->name('ifta.miles.index');
            Route::get('/add', [IFTAMilesController::class, 'add'])->name('ifta.miles.add');
            Route::post('/store', [IFTAMilesController::class, 'store'])->name('ifta.miles.store');
            Route::get('/edit/{milesId?}', [IFTAMilesController::class, 'edit'])->name('ifta.miles.edit');
            Route::post('/update', [IFTAMilesController::class, 'update'])->name('ifta.miles.update');
            Route::get('/delete/{milesId?}', [IFTAMilesController::class, 'delete'])->name('ifta.miles.delete');
            Route::get('/multipleDelete', [IFTAMilesController::class, 'multipleDelete'])->name('ifta.miles.multipleDelete');
            Route::post('/import', [IFTAMilesController::class, 'import'])->name('ifta.miles.import');
        });

        Route::prefix('generateIFTA')->group(function () {
            Route::get('/index', [GenerateIFTAController::class, 'index'])->name('ifta.generateIFTA.index');
            Route::get('/checkReportIndex', [GenerateIFTAController::class, 'checkReportIndex'])->name('ifta.generateIFTA.checkReportIndex');
            Route::get('/checkReport/{year?}/{quarter?}/{carrier?}/{mode?}/{unit?}', [GenerateIFTAController::class, 'checkReport'])->name('ifta.generateIFTA.checkReport');
            Route::get('/downloadPDF/{year?}/{quarter?}/{carrier?}/{mode?}/{unit?}', [GenerateIFTAController::class, 'downloadPDF'])->name('ifta.generateIFTA.downloadPDF');
            Route::get('/downloadCsv/{year?}/{quarter?}/{carrier?}/{mode?}/{unit?}', [GenerateIFTAController::class, 'downloadCsv'])->name('ifta.generateIFTA.downloadCsv');
            Route::get('/saveReport/{year?}/{quarter?}/{carrier?}', [GenerateIFTAController::class, 'saveReport'])->name('ifta.generateIFTA.saveReport');
            Route::get('/delete/{id?}', [GenerateIFTAController::class, 'delete'])->name('ifta.generateIFTA.delete');
        });

        Route::prefix('calculate_miles')->group(function () {
            Route::get('/index/{loadId?}/{memId?}', [MilesCalculatorController::class, 'index'])->name('ifta.calculate_miles.index');
            Route::post('/calculate', [MilesCalculatorController::class, 'calculate'])->name('ifta.calculate_miles.calculate');
            Route::post('/postCalculate', [MilesCalculatorController::class, 'postCalculate'])->name('ifta.calculate_miles.postCalculate');
            Route::post('/import', [MilesCalculatorController::class, 'import'])->name('ifta.calculate_miles.import');
            Route::get('/delete/{id?}', [MilesCalculatorController::class, 'delete'])->name('ifta.calculate_miles.delete');
            Route::get('/upgradePlan', [MilesCalculatorController::class, 'upgradePlan'])->name('ifta.calculate_miles.upgradePlan');
            Route::post('/storePlan', [MilesCalculatorController::class, 'storePlan'])->name('ifta.calculate_miles.storePlan');
        });
    });

    Route::prefix('monthlyReports')->group(function () {
        Route::get('/', [MonthlyReportsController::class, 'index'])->name('monthlyReports.index');
        Route::get('/petroGallonsGallons', [MonthlyReportsController::class, 'PetroGallonsGallons'])->name('monthlyReports.petroGallonsGallons');
        Route::get('/TAPetroGallonsGallons', [MonthlyReportsController::class, 'TAPetroGallonsGallons'])->name('monthlyReports.TAPetroGallonsGallons');
        Route::get('/lovesGallons', [MonthlyReportsController::class, 'lovesGallons'])->name('monthlyReports.lovesGallons');
        Route::get('/outOfNetworkGallons', [MonthlyReportsController::class, 'outOfNetworkGallons'])->name('monthlyReports.outOfNetworkGallons');
        Route::get('/gallonsByState', [MonthlyReportsController::class, 'gallonsByState'])->name('monthlyReports.gallonsByState');
        Route::get('/milesByState', [MonthlyReportsController::class, 'milesByState'])->name('monthlyReports.milesByState');
        Route::get('/gallonsByStore', [MonthlyReportsController::class, 'gallonsByStore'])->name('monthlyReports.gallonsByStore');
    });

    Route::prefix('myProfile')->group(function () {
        Route::get('/', [MyProfileController::class, 'index'])->name('myProfile.index');
        Route::get('/updateLowBalance', [MyProfileController::class, 'updateLowBalance'])->name('myProfile.updateLowBalance');
        Route::get('/update_two_factor_auth', [MyProfileController::class, 'update_two_factor_auth'])->name('myProfile.update_two_factor_auth');
        Route::post('/change_password', [MyProfileController::class, 'change_password'])->name('myProfile.change_password');
    });

    Route::prefix('assets')->group(function () {
        Route::prefix('drivers')->group(function () {
            Route::get('/', [DriverController::class, 'index'])->name('assets.drivers.index');
            Route::get('/add', [DriverController::class, 'add'])->name('assets.drivers.add');
            Route::post('/store', [DriverController::class, 'store'])->name('assets.drivers.store');
            Route::get('/edit/{driverId?}', [DriverController::class, 'edit'])->name('assets.drivers.edit');
            Route::post('/update', [DriverController::class, 'update'])->name('assets.drivers.update');
            Route::get('/delete/{driverId?}', [DriverController::class, 'delete'])->name('assets.drivers.delete');
            Route::post('/import', [DriverController::class, 'import'])->name('assets.drivers.import');
        });

        Route::prefix('trucks')->group(function () {
            Route::get('/', [TruckController::class, 'index'])->name('assets.trucks.index');
            Route::get('/add', [TruckController::class, 'add'])->name('assets.trucks.add');
            Route::post('/store', [TruckController::class, 'store'])->name('assets.trucks.store');
            Route::get('/edit/{truckId?}', [TruckController::class, 'edit'])->name('assets.trucks.edit');
            Route::post('/update', [TruckController::class, 'update'])->name('assets.trucks.update');
            Route::get('/delete/{truckId?}', [TruckController::class, 'delete'])->name('assets.trucks.delete');
            Route::post('/import', [TruckController::class, 'import'])->name('assets.trucks.import');
            Route::post('/update-truck-status', [TruckController::class, 'statusChange'])->name('assets.trucks.status');
        });

        Route::prefix('trailers')->group(function () {
            Route::get('/', [TrailerController::class, 'index'])->name('assets.trailers.index');
            Route::get('/add', [TrailerController::class, 'add'])->name('assets.trailers.add');
            Route::post('/store', [TrailerController::class, 'store'])->name('assets.trailers.store');
            Route::get('/edit/{truckId?}', [TrailerController::class, 'edit'])->name('assets.trailers.edit');
            Route::post('/update', [TrailerController::class, 'update'])->name('assets.trailers.update');
            Route::get('/delete/{truckId?}', [TrailerController::class, 'delete'])->name('assets.trailers.delete');
        });

        Route::prefix('calendar')->group(function () {
            Route::get('/', [CalendarController::class, 'index'])->name('assets.calendar.index');
        });
    });
});
